@@ -190,6 +190,16 @@ QByteArray Utility::userAgentString()
     return re.toLatin1();
 }
 
+bool Utility::hasSystemLaunchOnStartup(const QString &appName)
+{
+#if defined(Q_OS_WIN)
+    return hasSystemLaunchOnStartup_private(appName);
+#else
+    Q_UNUSED(appName)
+    return false;
+#endif
+}
+
 bool Utility::hasLaunchOnStartup(const QString &appName)
 {
     return hasLaunchOnStartup_private(appName);
@@ -254,15 +264,17 @@ void Utility::usleep(int usec)
     QThread::usleep(usec);
 }
 
-bool Utility::fsCasePreserving()
-{
-#ifdef WITH_TESTING
+// This can be overriden from the tests
+OCSYNC_EXPORT bool fsCasePreserving_override = []()-> bool {
     QByteArray env = qgetenv("OWNCLOUD_TEST_CASE_PRESERVING");
     if (!env.isEmpty())
         return env.toInt();
-#endif
+    return Utility::isWindows() || Utility::isMac();
+}();
 
-    return isWindows() || isMac();
+bool Utility::fsCasePreserving()
+{
+    return fsCasePreserving_override;
 }
 
 bool Utility::fileNamesEqual(const QString &fn1, const QString &fn2)
@@ -296,9 +308,7 @@ namespace {
 
         QString description(quint64 value) const
         {
-            return QCoreApplication::translate(
-                "Utility", name, 0, QCoreApplication::UnicodeUTF8,
-                value);
+            return QCoreApplication::translate("Utility", name, 0, value);
         }
     };
 // QTBUG-3945 and issue #4855: QT_TRANSLATE_NOOP does not work with plural form because lupdate
@@ -520,11 +530,11 @@ void Utility::sortFilenames(QStringList &fileNames)
     QCollator collator;
     collator.setNumericMode(true);
     collator.setCaseSensitivity(Qt::CaseInsensitive);
-    qSort(fileNames.begin(), fileNames.end(), collator);
+    std::sort(fileNames.begin(), fileNames.end(), collator);
 }
 
 QUrl Utility::concatUrlPath(const QUrl &url, const QString &concatPath,
-    const QList<QPair<QString, QString>> &queryItems)
+    const QUrlQuery &queryItems)
 {
     QString path = url.path();
     if (!concatPath.isEmpty()) {
@@ -540,12 +550,36 @@ QUrl Utility::concatUrlPath(const QUrl &url, const QString &concatPath,
 
     QUrl tmpUrl = url;
     tmpUrl.setPath(path);
-    if (queryItems.size() > 0) {
-        tmpUrl.setQueryItems(queryItems);
-    }
+    tmpUrl.setQuery(queryItems);
     return tmpUrl;
 }
 
+QString Utility::makeConflictFileName(
+    const QString &fn, const QDateTime &dt, const QString &user)
+{
+    QString conflictFileName(fn);
+    // Add conflict tag before the extension.
+    int dotLocation = conflictFileName.lastIndexOf('.');
+    // If no extension, add it at the end  (take care of cases like foo/.hidden or foo.bar/file)
+    if (dotLocation <= conflictFileName.lastIndexOf('/') + 1) {
+        dotLocation = conflictFileName.size();
+    }
+
+    QString conflictMarker = QStringLiteral(" (conflicted copy ");
+    if (!user.isEmpty()) {
+        // Don't allow parens in the user name, to ensure
+        // we can find the beginning and end of the conflict tag.
+        const auto userName = sanitizeForFileName(user).replace('(', '_').replace(')', '_');
+        conflictMarker.append(userName);
+        conflictMarker.append(' ');
+    }
+    conflictMarker.append(dt.toString("yyyy-MM-dd hhmmss"));
+    conflictMarker.append(')');
+
+    conflictFileName.insert(dotLocation, conflictMarker);
+    return conflictFileName;
+}
+
 bool Utility::isConflictFile(const char *name)
 {
     const char *bname = std::strrchr(name, '/');
@@ -555,32 +589,73 @@ bool Utility::isConflictFile(const char *name)
         bname = name;
     }
 
+    // Old pattern
     if (std::strstr(bname, "_conflict-"))
         return true;
 
-    if (shouldUploadConflictFiles()) {
-        // For uploads, we want to consider files with any kind of username tag
-        // as conflict files. (pattern *_conflict_*-)
-        const char *startOfMarker = std::strstr(bname, "_conflict_");
-        if (startOfMarker && std::strchr(startOfMarker, '-'))
-            return true;
-    } else {
-        // Old behavior: optionally, files with the specific string in the env variable
-        // appended are also considered conflict files.
-        static auto conflictFileUsername = qgetenv("CSYNC_CONFLICT_FILE_USERNAME");
-        static auto usernameConflictId = QByteArray("_conflict_" + conflictFileUsername + "-");
-        if (!conflictFileUsername.isEmpty() && std::strstr(bname, usernameConflictId.constData())) {
-            return true;
-        }
-    }
+    // New pattern
+    if (std::strstr(bname, "(conflicted copy"))
+        return true;
 
     return false;
 }
 
-bool Utility::shouldUploadConflictFiles()
+bool Utility::isConflictFile(const QString &name)
 {
-    static bool uploadConflictFiles = qEnvironmentVariableIntValue("OWNCLOUD_UPLOAD_CONFLICT_FILES") != 0;
-    return uploadConflictFiles;
+    auto bname = name.midRef(name.lastIndexOf('/') + 1);
+
+    if (bname.contains(QStringLiteral("_conflict-")))
+        return true;
+
+    if (bname.contains(QStringLiteral("(conflicted copy")))
+        return true;
+
+    return false;
+}
+
+QByteArray Utility::conflictFileBaseName(const QByteArray &conflictName)
+{
+    // This function must be able to deal with conflict files for conflict files.
+    // To do this, we scan backwards, for the outermost conflict marker and
+    // strip only that to generate the conflict file base name.
+    auto startOld = conflictName.lastIndexOf("_conflict-");
+
+    // A single space before "(conflicted copy" is considered part of the tag
+    auto startNew = conflictName.lastIndexOf("(conflicted copy");
+    if (startNew > 0 && conflictName[startNew - 1] == ' ')
+        startNew -= 1;
+
+    // The rightmost tag is relevant
+    auto tagStart = qMax(startOld, startNew);
+    if (tagStart == -1)
+        return "";
+
+    // Find the end of the tag
+    auto tagEnd = conflictName.size();
+    auto dot = conflictName.lastIndexOf('.'); // dot could be part of user name for new tag!
+    if (dot > tagStart)
+        tagEnd = dot;
+    if (tagStart == startNew) {
+        auto paren = conflictName.indexOf(')', tagStart);
+        if (paren != -1)
+            tagEnd = paren + 1;
+    }
+    return conflictName.left(tagStart) + conflictName.mid(tagEnd);
+}
+
+QString Utility::sanitizeForFileName(const QString &name)
+{
+    const auto invalid = QStringLiteral("/?<>\\:*|\"");
+    QString result;
+    result.reserve(name.size());
+    for (const auto c : name) {
+        if (!invalid.contains(c)
+            && c.category() != QChar::Other_Control
+            && c.category() != QChar::Other_Format) {
+            result.append(c);
+        }
+    }
+    return result;
 }
 
 } // namespace OCC
