@@ -27,6 +27,7 @@
 #include "csync_rename.h"
 #include "common/c_jhash.h"
 #include "common/asserts.h"
+#include "common/syncjournalfilerecord.h"
 
 #include <QLoggingCategory>
 Q_LOGGING_CATEGORY(lcReconcile, "sync.csync.reconciler", QtInfoMsg)
@@ -91,7 +92,7 @@ static csync_file_stat_t *_csync_check_ignored(csync_s::FileMap *tree, const Byt
  * (timestamp is newer), it is not overwritten. If both files, on the
  * source and the destination, have been changed, the newer file wins.
  */
-static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
+static void _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
     csync_s::FileMap *our_tree = nullptr;
     csync_s::FileMap *other_tree = nullptr;
 
@@ -109,7 +110,7 @@ static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
         break;
     }
 
-    csync_file_stat_t *other = other_tree->findFile(cur->path);;
+    csync_file_stat_t *other = other_tree->findFile(cur->path);
 
     if (!other) {
         /* Check the renamed path as well. */
@@ -121,6 +122,31 @@ static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
         /* If it is ignored, other->instruction will be  IGNORE so this one will also be ignored */
     }
 
+    // If the user adds a file locally check whether a virtual file for that name exists.
+    // If so, go to "potential conflict" mode by switching the remote entry to be a
+    // real file.
+    if (!other
+        && ctx->current == LOCAL_REPLICA
+        && cur->instruction == CSYNC_INSTRUCTION_NEW
+        && cur->type != ItemTypeVirtualFile) {
+        // Check if we have a virtual file  entry in the remote tree
+        auto virtualFilePath = cur->path;
+        virtualFilePath.append(ctx->virtual_file_suffix);
+        other = other_tree->findFile(virtualFilePath);
+        if (!other) {
+            /* Check the renamed path as well. */
+            other = other_tree->findFile(csync_rename_adjust_parent_path(ctx, virtualFilePath));
+        }
+        if (other && other->type == ItemTypeVirtualFile) {
+            qCInfo(lcReconcile) << "Found virtual file for local" << cur->path << "in remote tree";
+            other->path = cur->path;
+            other->type = ItemTypeVirtualFileDownload;
+            other->instruction = CSYNC_INSTRUCTION_EVAL;
+        } else {
+            other = nullptr;
+        }
+    }
+
     /* file only found on current replica */
     if (!other) {
         switch(cur->instruction) {
@@ -140,6 +166,34 @@ static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
                 cur->instruction = CSYNC_INSTRUCTION_NEW;
                 break;
             }
+            /* If the local virtual file is gone, it should be reestablished.
+             * Unless the base file is seen in the local tree now. */
+            if (cur->type == ItemTypeVirtualFile
+                && ctx->current == REMOTE_REPLICA
+                && cur->path.endsWith(ctx->virtual_file_suffix)
+                && !other_tree->findFile(cur->path.left(cur->path.size() - ctx->virtual_file_suffix.size()))) {
+                cur->instruction = CSYNC_INSTRUCTION_NEW;
+                break;
+            }
+
+            /* If a virtual file is supposed to be downloaded, the local tree
+             * will see "foo.owncloud" NONE while the remote might see "foo".
+             * In the common case of remote NEW we don't want to trigger the REMOVE
+             * that would normally be done for foo.owncloud since the download for
+             * "foo" will take care of it.
+             * If it was removed remotely, or moved remotely, the REMOVE is what we want.
+             */
+            if (cur->type == ItemTypeVirtualFileDownload
+                && ctx->current == LOCAL_REPLICA
+                && cur->path.endsWith(ctx->virtual_file_suffix)) {
+                auto actualOther = other_tree->findFile(cur->path.left(cur->path.size() - ctx->virtual_file_suffix.size()));
+                if (actualOther
+                    && (actualOther->instruction == CSYNC_INSTRUCTION_NEW
+                        || actualOther->instruction == CSYNC_INSTRUCTION_CONFLICT)) {
+                    cur->instruction = CSYNC_INSTRUCTION_NONE;
+                    break;
+                }
+            }
             cur->instruction = CSYNC_INSTRUCTION_REMOVE;
             break;
         case CSYNC_INSTRUCTION_EVAL_RENAME: {
@@ -156,7 +210,7 @@ static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
                 /* First, check that the file is NOT in our tree (another file with the same name was added) */
                 if (our_tree->findFile(basePath)) {
                     other = nullptr;
-                    qCDebug(lcReconcile, "Origin found in our tree : %s", basePath.constData());
+                    qCInfo(lcReconcile, "Origin found in our tree : %s", basePath.constData());
                 } else {
                     /* Find the potential rename source file in the other tree.
                     * If the renamed file could not be found in the opposite tree, that is because it
@@ -164,7 +218,7 @@ static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
                     * The journal is cleaned up later after propagation.
                     */
                     other = other_tree->findFile(basePath);
-                    qCDebug(lcReconcile, "Rename origin in other tree (%s) %s",
+                    qCInfo(lcReconcile, "Rename origin in other tree (%s) %s",
                         basePath.constData(), other ? "found" : "not found");
                 }
 
@@ -175,9 +229,9 @@ static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
                     // Some other EVAL_RENAME already claimed other.
                     // We do nothing: maybe a different candidate for
                     // other is found as well?
-                    qCDebug(lcReconcile, "Other has already been renamed to %s",
+                    qCInfo(lcReconcile, "Other has already been renamed to %s",
                         other->rename_path.constData());
-                } else if (cur->type == CSYNC_FTW_TYPE_DIR
+                } else if (cur->type == ItemTypeDirectory
                     // The local replica is reconciled first, so the remote tree would
                     // have either NONE or UPDATE_METADATA if the remote file is safe to
                     // move.
@@ -187,13 +241,17 @@ static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
                     || other->instruction == CSYNC_INSTRUCTION_NONE
                     || other->instruction == CSYNC_INSTRUCTION_UPDATE_METADATA
                     || other->instruction == CSYNC_INSTRUCTION_REMOVE) {
-                    qCDebug(lcReconcile, "Switching %s to RENAME to %s",
+                    qCInfo(lcReconcile, "Switching %s to RENAME to %s",
                         other->path.constData(), cur->path.constData());
                     other->instruction = CSYNC_INSTRUCTION_RENAME;
                     other->rename_path = cur->path;
                     if( !cur->file_id.isEmpty() ) {
                         other->file_id = cur->file_id;
                     }
+                    if (ctx->current == LOCAL_REPLICA) {
+                        // Keep the local mtime.
+                        other->modtime = cur->modtime;
+                    }
                     other->inode = cur->inode;
                     cur->instruction = CSYNC_INSTRUCTION_NONE;
                     // We have consumed 'other': exit this loop to not consume another one.
@@ -207,7 +265,7 @@ static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
                     // Local: The remote reconcile will be able to deal with this.
                     // Remote: The local replica has already dealt with this.
                     //         See the EVAL_RENAME case when other was found directly.
-                    qCDebug(lcReconcile, "File in a renamed directory, other side's instruction: %d",
+                    qCInfo(lcReconcile, "File in a renamed directory, other side's instruction: %d",
                         other->instruction);
                     cur->instruction = CSYNC_INSTRUCTION_NONE;
                 } else {
@@ -215,7 +273,7 @@ static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
                     // and the instruction in the local tree is NEW while cur has EVAL_RENAME
                     // due to a remote move of the same file. In these scenarios we just
                     // want the instruction to stay NEW.
-                    qCDebug(lcReconcile, "Other already has instruction %d",
+                    qCInfo(lcReconcile, "Other already has instruction %d",
                         other->instruction);
                 }
             };
@@ -223,7 +281,7 @@ static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
             if (ctx->current == LOCAL_REPLICA) {
                 /* use the old name to find the "other" node */
                 OCC::SyncJournalFileRecord base;
-                qCDebug(lcReconcile, "Finding rename origin through inode %" PRIu64 "",
+                qCInfo(lcReconcile, "Finding rename origin through inode %" PRIu64 "",
                     cur->inode);
                 ctx->statedb->getFileRecordByInode(cur->inode, &base);
                 renameCandidateProcessing(base._path);
@@ -236,7 +294,7 @@ static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
                 // line.
                 auto basePath = csync_rename_adjust_full_path_source(ctx, cur->path);
                 if (basePath != cur->path) {
-                    qCDebug(lcReconcile, "Trying rename origin by csync_rename mapping %s",
+                    qCInfo(lcReconcile, "Trying rename origin by csync_rename mapping %s",
                         basePath.constData());
                     // We go through getFileRecordsByFileId to ensure the basePath
                     // computed in this way also has the expected fileid.
@@ -249,7 +307,7 @@ static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
 
                 // Also feed all the other files with the same fileid if necessary
                 if (!processedRename) {
-                    qCDebug(lcReconcile, "Finding rename origin through file ID %s",
+                    qCInfo(lcReconcile, "Finding rename origin through file ID %s",
                         cur->file_id.constData());
                     ctx->statedb->getFileRecordsByFileId(cur->file_id,
                         [&](const OCC::SyncJournalFileRecord &base) { renameCandidateProcessing(base._path); });
@@ -278,26 +336,16 @@ static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
             /* If the file already exist on the other side, we have a conflict.
                Abort the rename and consider it is a new file. */
             cur->instruction = CSYNC_INSTRUCTION_NEW;
-            /* fall trough */
+            /* fall through */
         /* file on current replica is changed or new */
         case CSYNC_INSTRUCTION_EVAL:
         case CSYNC_INSTRUCTION_NEW:
-            // This operation is usually a no-op and will by default return false
-            if (csync_file_locked_or_open(ctx->local.uri, cur->path)) {
-                qCDebug(lcReconcile, "[Reconciler] IGNORING file %s/%s since it is locked / open", ctx->local.uri, cur->path.constData());
-                cur->instruction = CSYNC_INSTRUCTION_ERROR;
-                if (cur->error_status == CSYNC_STATUS_OK) // don't overwrite error
-                    cur->error_status = CYSNC_STATUS_FILE_LOCKED_OR_OPEN;
-                break;
-            } else {
-                //qCDebug(lcReconcile, "[Reconciler] not ignoring file %s/%s", ctx->local.uri, cur->path);
-            }
             switch (other->instruction) {
             /* file on other replica is changed or new */
             case CSYNC_INSTRUCTION_NEW:
             case CSYNC_INSTRUCTION_EVAL:
-                if (other->type == CSYNC_FTW_TYPE_DIR &&
-                        cur->type == CSYNC_FTW_TYPE_DIR) {
+                if (other->type == ItemTypeDirectory &&
+                        cur->type == ItemTypeDirectory) {
                     // Folders of the same path are always considered equals
                     is_conflict = false;
                 } else {
@@ -316,6 +364,36 @@ static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
                         (ctx->current == REMOTE_REPLICA ? cur->checksumHeader : other->checksumHeader);
                     if (!remoteChecksumHeader.isEmpty()) {
                         is_conflict = true;
+
+                        // Do we have an UploadInfo for this?
+                        // Maybe the Upload was completed, but the connection was broken just before
+                        // we recieved the etag (Issue #5106)
+                        auto up = ctx->statedb->getUploadInfo(cur->path);
+                        if (up._valid && up._contentChecksum == remoteChecksumHeader) {
+                            // Solve the conflict into an upload, or nothing
+                            auto remoteNode = ctx->current == REMOTE_REPLICA ? cur : other;
+                            auto localNode = ctx->current == REMOTE_REPLICA ? other : cur;
+                            remoteNode->instruction = CSYNC_INSTRUCTION_NONE;
+                            localNode->instruction = up._modtime == localNode->modtime && up._size == localNode->size ?
+                                CSYNC_INSTRUCTION_UPDATE_METADATA : CSYNC_INSTRUCTION_SYNC;
+
+                            // Update the etag and other server metadata in the journal already
+                            // (We can't use a typical CSYNC_INSTRUCTION_UPDATE_METADATA because
+                            // we must not store the size/modtime from the file system)
+                            OCC::SyncJournalFileRecord rec;
+                            if (ctx->statedb->getFileRecord(remoteNode->path, &rec)) {
+                                rec._path = remoteNode->path;
+                                rec._etag = remoteNode->etag;
+                                rec._fileId = remoteNode->file_id;
+                                rec._modtime = remoteNode->modtime;
+                                rec._type = remoteNode->type;
+                                rec._fileSize = remoteNode->size;
+                                rec._remotePerm = remoteNode->remotePerm;
+                                rec._checksumHeader = remoteNode->checksumHeader;
+                                ctx->statedb->setFileRecordMetadata(rec);
+                            }
+                            break;
+                        }
                     }
 
                     // SO: If there is no checksum, we can have !is_conflict here
@@ -345,11 +423,14 @@ static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
                     // needs to delete the other entity first.
                     cur->instruction = CSYNC_INSTRUCTION_TYPE_CHANGE;
                     other->instruction = CSYNC_INSTRUCTION_NONE;
-                } else if (cur->type == CSYNC_FTW_TYPE_DIR) {
+                } else if (cur->type == ItemTypeDirectory) {
                     cur->instruction = CSYNC_INSTRUCTION_UPDATE_METADATA;
                     other->instruction = CSYNC_INSTRUCTION_NONE;
                 } else {
-                    cur->instruction = CSYNC_INSTRUCTION_SYNC;
+                    if (cur->instruction != CSYNC_INSTRUCTION_NEW
+                        && cur->instruction != CSYNC_INSTRUCTION_SYNC) {
+                        cur->instruction = CSYNC_INSTRUCTION_SYNC;
+                    }
                     other->instruction = CSYNC_INSTRUCTION_NONE;
                 }
                 break;
@@ -368,17 +449,38 @@ static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
             if (cur->instruction == CSYNC_INSTRUCTION_EVAL)
                 cur->instruction = CSYNC_INSTRUCTION_NEW;
             break;
+        case CSYNC_INSTRUCTION_NONE:
+            // NONE/NONE on virtual files might become a REMOVE if the base file
+            // is found in the local tree.
+            if (cur->type == ItemTypeVirtualFile
+                && other->instruction == CSYNC_INSTRUCTION_NONE
+                && ctx->current == LOCAL_REPLICA
+                && cur->path.endsWith(ctx->virtual_file_suffix)
+                && ctx->local.files.findFile(cur->path.left(cur->path.size() - ctx->virtual_file_suffix.size()))) {
+                cur->instruction = CSYNC_INSTRUCTION_REMOVE;
+            }
+            break;
         default:
             break;
         }
     }
 
+    // We *never* propagate a local NEW virtual file to the server
+    // This can happen if we had EVAL_RENAME but something else happened on the remote
+    // and the rename doesn't go through.
+    if ((cur->type == ItemTypeVirtualFile || cur->type == ItemTypeVirtualFileDownload)
+        && ctx->current == LOCAL_REPLICA
+        && cur->instruction == CSYNC_INSTRUCTION_NEW) {
+        qCInfo(lcReconcile) << "Downgrading NEW to IGNORE for local virtual file" << cur->path;
+        cur->instruction = CSYNC_INSTRUCTION_IGNORE;
+    }
+
     //hide instruction NONE messages when log level is set to debug,
     //only show these messages on log level trace
     const char *repo = ctx->current == REMOTE_REPLICA ? "server" : "client";
     if(cur->instruction ==CSYNC_INSTRUCTION_NONE)
     {
-        if(cur->type == CSYNC_FTW_TYPE_DIR)
+        if(cur->type == ItemTypeDirectory)
         {
             qCDebug(lcReconcile,
                       "%-30s %s dir:  %s",
@@ -397,7 +499,7 @@ static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
     }
     else
     {
-        if(cur->type == CSYNC_FTW_TYPE_DIR)
+        if(cur->type == ItemTypeDirectory)
         {
             qCInfo(lcReconcile,
                       "%-30s %s dir:  %s",
@@ -414,11 +516,9 @@ static int _csync_merge_algorithm_visitor(csync_file_stat_t *cur, CSYNC * ctx) {
                       cur->path.constData());
         }
     }
-
-    return 0;
 }
 
-int csync_reconcile_updates(CSYNC *ctx) {
+void csync_reconcile_updates(CSYNC *ctx) {
   csync_s::FileMap *tree = nullptr;
 
   switch (ctx->current) {
@@ -433,12 +533,8 @@ int csync_reconcile_updates(CSYNC *ctx) {
   }
 
   for (auto &pair : *tree) {
-    if (_csync_merge_algorithm_visitor(pair.second.get(), ctx) < 0) {
-      ctx->status_code = CSYNC_STATUS_RECONCILE_ERROR;
-      return -1;
-    }
+    _csync_merge_algorithm_visitor(pair.second.get(), ctx);
   }
-  return 0;
 }
 
 /* vim: set ts=8 sw=2 et cindent: */
