@@ -38,6 +38,7 @@
 #include "csync_update.h"
 #include "csync_util.h"
 #include "csync_misc.h"
+#include "common/utility.h"
 
 #include "vio/csync_vio.h"
 
@@ -47,6 +48,7 @@
 #include "common/asserts.h"
 
 #include <QtCore/QTextCodec>
+#include <QtCore/QFile>
 
 // Needed for PRIu64 on MinGW in C++ mode.
 #define __STDC_FORMAT_MACROS
@@ -108,20 +110,15 @@ static bool _csync_mtime_equal(time_t a, time_t b)
  * See doc/dev/sync-algorithm.md for an overview.
  */
 static int _csync_detect_update(CSYNC *ctx, std::unique_ptr<csync_file_stat_t> fs) {
+  Q_ASSERT(fs);
   OCC::SyncJournalFileRecord base;
-  CSYNC_EXCLUDE_TYPE excluded;
-
-  if (fs == NULL) {
-    errno = EINVAL;
-    ctx->status_code = CSYNC_STATUS_PARAM_ERROR;
-    return -1;
-  }
-
-  if (fs->type == CSYNC_FTW_TYPE_SKIP) {
+  CSYNC_EXCLUDE_TYPE excluded = CSYNC_NOT_EXCLUDED;
+  if (fs->type == ItemTypeSkip) {
       excluded =CSYNC_FILE_EXCLUDE_STAT_FAILED;
   } else {
-    /* Check if file is excluded */
-    excluded = csync_excluded_traversal(ctx, fs->path, fs->type);
+      /* Check if file is excluded */
+      if (ctx->exclude_traversal_fn)
+          excluded = ctx->exclude_traversal_fn(fs->path, fs->type);
   }
 
   if( excluded == CSYNC_NOT_EXCLUDED ) {
@@ -130,12 +127,12 @@ static int _csync_detect_update(CSYNC *ctx, std::unique_ptr<csync_file_stat_t> f
        * This code should probably be in csync_exclude, but it does not have the fs parameter.
        * Keep it here for now */
       if (ctx->ignore_hidden_files && (fs->is_hidden)) {
-          qCDebug(lcUpdate, "file excluded because it is a hidden file: %s", fs->path.constData());
+          qCInfo(lcUpdate, "file excluded because it is a hidden file: %s", fs->path.constData());
           excluded = CSYNC_FILE_EXCLUDE_HIDDEN;
       }
   } else {
       /* File is ignored because it's matched by a user- or system exclude pattern. */
-      qCDebug(lcUpdate, "%s excluded  (%d)", fs->path.constData(), excluded);
+      qCInfo(lcUpdate, "%s excluded  (%d)", fs->path.constData(), excluded);
       if (excluded == CSYNC_FILE_EXCLUDE_AND_REMOVE) {
           return 1;
       }
@@ -151,28 +148,29 @@ static int _csync_detect_update(CSYNC *ctx, std::unique_ptr<csync_file_stat_t> f
   }
 
   auto localCodec = QTextCodec::codecForLocale();
-  if (ctx->current == REMOTE_REPLICA && localCodec->mibEnum() != 106) {
+  if (!OCC::Utility::isWindows() && ctx->current == REMOTE_REPLICA && localCodec->mibEnum() != 106) {
       /* If the locale codec is not UTF-8, we must check that the filename from the server can
        * be encoded in the local file system.
+       * (Note: on windows, the FS is always UTF-16, so we don't need to check)
        *
        * We cannot use QTextCodec::canEncode() since that can incorrectly return true, see
        * https://bugreports.qt.io/browse/QTBUG-6925.
        */
       QTextEncoder encoder(localCodec, QTextCodec::ConvertInvalidToNull);
       if (encoder.fromUnicode(QString::fromUtf8(fs->path)).contains('\0')) {
-          qCDebug(lcUpdate, "cannot encode %s to local encoding %d",
+          qCInfo(lcUpdate, "cannot encode %s to local encoding %d",
               fs->path.constData(), localCodec->mibEnum());
           excluded = CSYNC_FILE_EXCLUDE_CANNOT_ENCODE;
       }
   }
 
-  if (fs->type == CSYNC_FTW_TYPE_FILE ) {
+  if (fs->type == ItemTypeFile ) {
     if (fs->modtime == 0) {
-      qCDebug(lcUpdate, "file: %s - mtime is zero!", fs->path.constData());
+      qCInfo(lcUpdate, "file: %s - mtime is zero!", fs->path.constData());
     }
   }
 
-  if (excluded > CSYNC_NOT_EXCLUDED || fs->type == CSYNC_FTW_TYPE_SLINK) {
+  if (excluded > CSYNC_NOT_EXCLUDED || fs->type == ItemTypeSoftLink) {
       fs->instruction = CSYNC_INSTRUCTION_IGNORE;
       if (ctx->current_fs) {
           ctx->current_fs->has_ignored_files = true;
@@ -191,19 +189,64 @@ static int _csync_detect_update(CSYNC *ctx, std::unique_ptr<csync_file_stat_t> f
       return -1;
   }
 
+  // The db entry might be for a virtual file, so look for that on the
+  // remote side. If we find one, change the current fs to look like a
+  // virtual file too, because that's what one would see if the remote
+  // db was filled from the database.
+  if (ctx->current == REMOTE_REPLICA && !base.isValid() && fs->type == ItemTypeFile) {
+      auto virtualFilePath = fs->path;
+      virtualFilePath.append(ctx->virtual_file_suffix);
+      ctx->statedb->getFileRecord(virtualFilePath, &base);
+      if (base.isValid() && base._type == ItemTypeVirtualFile) {
+          fs->type = ItemTypeVirtualFile;
+          fs->path = virtualFilePath;
+      } else {
+          base = OCC::SyncJournalFileRecord();
+      }
+  }
+
   if(base.isValid()) { /* there is an entry in the database */
       /* we have an update! */
       qCInfo(lcUpdate, "Database entry found, compare: %" PRId64 " <-> %" PRId64
                                           ", etag: %s <-> %s, inode: %" PRId64 " <-> %" PRId64
-                                          ", size: %" PRId64 " <-> %" PRId64 ", perms: %x <-> %x, ignore: %d",
+                                          ", size: %" PRId64 " <-> %" PRId64
+                                          ", perms: %x <-> %x"
+                                          ", type: %d <-> %d"
+                                          ", checksum: %s <-> %s"
+                                          ", ignore: %d",
                 ((int64_t) fs->modtime), ((int64_t) base._modtime),
                 fs->etag.constData(), base._etag.constData(), (uint64_t) fs->inode, (uint64_t) base._inode,
-                (uint64_t) fs->size, (uint64_t) base._fileSize, *reinterpret_cast<short*>(&fs->remotePerm), *reinterpret_cast<short*>(&base._remotePerm), base._serverHasIgnoredFiles );
+                (uint64_t) fs->size, (uint64_t) base._fileSize,
+                *reinterpret_cast<short*>(&fs->remotePerm), *reinterpret_cast<short*>(&base._remotePerm),
+                fs->type, base._type,
+                fs->checksumHeader.constData(), base._checksumHeader.constData(),
+                base._serverHasIgnoredFiles );
+
+      // If the db suggests a virtual file should be downloaded,
+      // treat the file as new on the remote.
+      if (ctx->current == REMOTE_REPLICA && base._type == ItemTypeVirtualFileDownload) {
+          fs->instruction = CSYNC_INSTRUCTION_NEW;
+          fs->type = ItemTypeVirtualFileDownload;
+          goto out;
+      }
+
+      // If what the db thinks is a virtual file is actually a file/dir,
+      // treat it as new locally.
+      if (ctx->current == LOCAL_REPLICA
+          && (base._type == ItemTypeVirtualFile || base._type == ItemTypeVirtualFileDownload)
+          && fs->type != ItemTypeVirtualFile) {
+          fs->instruction = CSYNC_INSTRUCTION_EVAL;
+          goto out;
+      }
+
       if (ctx->current == REMOTE_REPLICA && fs->etag != base._etag) {
           fs->instruction = CSYNC_INSTRUCTION_EVAL;
 
-          // Preserve the EVAL flag later on if the type has changed.
-          if (base._type != fs->type) {
+          if (fs->type == ItemTypeVirtualFile) {
+              // If the local thing is a virtual file, we just update the metadata
+              fs->instruction = CSYNC_INSTRUCTION_UPDATE_METADATA;
+          } else if (base._type != fs->type) {
+              // Preserve the EVAL flag later on if the type has changed.
               fs->child_modified = true;
           }
 
@@ -228,7 +271,7 @@ static int _csync_detect_update(CSYNC *ctx, std::unique_ptr<csync_file_stat_t> f
                   checksumIdentical = fs->checksumHeader == base._checksumHeader;
               }
               if (checksumIdentical) {
-                  qCDebug(lcUpdate, "NOTE: Checksums are identical, file did not actually change: %s", fs->path.constData());
+                  qCInfo(lcUpdate, "NOTE: Checksums are identical, file did not actually change: %s", fs->path.constData());
                   fs->instruction = CSYNC_INSTRUCTION_UPDATE_METADATA;
                   goto out;
               }
@@ -245,14 +288,14 @@ static int _csync_detect_update(CSYNC *ctx, std::unique_ptr<csync_file_stat_t> f
       bool metadata_differ = (ctx->current == REMOTE_REPLICA && (fs->file_id != base._fileId
                                                           || fs->remotePerm != base._remotePerm))
                            || (ctx->current == LOCAL_REPLICA && fs->inode != base._inode);
-      if (fs->type == CSYNC_FTW_TYPE_DIR && ctx->current == REMOTE_REPLICA
+      if (fs->type == ItemTypeDirectory && ctx->current == REMOTE_REPLICA
               && !metadata_differ && ctx->read_remote_from_db) {
           /* If both etag and file id are equal for a directory, read all contents from
            * the database.
            * The metadata comparison ensure that we fetch all the file id or permission when
            * upgrading owncloud
            */
-          qCDebug(lcUpdate, "Reading from database: %s", fs->path.constData());
+          qCInfo(lcUpdate, "Reading from database: %s", fs->path.constData());
           ctx->remote.read_from_db = true;
       }
       /* If it was remembered in the db that the remote dir has ignored files, store
@@ -263,7 +306,7 @@ static int _csync_detect_update(CSYNC *ctx, std::unique_ptr<csync_file_stat_t> f
       }
       if (metadata_differ) {
           /* file id or permissions has changed. Which means we need to update them in the DB. */
-          qCDebug(lcUpdate, "Need to update metadata for: %s", fs->path.constData());
+          qCInfo(lcUpdate, "Need to update metadata for: %s", fs->path.constData());
           fs->instruction = CSYNC_INSTRUCTION_UPDATE_METADATA;
       } else {
           fs->instruction = CSYNC_INSTRUCTION_NONE;
@@ -271,7 +314,7 @@ static int _csync_detect_update(CSYNC *ctx, std::unique_ptr<csync_file_stat_t> f
   } else {
       /* check if it's a file and has been renamed */
       if (ctx->current == LOCAL_REPLICA) {
-          qCDebug(lcUpdate, "Checking for rename based on inode # %" PRId64 "", (uint64_t) fs->inode);
+          qCInfo(lcUpdate, "Checking for rename based on inode # %" PRId64 "", (uint64_t) fs->inode);
 
           OCC::SyncJournalFileRecord base;
           if(!ctx->statedb->getFileRecordByInode(fs->inode, &base)) {
@@ -283,37 +326,89 @@ static int _csync_detect_update(CSYNC *ctx, std::unique_ptr<csync_file_stat_t> f
           fs->instruction = CSYNC_INSTRUCTION_NEW;
 
           bool isRename =
-              base.isValid() && base._type == fs->type
-                  && ((base._modtime == fs->modtime && base._fileSize == fs->size) || fs->type == CSYNC_FTW_TYPE_DIR)
+              base.isValid()
+              && (base._type == fs->type
+                     // The file could be scheduled for download and renamed
+                     || (base._type == ItemTypeVirtualFileDownload && fs->type == ItemTypeVirtualFile))
+              && ((base._modtime == fs->modtime && base._fileSize == fs->size)
+                     // Directories and virtual files don't need size/mtime equality
+                     || fs->type == ItemTypeDirectory || fs->type == ItemTypeVirtualFile)
 #ifdef NO_RENAME_EXTENSION
-                  && _csync_sameextension(base._path, fs->path)
+              && _csync_sameextension(base._path, fs->path)
 #endif
               ;
 
 
           // Verify the checksum where possible
           if (isRename && !base._checksumHeader.isEmpty() && ctx->callbacks.checksum_hook
-              && fs->type == CSYNC_FTW_TYPE_FILE) {
+              && fs->type == ItemTypeFile) {
                   fs->checksumHeader = ctx->callbacks.checksum_hook(
                       _rel_to_abs(ctx, fs->path), base._checksumHeader,
                       ctx->callbacks.checksum_userdata);
               if (!fs->checksumHeader.isEmpty()) {
-                  qCDebug(lcUpdate, "checking checksum of potential rename %s %s <-> %s", fs->path.constData(), fs->checksumHeader.constData(), base._checksumHeader.constData());
+                  qCInfo(lcUpdate, "checking checksum of potential rename %s %s <-> %s", fs->path.constData(), fs->checksumHeader.constData(), base._checksumHeader.constData());
                   isRename = fs->checksumHeader == base._checksumHeader;
               }
           }
 
           if (isRename) {
-              qCDebug(lcUpdate, "pot rename detected based on inode # %" PRId64 "", (uint64_t) fs->inode);
+              qCInfo(lcUpdate, "pot rename detected based on inode # %" PRId64 "", (uint64_t) fs->inode);
               /* inode found so the file has been renamed */
               fs->instruction = CSYNC_INSTRUCTION_EVAL_RENAME;
-              if (fs->type == CSYNC_FTW_TYPE_DIR) {
+              if (fs->type == ItemTypeDirectory) {
                   csync_rename_record(ctx, base._path, fs->path);
               }
+
+              goto out;
           }
+
+          if (fs->type == ItemTypeVirtualFile) {
+              // It's allowed to rename "abc" -> "abc.owncloud": In that case the file
+              // becomes a placeholder by being drained of contents.
+              if (base.isValid()
+                  && base._type == ItemTypeFile
+                  && fs->size == base._fileSize
+                  && fs->modtime == base._modtime
+                  && fs->path.left(fs->path.size() - ctx->virtual_file_suffix.size()) == base._path) {
+                  qCInfo(lcUpdate) << "Base file was renamed to virtual file:" << fs->path;
+
+                  // Wipe the base db entry to make sure the base file isn't removed
+                  // from the server later!
+                  // WARNING: Side effect on database during update phase:
+                  // Makes local/remote discovery order dependent!
+                  ctx->statedb->deleteFileRecord(base._path);
+
+                  // Wipe the placeholder file such that the remote discovery will
+                  // note it must be recreated with the right size and flags.
+                  QFile::remove(QString::fromUtf8(ctx->local.uri) + "/" + fs->path);
+
+                  // Also set up the expected db entry
+                  // WARNING: Side effect on database during update phase again
+                  base._path = fs->path;
+                  base._type = ItemTypeVirtualFile;
+                  ctx->statedb->setFileRecord(base);
+
+                  return 0; // Don't add as entry into local files
+              }
+
+              // Remove the spurious file if it looks like a placeholder file
+              // (we know placeholder files contain " ")
+              if (fs->size <= 1) {
+                  QFile::remove(QString::fromUtf8(ctx->local.uri) + "/" + fs->path);
+                  qCWarning(lcUpdate) << "Wiping virtual file without db entry for" << fs->path;
+                  return 0; // Don't add as entry into local files
+              } else {
+                  fs->instruction = CSYNC_INSTRUCTION_IGNORE;
+                  qCWarning(lcUpdate) << "Virtual file without db entry for" << fs->path
+                                      << "but looks odd, keeping";
+              }
+          }
+
           goto out;
 
       } else {
+          qCInfo(lcUpdate, "Checking for rename based on fileid %s", fs->file_id.constData());
+
           /* Remote Replica Rename check */
           fs->instruction = CSYNC_INSTRUCTION_NEW;
 
@@ -324,23 +419,40 @@ static int _csync_detect_update(CSYNC *ctx, std::unique_ptr<csync_file_stat_t> f
               if (!base.isValid())
                   return;
 
+              if (base._type == ItemTypeVirtualFileDownload) {
+                  // Remote rename of a virtual file we have locally scheduled
+                  // for download. We just consider this NEW but mark it for download.
+                  fs->type = ItemTypeVirtualFileDownload;
+                  done = true;
+                  return;
+              }
+
               // Some things prohibit rename detection entirely.
               // Since we don't do the same checks again in reconcile, we can't
               // just skip the candidate, but have to give up completely.
-              if (base._type != fs->type) {
+              if (base._type != fs->type
+                  && base._type != ItemTypeVirtualFile) {
                   qCWarning(lcUpdate, "file types different, not a rename");
                   done = true;
                   return;
               }
-              if (fs->type != CSYNC_FTW_TYPE_DIR && base._etag != fs->etag) {
+              if (fs->type != ItemTypeDirectory && base._etag != fs->etag) {
                   /* File with different etag, don't do a rename, but download the file again */
                   qCWarning(lcUpdate, "file etag different, not a rename");
                   done = true;
                   return;
               }
 
+              // Now we know there is a sane rename candidate.
+
+              // Rename of a virtual file
+              if (base._type == ItemTypeVirtualFile && fs->type == ItemTypeFile) {
+                  fs->type = ItemTypeVirtualFile;
+                  fs->path.append(ctx->virtual_file_suffix);
+              }
+
               // Record directory renames
-              if (fs->type == CSYNC_FTW_TYPE_DIR) {
+              if (fs->type == ItemTypeDirectory) {
                   // If the same folder was already renamed by a different entry,
                   // skip to the next candidate
                   if (ctx->renames.folder_renamed_to.count(base._path) > 0) {
@@ -350,7 +462,7 @@ static int _csync_detect_update(CSYNC *ctx, std::unique_ptr<csync_file_stat_t> f
                   csync_rename_record(ctx, base._path, fs->path);
               }
 
-              qCDebug(lcUpdate, "remote rename detected based on fileid %s --> %s", base._path.constData(), fs->path.constData());
+              qCInfo(lcUpdate, "remote rename detected based on fileid %s --> %s", base._path.constData(), fs->path.constData());
               fs->instruction = CSYNC_INSTRUCTION_EVAL_RENAME;
               done = true;
           };
@@ -361,13 +473,22 @@ static int _csync_detect_update(CSYNC *ctx, std::unique_ptr<csync_file_stat_t> f
           }
 
           if (fs->instruction == CSYNC_INSTRUCTION_NEW
-              && fs->type == CSYNC_FTW_TYPE_DIR
+              && fs->type == ItemTypeDirectory
               && ctx->current == REMOTE_REPLICA
               && ctx->callbacks.checkSelectiveSyncNewFolderHook) {
               if (ctx->callbacks.checkSelectiveSyncNewFolderHook(ctx->callbacks.update_callback_userdata, fs->path, fs->remotePerm)) {
                   return 1;
               }
           }
+
+          // Turn new remote files into virtual files if the option is enabled.
+          if (ctx->new_files_are_virtual
+              && fs->instruction == CSYNC_INSTRUCTION_NEW
+              && fs->type == ItemTypeFile) {
+              fs->type = ItemTypeVirtualFile;
+              fs->path.append(ctx->virtual_file_suffix);
+          }
+
           goto out;
       }
   }
@@ -376,7 +497,7 @@ out:
 
   /* Set the ignored error string. */
   if (fs->instruction == CSYNC_INSTRUCTION_IGNORE) {
-      if( fs->type == CSYNC_FTW_TYPE_SLINK ) {
+      if( fs->type == ItemTypeSoftLink ) {
           fs->error_status = CSYNC_STATUS_INDIVIDUAL_IS_SYMLINK; /* Symbolic links are ignored. */
       } else {
           if (excluded == CSYNC_FILE_EXCLUDE_LIST) {
@@ -401,13 +522,13 @@ out:
   if (fs->instruction != CSYNC_INSTRUCTION_NONE
       && fs->instruction != CSYNC_INSTRUCTION_IGNORE
       && fs->instruction != CSYNC_INSTRUCTION_UPDATE_METADATA
-      && fs->type != CSYNC_FTW_TYPE_DIR) {
+      && fs->type != ItemTypeDirectory) {
     fs->child_modified = true;
   }
 
   // If conflict files are uploaded, they won't be marked as IGNORE / CSYNC_FILE_EXCLUDE_CONFLICT
   // but we still want them marked!
-  if (OCC::Utility::shouldUploadConflictFiles()) {
+  if (ctx->upload_conflict_files) {
       if (OCC::Utility::isConflictFile(fs->path.constData())) {
           fs->error_status = CSYNC_STATUS_INDIVIDUAL_IS_CONFLICT_FILE;
       }
@@ -443,26 +564,35 @@ int csync_walker(CSYNC *ctx, std::unique_ptr<csync_file_stat_t> fs) {
   }
 
   switch (fs->type) {
-    case CSYNC_FTW_TYPE_FILE:
+  case ItemTypeFile:
       if (ctx->current == REMOTE_REPLICA) {
           qCDebug(lcUpdate, "file: %s [file_id=%s size=%" PRIu64 "]", fs->path.constData(), fs->file_id.constData(), fs->size);
       } else {
           qCDebug(lcUpdate, "file: %s [inode=%" PRIu64 " size=%" PRIu64 "]", fs->path.constData(), fs->inode, fs->size);
       }
       break;
-  case CSYNC_FTW_TYPE_DIR: /* enter directory */
+  case ItemTypeVirtualFile:
+      // This can happen when locally a virtual file is seen that doen't have
+      // a matching database entry.
+      if (ctx->current == REMOTE_REPLICA) {
+          qCDebug(lcUpdate, "virtual file: %s [file_id=%s size=%" PRIu64 "]", fs->path.constData(), fs->file_id.constData(), fs->size);
+      } else {
+          qCDebug(lcUpdate, "virtual file: %s [inode=%" PRIu64 " size=%" PRIu64 "]", fs->path.constData(), fs->inode, fs->size);
+      }
+      break;
+  case ItemTypeDirectory: /* enter directory */
       if (ctx->current == REMOTE_REPLICA) {
           qCDebug(lcUpdate, "directory: %s [file_id=%s]", fs->path.constData(), fs->file_id.constData());
       } else {
           qCDebug(lcUpdate, "directory: %s [inode=%" PRIu64 "]", fs->path.constData(), fs->inode);
       }
       break;
-  case CSYNC_FTW_TYPE_SLINK:
-    qCDebug(lcUpdate, "symlink: %s - not supported", fs->path.constData());
+  case ItemTypeSoftLink:
+    qCInfo(lcUpdate, "symlink: %s - not supported", fs->path.constData());
     break;
   default:
+    qCInfo(lcUpdate, "item: %s - item type %d not iterated", fs->path.constData(), fs->type);
     return 0;
-    break;
   }
 
   rc = _csync_detect_update(ctx, std::move(fs));
@@ -470,32 +600,35 @@ int csync_walker(CSYNC *ctx, std::unique_ptr<csync_file_stat_t> fs) {
   return rc;
 }
 
-static bool fill_tree_from_db(CSYNC *ctx, const char *uri)
+static bool fill_tree_from_db(CSYNC *ctx, const char *uri, bool singleFile = false)
 {
     int64_t count = 0;
     QByteArray skipbase;
-    auto rowCallback = [ctx, &count, &skipbase](const OCC::SyncJournalFileRecord &rec) {
-        /* When selective sync is used, the database may have subtrees with a parent
-         * whose etag (md5) is _invalid_. These are ignored and shall not appear in the
-         * remote tree.
-         * Sometimes folders that are not ignored by selective sync get marked as
-         * _invalid_, but that is not a problem as the next discovery will retrieve
-         * their correct etags again and we don't run into this case.
-         */
-        if( rec._etag == "_invalid_") {
-            qCDebug(lcUpdate, "%s selective sync excluded", rec._path.constData());
-            skipbase = rec._path;
-            skipbase += '/';
-            return;
-        }
+    auto &files = ctx->current == LOCAL_REPLICA ? ctx->local.files : ctx->remote.files;
+    auto rowCallback = [ctx, &count, &skipbase, &files](const OCC::SyncJournalFileRecord &rec) {
+        if (ctx->current == REMOTE_REPLICA) {
+            /* When selective sync is used, the database may have subtrees with a parent
+             * whose etag is _invalid_. These are ignored and shall not appear in the
+             * remote tree.
+             * Sometimes folders that are not ignored by selective sync get marked as
+             * _invalid_, but that is not a problem as the next discovery will retrieve
+             * their correct etags again and we don't run into this case.
+             */
+            if (rec._etag == "_invalid_") {
+                qCInfo(lcUpdate, "%s selective sync excluded", rec._path.constData());
+                skipbase = rec._path;
+                skipbase += '/';
+                return;
+            }
 
-        /* Skip over all entries with the same base path. Note that this depends
-         * strongly on the ordering of the retrieved items. */
-        if( !skipbase.isEmpty() && rec._path.startsWith(skipbase) ) {
-            qCDebug(lcUpdate, "%s selective sync excluded because the parent is", rec._path.constData());
-            return;
-        } else {
-            skipbase.clear();
+            /* Skip over all entries with the same base path. Note that this depends
+             * strongly on the ordering of the retrieved items. */
+            if (!skipbase.isEmpty() && rec._path.startsWith(skipbase)) {
+                qCDebug(lcUpdate, "%s selective sync excluded because the parent is", rec._path.constData());
+                return;
+            } else {
+                skipbase.clear();
+            }
         }
 
         std::unique_ptr<csync_file_stat_t> st = csync_file_stat_t::fromSyncJournalFileRecord(rec);
@@ -503,7 +636,9 @@ static bool fill_tree_from_db(CSYNC *ctx, const char *uri)
         /* Check for exclusion from the tree.
          * Note that this is only a safety net in case the ignore list changes
          * without a full remote discovery being triggered. */
-        CSYNC_EXCLUDE_TYPE excluded = csync_excluded_traversal(ctx, st->path, st->type);
+        CSYNC_EXCLUDE_TYPE excluded = CSYNC_NOT_EXCLUDED;
+        if (ctx->exclude_traversal_fn)
+            excluded = ctx->exclude_traversal_fn(st->path, st->type);
         if (excluded != CSYNC_NOT_EXCLUDED) {
             qInfo(lcUpdate, "%s excluded from db read (%d)", st->path.constData(), excluded);
 
@@ -516,13 +651,23 @@ static bool fill_tree_from_db(CSYNC *ctx, const char *uri)
         }
 
         /* store into result list. */
-        ctx->remote.files[rec._path] = std::move(st);
+        files[rec._path] = std::move(st);
         ++count;
     };
 
-    if (!ctx->statedb->getFilesBelowPath(uri, rowCallback)) {
-        ctx->status_code = CSYNC_STATUS_STATEDB_LOAD_ERROR;
-        return false;
+    if (singleFile) {
+        OCC::SyncJournalFileRecord record;
+        if (ctx->statedb->getFileRecord(QByteArray(uri), &record) && record.isValid()) {
+            rowCallback(record);
+        } else {
+            ctx->status_code = CSYNC_STATUS_STATEDB_LOAD_ERROR;
+            return false;
+        }
+    } else {
+        if (!ctx->statedb->getFilesBelowPath(uri, rowCallback)) {
+            ctx->status_code = CSYNC_STATUS_STATEDB_LOAD_ERROR;
+            return false;
+        }
     }
     qInfo(lcUpdate, "%" PRId64 " entries read below path %s from db.", count, uri);
 
@@ -561,6 +706,15 @@ int csync_ftw(CSYNC *ctx, const char *uri, csync_walker_fn fn,
   int rc = 0;
 
   bool do_read_from_db = (ctx->current == REMOTE_REPLICA && ctx->remote.read_from_db);
+  const char *db_uri = uri;
+
+  if (ctx->current == LOCAL_REPLICA && ctx->should_discover_locally_fn) {
+      const char *local_uri = uri + strlen(ctx->local.uri);
+      if (*local_uri == '/')
+          ++local_uri;
+      db_uri = local_uri;
+      do_read_from_db = !ctx->should_discover_locally_fn(QByteArray(local_uri));
+  }
 
   if (!depth) {
     mark_current_item_ignored(ctx, previous_fs, CSYNC_STATUS_INDIVIDUAL_TOO_DEEP);
@@ -572,7 +726,7 @@ int csync_ftw(CSYNC *ctx, const char *uri, csync_walker_fn fn,
   // if the etag of this dir is still the same, its content is restored from the
   // database.
   if( do_read_from_db ) {
-      if( ! fill_tree_from_db(ctx, uri) ) {
+      if( ! fill_tree_from_db(ctx, db_uri) ) {
         errno = ENOENT;
         ctx->status_code = CSYNC_STATUS_OPENDIR_ERROR;
         goto error;
@@ -586,7 +740,6 @@ int csync_ftw(CSYNC *ctx, const char *uri, csync_walker_fn fn,
           ctx->status_code = CSYNC_STATUS_ABORTED;
           goto error;
       }
-      int asp = 0;
       /* permission denied */
       ctx->status_code = csync_errno_to_status(errno, CSYNC_STATUS_OPENDIR_ERROR);
       if (errno == EACCES) {
@@ -595,8 +748,7 @@ int csync_ftw(CSYNC *ctx, const char *uri, csync_walker_fn fn,
               return 0;
           }
       } else if(errno == ENOENT) {
-          asp = asprintf( &ctx->error_string, "%s", uri);
-          ASSERT(asp >= 0);
+          ctx->error_string = QString::fromUtf8(uri);
       }
       // 403 Forbidden can be sent by the server if the file firewall is active.
       // A file or directory should be ignored and sync must continue. See #3490
@@ -623,11 +775,25 @@ int csync_ftw(CSYNC *ctx, const char *uri, csync_walker_fn fn,
       goto error;
   }
 
-  while ((dirent = csync_vio_readdir(ctx, dh))) {
+  while (true) {
+    // Get the next item in the directory
+    errno = 0;
+    dirent = csync_vio_readdir(ctx, dh);
+    if (!dirent) {
+        if (errno != 0) {
+            // Note: Windows vio converts any error into EACCES
+            qCWarning(lcUpdate, "readdir failed for file in %s - errno %d", uri, errno);
+            goto error;
+        }
+
+        // Normal case: End of items in directory
+        break;
+    }
+
     /* Conversion error */
     if (dirent->path.isEmpty() && !dirent->original_path.isEmpty()) {
         ctx->status_code = CSYNC_STATUS_INVALID_CHARACTERS;
-        ctx->error_string = c_strdup(dirent->original_path);
+        ctx->error_string = QString::fromUtf8(dirent->original_path);
         dirent->original_path.clear();
         goto error;
     }
@@ -650,6 +816,21 @@ int csync_ftw(CSYNC *ctx, const char *uri, csync_walker_fn fn,
         fullpath = QByteArray() % uri % '/' % filename;
     }
 
+    // When encountering virtual files, read the relevant
+    // entry from the db instead.
+    if (ctx->current == LOCAL_REPLICA
+        && dirent->type == ItemTypeFile
+        && filename.endsWith(ctx->virtual_file_suffix)) {
+        QByteArray db_uri = fullpath.mid(strlen(ctx->local.uri) + 1);
+
+        if (fill_tree_from_db(ctx, db_uri.constData(), true))
+            continue;
+
+        // We didn't find this file in the database: Let this proceed into
+        // rename detection as a virtual file.
+        dirent->type = ItemTypeVirtualFile;
+    }
+
     /* if the filename starts with a . we consider it a hidden file
      * For windows, the hidden state is also discovered within the vio
      * local stat function.
@@ -663,16 +844,13 @@ int csync_ftw(CSYNC *ctx, const char *uri, csync_walker_fn fn,
     // Now process to have a relative path to the sync root for the local replica, or to the data root on the remote.
     dirent->path = fullpath;
     if (ctx->current == LOCAL_REPLICA) {
-        if (dirent->path.size() <= (int)strlen(ctx->local.uri)) {
-            ctx->status_code = CSYNC_STATUS_PARAM_ERROR;
-            goto error;
-        }
+        ASSERT(dirent->path.startsWith(ctx->local.uri)); // path is relative to uri
         // "len + 1" to include the slash in-between.
         dirent->path = dirent->path.mid(strlen(ctx->local.uri) + 1);
     }
 
     previous_fs = ctx->current_fs;
-    bool recurse = dirent->type == CSYNC_FTW_TYPE_DIR;
+    bool recurse = dirent->type == ItemTypeDirectory;
 
     /* Call walker function for each file */
     rc = fn(ctx, std::move(dirent));
@@ -720,7 +898,7 @@ int csync_ftw(CSYNC *ctx, const char *uri, csync_walker_fn fn,
   }
 
   csync_vio_closedir(ctx, dh);
-  qCDebug(lcUpdate, " <= Closing walk for %s with read_from_db %d", uri, read_from_db);
+  qCInfo(lcUpdate, " <= Closing walk for %s with read_from_db %d", uri, read_from_db);
 
   return rc;
 
