@@ -94,7 +94,6 @@ public:
 
     ConnectionStatus connectionStatus() const;
     QStringList connectionErrors() const;
-    static QString connectionStatusString(ConnectionStatus status);
 
     State state() const;
     static QString stateString(State state);
@@ -105,6 +104,16 @@ public:
      * for the account and forgets the password. */
     void signOutByUi();
 
+    /** Tries to connect from scratch.
+     *
+     * Does nothing for signed out accounts.
+     * Connected accounts will be disconnected and try anew.
+     * Disconnected accounts will go to checkConnectivity().
+     *
+     * Useful for when network settings (proxy) change.
+     */
+    void freshConnectionAttempt();
+
     /// Move from SignedOut state to Disconnected (attempting to connect)
     void signIn();
 
