@@ -135,11 +135,6 @@ void ActivityItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &
     const QString elidedAction = fm.elidedText(actionText, Qt::ElideRight, actionTextBox.width());
     painter->drawText(actionTextBox, elidedAction);
 
-    int atPos = accountRole.indexOf(QLatin1Char('@'));
-    if (atPos > -1) {
-        accountRole.remove(0, atPos + 1);
-    }
-
     QString timeStr;
     if (accountOnline) {
         timeStr = tr("%1 on %2").arg(timeText, accountRole);
