@@ -84,6 +84,11 @@ ActivityWidget::ActivityWidget(QWidget *parent)
     connect(_model, &ActivityListModel::activityJobStatusCode,
         this, &ActivityWidget::slotAccountActivityStatus);
 
+    connect(AccountManager::instance(), &AccountManager::accountRemoved, this, [this](AccountState *ast) {
+        if (_accountsWithoutActivities.remove(ast->account()->displayName()))
+            showLabels();
+    });
+
     _copyBtn = _ui->_dialogButtonBox->addButton(tr("Copy"), QDialogButtonBox::ActionRole);
     _copyBtn->setToolTip(tr("Copy the activity list to the clipboard."));
     connect(_copyBtn, &QAbstractButton::clicked, this, &ActivityWidget::copyToClipboard);
@@ -290,11 +295,6 @@ void ActivityWidget::slotBuildNotificationDisplay(const ActivityList &list)
             QString host = activity._accName;
             // store the name of the account that sends the notification to be
             // able to add it to the tray notification
-            // remove the user name from the account as that is not accurate here.
-            int indx = host.indexOf(QChar('@'));
-            if (indx > -1) {
-                host.remove(0, 1 + indx);
-            }
             if (!host.isEmpty()) {
                 if (accNotified.contains(host)) {
                     accNotified[host] = accNotified[host] + 1;
@@ -544,10 +544,10 @@ ActivitySettings::ActivitySettings(QWidget *parent)
     _tab->setCurrentIndex(1);
 }
 
-void ActivitySettings::setNotificationRefreshInterval(quint64 interval)
+void ActivitySettings::setNotificationRefreshInterval(std::chrono::milliseconds interval)
 {
-    qCDebug(lcActivity) << "Starting Notification refresh timer with " << interval / 1000 << " sec interval";
-    _notificationCheckTimer.start(interval);
+    qCDebug(lcActivity) << "Starting Notification refresh timer with " << interval.count() / 1000 << " sec interval";
+    _notificationCheckTimer.start(interval.count());
 }
 
 void ActivitySettings::setActivityTabHidden(bool hidden)
