@@ -72,13 +72,17 @@ public slots:
     // TODO: this should not be public
     void slotownCloudWizardDone(int);
     void slotCrash();
+    /**
+     * Will download a virtual file, and open the result.
+     * The argument is the filename of the virtual file (including the extension)
+     */
+    void openVirtualFile(const QString &filename);
 
 protected:
     void parseOptions(const QStringList &);
     void setupTranslations();
     void setupLogging();
-    void enterNextLogFile();
-    bool checkConfigExists(bool openSettings);
+    bool event(QEvent *event);
 
 signals:
     void folderRemoved();
@@ -96,6 +100,12 @@ protected slots:
 private:
     void setHelp();
 
+    /**
+     * Maybe a newer version of the client was used with this config file:
+     * if so, backup, confirm with user and remove the config that can't be read.
+     */
+    bool configVersionMigration();
+
     QPointer<ownCloudGui> _gui;
 
     Theme *_theme;
