@@ -14,25 +14,28 @@
 
 #include "shibbolethuserjob.h"
 #include <account.h>
-#include <json.h>
+
+#include <QJsonDocument>
+#include <QJsonObject>
+#include <QLoggingCategory>
 
 namespace OCC {
 
-ShibbolethUserJob::ShibbolethUserJob(AccountPtr account, QObject* parent)
+Q_DECLARE_LOGGING_CATEGORY(lcShibboleth)
+
+ShibbolethUserJob::ShibbolethUserJob(AccountPtr account, QObject *parent)
     : JsonApiJob(account, QLatin1String("ocs/v1.php/cloud/user"), parent)
 {
     setIgnoreCredentialFailure(true);
-    connect(this, SIGNAL(jsonReceived(QVariantMap, int)), this, SLOT(slotJsonReceived(QVariantMap, int)));
+    connect(this, &JsonApiJob::jsonReceived, this, &ShibbolethUserJob::slotJsonReceived);
 }
 
-void ShibbolethUserJob::slotJsonReceived(const QVariantMap &json, int statusCode)
+void ShibbolethUserJob::slotJsonReceived(const QJsonDocument &json, int statusCode)
 {
-    if( statusCode != 100 ) {
-        qWarning() << "JSON Api call resulted in status code != 100";
+    if (statusCode != 100) {
+        qCWarning(lcShibboleth) << "JSON Api call resulted in status code != 100";
     }
-    QString user =  json.value("ocs").toMap().value("data").toMap().value("id").toString();
-    //qDebug() << "cloud/user: " << json << "->" << user;
+    QString user = json.object().value("ocs").toObject().value("data").toObject().value("id").toString();
     emit userFetched(user);
 }
-
 }
