@@ -16,6 +16,7 @@
 #define MIRALL_CREDS_SHIBBOLETH_CREDENTIALS_H
 
 #include <QList>
+#include <QLoggingCategory>
 #include <QMap>
 #include <QNetworkCookie>
 #include <QUrl>
@@ -24,13 +25,14 @@
 #include "creds/abstractcredentials.h"
 
 namespace QKeychain {
-    class Job;
+class Job;
 }
 
 class QAuthenticator;
 
-namespace OCC
-{
+namespace OCC {
+
+Q_DECLARE_LOGGING_CATEGORY(lcShibboleth)
 
 class ShibbolethWebView;
 
@@ -40,7 +42,7 @@ class ShibbolethWebView;
  */
 class ShibbolethCredentials : public AbstractCredentials
 {
-Q_OBJECT
+    Q_OBJECT
 
 public:
     ShibbolethCredentials();
@@ -48,10 +50,10 @@ public:
     /* create credentials for an already connected account */
     ShibbolethCredentials(const QNetworkCookie &cookie);
 
-    void setAccount(Account* account) Q_DECL_OVERRIDE;
+    void setAccount(Account *account) Q_DECL_OVERRIDE;
     QString authType() const Q_DECL_OVERRIDE;
     QString user() const Q_DECL_OVERRIDE;
-    QNetworkAccessManager* getQNAM() const Q_DECL_OVERRIDE;
+    QNetworkAccessManager *createQNAM() const Q_DECL_OVERRIDE;
     bool ready() const Q_DECL_OVERRIDE;
     void fetchFromKeychain() Q_DECL_OVERRIDE;
     void askFromUser() Q_DECL_OVERRIDE;
@@ -67,21 +69,25 @@ public:
     static QByteArray shibCookieName();
 
 private Q_SLOTS:
-    void onShibbolethCookieReceived(const QNetworkCookie&);
+    void onShibbolethCookieReceived(const QNetworkCookie &);
     void slotBrowserRejected();
-    void slotReadJobDone(QKeychain::Job*);
-    void slotReplyFinished(QNetworkReply*);
-    void slotUserFetched(const QString& user);
+    void slotReadJobDone(QKeychain::Job *);
+    void slotReplyFinished(QNetworkReply *);
+    void slotUserFetched(const QString &user);
     void slotFetchUser();
     void slotFetchUserHelper();
 
 Q_SIGNALS:
-    void newCookie(const QNetworkCookie& cookie);
+    void newCookie(const QNetworkCookie &cookie);
 
 private:
     void storeShibCookie(const QNetworkCookie &cookie);
     void removeShibCookie();
     void addToCookieJar(const QNetworkCookie &cookie);
+
+    /// Reads data from keychain, progressing to slotReadJobDone
+    void fetchFromKeychainHelper();
+
     QUrl _url;
     QByteArray prepareCookieData() const;
 
@@ -90,6 +96,7 @@ private:
     QPointer<ShibbolethWebView> _browser;
     QNetworkCookie _shibCookie;
     QString _user;
+    bool _keychainMigration;
 };
 
 } // namespace OCC
