@@ -30,7 +30,7 @@
 #include "socketapi.h"
 #include "theme.h"
 #include "filesystem.h"
-#include "excludedfiles.h"
+#include "localdiscoverytracker.h"
 
 #include "creds/abstractcredentials.h"
 
@@ -42,6 +42,8 @@
 #include <QMessageBox>
 #include <QPushButton>
 
+static const char versionC[] = "version";
+
 namespace OCC {
 
 Q_LOGGING_CATEGORY(lcFolder, "gui.folder", QtInfoMsg)
@@ -53,13 +55,11 @@ Folder::Folder(const FolderDefinition &definition,
     , _accountState(accountState)
     , _definition(definition)
     , _csyncUnavail(false)
-    , _proxyDirty(true)
     , _lastSyncDuration(0)
     , _consecutiveFailingSyncs(0)
     , _consecutiveFollowUpSyncs(0)
     , _journal(_definition.absoluteJournalPath())
     , _fileLog(new SyncRunFileLog)
-    , _saveBackwardsCompatible(false)
 {
     _timeSinceLastSyncStart.start();
     _timeSinceLastSyncDone.start();
@@ -79,7 +79,8 @@ Folder::Folder(const FolderDefinition &definition,
     // pass the setting if hidden files are to be ignored, will be read in csync_update
     _engine->setIgnoreHiddenFiles(_definition.ignoreHiddenFiles);
 
-    if (!setIgnoredFiles())
+    ConfigFile::setupDefaultExcludeFilePaths(_engine->excludedFiles());
+    if (!reloadExcludes())
         qCWarning(lcFolder, "Could not read system exclude file");
 
     connect(_accountState.data(), &AccountState::isConnectedChanged, this, &Folder::canSyncChanged);
@@ -108,6 +109,15 @@ Folder::Folder(const FolderDefinition &definition,
     _scheduleSelfTimer.setInterval(SyncEngine::minimumFileAgeForUpload);
     connect(&_scheduleSelfTimer, &QTimer::timeout,
         this, &Folder::slotScheduleThisFolder);
+
+    connect(ProgressDispatcher::instance(), &ProgressDispatcher::folderConflicts,
+        this, &Folder::slotFolderConflicts);
+
+    _localDiscoveryTracker.reset(new LocalDiscoveryTracker);
+    connect(_engine.data(), &SyncEngine::finished,
+        _localDiscoveryTracker.data(), &LocalDiscoveryTracker::slotSyncFinished);
+    connect(_engine.data(), &SyncEngine::itemCompleted,
+        _localDiscoveryTracker.data(), &LocalDiscoveryTracker::slotItemCompleted);
 }
 
 Folder::~Folder()
@@ -116,11 +126,14 @@ Folder::~Folder()
     _engine.reset();
 }
 
-
 void Folder::checkLocalPath()
 {
     const QFileInfo fi(_definition.localPath);
     _canonicalLocalPath = fi.canonicalFilePath();
+#ifdef Q_OS_MAC
+    // Workaround QTBUG-55896  (Should be fixed in Qt 5.8)
+    _canonicalLocalPath = _canonicalLocalPath.normalized(QString::NormalizationForm_C);
+#endif
     if (_canonicalLocalPath.isEmpty()) {
         qCWarning(lcFolder) << "Broken symlink:" << _definition.localPath;
         _canonicalLocalPath = _definition.localPath;
@@ -448,12 +461,25 @@ int Folder::slotWipeErrorBlacklist()
 
 void Folder::slotWatchedPathChanged(const QString &path)
 {
+    if (!path.startsWith(this->path())) {
+        qCDebug(lcFolder) << "Changed path is not contained in folder, ignoring:" << path;
+        return;
+    }
+
+    auto relativePath = path.midRef(this->path().size());
+
+    // Add to list of locally modified paths
+    //
+    // We do this before checking for our own sync-related changes to make
+    // extra sure to not miss relevant changes.
+    auto relativePathBytes = relativePath.toUtf8();
+    _localDiscoveryTracker->addTouchedPath(relativePathBytes);
+
 // The folder watcher fires a lot of bogus notifications during
 // a sync operation, both for actual user files and the database
 // and log. Therefore we check notifications against operations
 // the sync is doing to filter out our own changes.
 #ifdef Q_OS_MAC
-    Q_UNUSED(path)
 // On OSX the folder watcher does not report changes done by our
 // own process. Therefore nothing needs to be done here!
 #else
@@ -465,17 +491,16 @@ void Folder::slotWatchedPathChanged(const QString &path)
 #endif
 
     // Check that the mtime actually changed.
-    if (path.startsWith(this->path())) {
-        auto relativePath = path.mid(this->path().size());
-        SyncJournalFileRecord record;
-        if (_journal.getFileRecord(relativePath, &record)
-            && record.isValid()
-            && !FileSystem::fileChanged(path, record._fileSize, record._modtime)) {
-            qCInfo(lcFolder) << "Ignoring spurious notification for file" << relativePath;
-            return; // probably a spurious notification
-        }
+    SyncJournalFileRecord record;
+    if (_journal.getFileRecord(relativePathBytes, &record)
+        && record.isValid()
+        && !FileSystem::fileChanged(path, record._fileSize, record._modtime)) {
+        qCInfo(lcFolder) << "Ignoring spurious notification for file" << relativePath;
+        return; // probably a spurious notification
     }
 
+    warnOnNewExcludedItem(record, relativePath);
+
     emit watchedFileChangedExternally(path);
 
     // Also schedule this folder for a sync, but only after some delay:
@@ -483,20 +508,49 @@ void Folder::slotWatchedPathChanged(const QString &path)
     scheduleThisFolderSoon();
 }
 
+void Folder::downloadVirtualFile(const QString &_relativepath)
+{
+    qCInfo(lcFolder) << "Download virtual file: " << _relativepath;
+    auto relativepath = _relativepath.toUtf8();
+
+    // Set in the database that we should download the file
+    SyncJournalFileRecord record;
+    _journal.getFileRecord(relativepath, &record);
+    if (!record.isValid())
+        return;
+    if (record._type == ItemTypeVirtualFile) {
+        record._type = ItemTypeVirtualFileDownload;
+        _journal.setFileRecord(record);
+        // Make sure we go over that file during the discovery
+        _journal.avoidReadFromDbOnNextSync(relativepath);
+    } else if (record._type == ItemTypeDirectory) {
+        _journal.markVirtualFileForDownloadRecursively(relativepath);
+    } else {
+        qCWarning(lcFolder) << "Invalid existing record " << record._type << " for file " << _relativepath;
+    }
+
+    // Schedule a sync (Folder man will start the sync in a few ms)
+    slotScheduleThisFolder();
+}
+
+
+void Folder::setUseVirtualFiles(bool enabled)
+{
+    _definition.useVirtualFiles = enabled;
+    if (enabled)
+        _saveInFoldersWithPlaceholders = true;
+    saveToSettings();
+}
+
 void Folder::saveToSettings() const
 {
     // Remove first to make sure we don't get duplicates
     removeFromSettings();
 
     auto settings = _accountState->settings();
+    QString settingsGroup = QStringLiteral("Multifolders");
 
-    // The folder is saved to backwards-compatible "Folders"
-    // section only if it has the migrate flag set (i.e. was in
-    // there before) or if the folder is the only one for the
-    // given target path.
-    // This ensures that older clients will not read a configuration
-    // where two folders for different accounts point at the same
-    // local folders.
+    // True if the folder path appears in only one account
     bool oneAccountOnly = true;
     foreach (Folder *other, FolderMan::instance()->map()) {
         if (other != this && other->cleanPath() == this->cleanPath()) {
@@ -505,13 +559,25 @@ void Folder::saveToSettings() const
         }
     }
 
-    bool compatible = _saveBackwardsCompatible || oneAccountOnly;
-
-    if (compatible) {
-        settings->beginGroup(QLatin1String("Folders"));
-    } else {
-        settings->beginGroup(QLatin1String("Multifolders"));
+    if (_definition.useVirtualFiles || _saveInFoldersWithPlaceholders) {
+        // If virtual files are enabled or even were enabled at some point,
+        // save the folder to a group that will not be read by older (<2.5.0) clients.
+        // The name is from when virtual files were called placeholders.
+        settingsGroup = QStringLiteral("FoldersWithPlaceholders");
+    } else if (_saveBackwardsCompatible || oneAccountOnly) {
+        // The folder is saved to backwards-compatible "Folders"
+        // section only if it has the migrate flag set (i.e. was in
+        // there before) or if the folder is the only one for the
+        // given target path.
+        // This ensures that older clients will not read a configuration
+        // where two folders for different accounts point at the same
+        // local folders.
+        settingsGroup = QStringLiteral("Folders");
     }
+
+    settings->beginGroup(settingsGroup);
+    // Note: Each of these groups might have a "version" tag, but that's
+    //       currently unused.
     FolderDefinition::save(*settings, _definition);
 
     settings->sync();
@@ -526,6 +592,9 @@ void Folder::removeFromSettings() const
     settings->endGroup();
     settings->beginGroup(QLatin1String("Multifolders"));
     settings->remove(FolderMan::escapeAlias(_definition.alias));
+    settings->endGroup();
+    settings->beginGroup(QLatin1String("FoldersWithPlaceholders"));
+    settings->remove(FolderMan::escapeAlias(_definition.alias));
 }
 
 bool Folder::isFileExcludedAbsolute(const QString &fullPath) const
@@ -584,42 +653,14 @@ void Folder::wipe()
         FolderMan::instance()->socketApi()->slotRegisterPath(alias());
 }
 
-bool Folder::setIgnoredFiles()
+bool Folder::reloadExcludes()
 {
-    // Note: Doing this on each sync run and on Folder construction is
-    // unnecessary, because _engine->excludedFiles() persists between
-    // sync runs. This is not a big problem because ExcludedFiles maintains
-    // a QSet of files to load.
-    ConfigFile cfg;
-    QString systemList = cfg.excludeFile(ConfigFile::SystemScope);
-    qCInfo(lcFolder) << "Adding system ignore list to csync:" << systemList;
-    _engine->excludedFiles().addExcludeFilePath(systemList);
-
-    QString userList = cfg.excludeFile(ConfigFile::UserScope);
-    if (QFile::exists(userList)) {
-        qCInfo(lcFolder) << "Adding user defined ignore list to csync:" << userList;
-        _engine->excludedFiles().addExcludeFilePath(userList);
-    }
-
-    return _engine->excludedFiles().reloadExcludes();
-}
-
-void Folder::setProxyDirty(bool value)
-{
-    _proxyDirty = value;
-}
-
-bool Folder::proxyDirty()
-{
-    return _proxyDirty;
+    return _engine->excludedFiles().reloadExcludeFiles();
 }
 
 void Folder::startSync(const QStringList &pathList)
 {
     Q_UNUSED(pathList)
-    if (proxyDirty()) {
-        setProxyDirty(false);
-    }
 
     if (isBusy()) {
         qCCritical(lcFolder) << "ERROR csync is still running and new sync requested.";
@@ -636,7 +677,7 @@ void Folder::startSync(const QStringList &pathList)
 
     _fileLog->start(path());
 
-    if (!setIgnoredFiles()) {
+    if (!reloadExcludes()) {
         slotSyncError(tr("Could not read system exclude file"));
         QMetaObject::invokeMethod(this, "slotSyncFinished", Qt::QueuedConnection, Q_ARG(bool, false));
         return;
@@ -645,6 +686,32 @@ void Folder::startSync(const QStringList &pathList)
     setDirtyNetworkLimits();
     setSyncOptions();
 
+    static std::chrono::milliseconds fullLocalDiscoveryInterval = []() {
+        auto interval = ConfigFile().fullLocalDiscoveryInterval();
+        QByteArray env = qgetenv("OWNCLOUD_FULL_LOCAL_DISCOVERY_INTERVAL");
+        if (!env.isEmpty()) {
+            interval = std::chrono::milliseconds(env.toLongLong());
+        }
+        return interval;
+    }();
+    bool hasDoneFullLocalDiscovery = _timeSinceLastFullLocalDiscovery.isValid();
+    bool periodicFullLocalDiscoveryNow =
+        fullLocalDiscoveryInterval.count() >= 0 // negative means we don't require periodic full runs
+        && _timeSinceLastFullLocalDiscovery.hasExpired(fullLocalDiscoveryInterval.count());
+    if (_folderWatcher && _folderWatcher->isReliable()
+        && hasDoneFullLocalDiscovery
+        && !periodicFullLocalDiscoveryNow) {
+        qCInfo(lcFolder) << "Allowing local discovery to read from the database";
+        _engine->setLocalDiscoveryOptions(
+            LocalDiscoveryStyle::DatabaseAndFilesystem,
+            _localDiscoveryTracker->localDiscoveryPaths());
+        _localDiscoveryTracker->startSyncPartialDiscovery();
+    } else {
+        qCInfo(lcFolder) << "Forbidding local discovery to read from the database";
+        _engine->setLocalDiscoveryOptions(LocalDiscoveryStyle::FilesystemOnly);
+        _localDiscoveryTracker->startSyncFullDiscovery();
+    }
+
     _engine->setIgnoreHiddenFiles(_definition.ignoreHiddenFiles);
 
     QMetaObject::invokeMethod(_engine.data(), "startSync", Qt::QueuedConnection);
@@ -660,6 +727,9 @@ void Folder::setSyncOptions()
     auto newFolderLimit = cfgFile.newBigFolderSizeLimit();
     opt._newBigFolderSizeLimit = newFolderLimit.first ? newFolderLimit.second * 1000LL * 1000LL : -1; // convert from MB to B
     opt._confirmExternalStorage = cfgFile.confirmExternalStorage();
+    opt._moveFilesToTrash = cfgFile.moveToTrash();
+    opt._newFilesAreVirtual = _definition.useVirtualFiles;
+    opt._virtualFileSuffix = QStringLiteral(APPLICATION_DOTVIRTUALFILE_SUFFIX);
 
     QByteArray chunkSizeEnv = qgetenv("OWNCLOUD_CHUNK_SIZE");
     if (!chunkSizeEnv.isEmpty()) {
@@ -689,7 +759,7 @@ void Folder::setSyncOptions()
 
     QByteArray targetChunkUploadDurationEnv = qgetenv("OWNCLOUD_TARGET_CHUNK_UPLOAD_DURATION");
     if (!targetChunkUploadDurationEnv.isEmpty()) {
-        opt._targetChunkUploadDuration = targetChunkUploadDurationEnv.toUInt();
+        opt._targetChunkUploadDuration = std::chrono::milliseconds(targetChunkUploadDurationEnv.toUInt());
     } else {
         opt._targetChunkUploadDuration = cfgFile.targetChunkUploadDuration();
     }
@@ -783,6 +853,15 @@ void Folder::slotSyncFinished(bool success)
         journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncWhiteList, QStringList());
     }
 
+    if ((_syncResult.status() == SyncResult::Success
+            || _syncResult.status() == SyncResult::Problem)
+        && success) {
+        if (_engine->lastLocalDiscoveryStyle() == LocalDiscoveryStyle::FilesystemOnly) {
+            _timeSinceLastFullLocalDiscovery.start();
+        }
+    }
+
+
     emit syncStateChange();
 
     // The syncFinished result that is to be triggered here makes the folderman
@@ -793,7 +872,7 @@ void Folder::slotSyncFinished(bool success)
     // all come in.
     QTimer::singleShot(200, this, &Folder::slotEmitFinishedDelayed);
 
-    _lastSyncDuration = _timeSinceLastSyncStart.elapsed();
+    _lastSyncDuration = std::chrono::milliseconds(_timeSinceLastSyncStart.elapsed());
     _timeSinceLastSyncDone.start();
 
     // Increment the follow-up sync counter if necessary.
@@ -848,10 +927,12 @@ void Folder::slotItemCompleted(const SyncFileItemPtr &item)
 
     // add new directories or remove gone away dirs to the watcher
     if (item->isDirectory() && item->_instruction == CSYNC_INSTRUCTION_NEW) {
-        FolderMan::instance()->addMonitorPath(alias(), path() + item->_file);
+        if (_folderWatcher)
+            _folderWatcher->addPath(path() + item->_file);
     }
     if (item->isDirectory() && item->_instruction == CSYNC_INSTRUCTION_REMOVE) {
-        FolderMan::instance()->removeMonitorPath(alias(), path() + item->_file);
+        if (_folderWatcher)
+            _folderWatcher->removePath(path() + item->_file);
     }
 
     _syncResult.processCompletedItem(item);
@@ -906,6 +987,68 @@ void Folder::slotScheduleThisFolder()
     FolderMan::instance()->scheduleFolder(this);
 }
 
+void Folder::slotNextSyncFullLocalDiscovery()
+{
+    _timeSinceLastFullLocalDiscovery.invalidate();
+}
+
+void Folder::slotFolderConflicts(const QString &folder, const QStringList &conflictPaths)
+{
+    if (folder != _definition.alias)
+        return;
+    auto &r = _syncResult;
+
+    // If the number of conflicts is too low, adjust it upwards
+    if (conflictPaths.size() > r.numNewConflictItems() + r.numOldConflictItems())
+        r.setNumOldConflictItems(conflictPaths.size() - r.numNewConflictItems());
+}
+
+void Folder::warnOnNewExcludedItem(const SyncJournalFileRecord &record, const QStringRef &path)
+{
+    // Never warn for items in the database
+    if (record.isValid())
+        return;
+
+    // Don't warn for items that no longer exist.
+    // Note: This assumes we're getting file watcher notifications
+    // for folders only on creation and deletion - if we got a notification
+    // on content change that would create spurious warnings.
+    QFileInfo fi(_canonicalLocalPath + path);
+    if (!fi.exists())
+        return;
+
+    bool ok = false;
+    auto blacklist = _journal.getSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList, &ok);
+    if (!ok)
+        return;
+    if (!blacklist.contains(path + "/"))
+        return;
+
+    const auto message = fi.isDir()
+        ? tr("The folder %1 was created but was excluded from synchronization previously. "
+             "Data inside it will not be synchronized.")
+              .arg(fi.filePath())
+        : tr("The file %1 was created but was excluded from synchronization previously. "
+             "It will not be synchronized.")
+              .arg(fi.filePath());
+
+    Logger::instance()->postOptionalGuiLog(Theme::instance()->appNameGUI(), message);
+}
+
+void Folder::slotWatcherUnreliable(const QString &message)
+{
+    qCWarning(lcFolder) << "Folder watcher for" << path() << "became unreliable:" << message;
+    auto fullMessage =
+        tr("Changes in synchronized folders could not be tracked reliably.\n"
+           "\n"
+           "This means that the synchronization client might not upload local changes "
+           "immediately and will instead only scan for local changes and upload them "
+           "occasionally (every two hours by default).\n"
+           "\n"
+           "%1").arg(message);
+    Logger::instance()->postGuiLog(Theme::instance()->appNameGUI(), fullMessage);
+}
+
 void Folder::scheduleThisFolderSoon()
 {
     if (!_scheduleSelfTimer.isActive()) {
@@ -918,6 +1061,23 @@ void Folder::setSaveBackwardsCompatible(bool save)
     _saveBackwardsCompatible = save;
 }
 
+void Folder::registerFolderWatcher()
+{
+    if (_folderWatcher)
+        return;
+    if (!QDir(path()).exists())
+        return;
+
+    _folderWatcher.reset(new FolderWatcher(this));
+    connect(_folderWatcher.data(), &FolderWatcher::pathChanged,
+        this, &Folder::slotWatchedPathChanged);
+    connect(_folderWatcher.data(), &FolderWatcher::lostChanges,
+        this, &Folder::slotNextSyncFullLocalDiscovery);
+    connect(_folderWatcher.data(), &FolderWatcher::becameUnreliable,
+        this, &Folder::slotWatcherUnreliable);
+    _folderWatcher->init(path());
+}
+
 void Folder::slotAboutToRemoveAllFiles(SyncFileItem::Direction dir, bool *cancel)
 {
     ConfigFile cfgFile;
@@ -981,6 +1141,14 @@ void FolderDefinition::save(QSettings &settings, const FolderDefinition &folder)
     settings.setValue(QLatin1String("targetPath"), folder.targetPath);
     settings.setValue(QLatin1String("paused"), folder.paused);
     settings.setValue(QLatin1String("ignoreHiddenFiles"), folder.ignoreHiddenFiles);
+    settings.setValue(QLatin1String("usePlaceholders"), folder.useVirtualFiles);
+    settings.setValue(QLatin1String(versionC), 1);
+
+    // Happens only on Windows when the explorer integration is enabled.
+    if (!folder.navigationPaneClsid.isNull())
+        settings.setValue(QLatin1String("navigationPaneClsid"), folder.navigationPaneClsid);
+    else
+        settings.remove(QLatin1String("navigationPaneClsid"));
     settings.endGroup();
 }
 
@@ -994,6 +1162,8 @@ bool FolderDefinition::load(QSettings &settings, const QString &alias,
     folder->targetPath = settings.value(QLatin1String("targetPath")).toString();
     folder->paused = settings.value(QLatin1String("paused")).toBool();
     folder->ignoreHiddenFiles = settings.value(QLatin1String("ignoreHiddenFiles"), QVariant(true)).toBool();
+    folder->navigationPaneClsid = settings.value(QLatin1String("navigationPaneClsid")).toUuid();
+    folder->useVirtualFiles = settings.value(QLatin1String("usePlaceholders")).toBool();
     settings.endGroup();
 
     // Old settings can contain paths with native separators. In the rest of the
