@@ -37,6 +37,10 @@
 #include <QtCore>
 #include <QMutableSetIterator>
 #include <QSet>
+#include <QNetworkProxy>
+
+static const char versionC[] = "version";
+static const int maxFoldersVersion = 1;
 
 namespace OCC {
 
@@ -49,6 +53,7 @@ FolderMan::FolderMan(QObject *parent)
     , _currentSyncFolder(0)
     , _syncEnabled(true)
     , _lockWatcher(new LockWatcher)
+    , _navigationPaneHelper(this)
     , _appRestartRequired(false)
 {
     ASSERT(!_instance);
@@ -57,9 +62,9 @@ FolderMan::FolderMan(QObject *parent)
     _socketApi.reset(new SocketApi);
 
     ConfigFile cfg;
-    int polltime = cfg.remotePollInterval();
-    qCInfo(lcFolderMan) << "setting remote poll timer interval to" << polltime << "msec";
-    _etagPollTimer.setInterval(polltime);
+    std::chrono::milliseconds polltime = cfg.remotePollInterval();
+    qCInfo(lcFolderMan) << "setting remote poll timer interval to" << polltime.count() << "msec";
+    _etagPollTimer.setInterval(polltime.count());
     QObject::connect(&_etagPollTimer, &QTimer::timeout, this, &FolderMan::slotEtagPollTimerTimeout);
     _etagPollTimer.start();
 
@@ -104,9 +109,6 @@ void FolderMan::unloadFolder(Folder *f)
 
     _socketApi->slotUnregisterPath(f->alias());
 
-    if (_folderWatchers.contains(f->alias())) {
-        _folderWatchers.remove(f->alias());
-    }
     _folderMap.remove(f->alias());
 
     disconnect(f, &Folder::syncStarted,
@@ -147,58 +149,28 @@ int FolderMan::unloadAndDeleteAllFolders()
     return cnt;
 }
 
-// add a monitor to the local file system. If there is a change in the
-// file system, the method slotFolderMonitorFired is triggered through
-// the SignalMapper
-void FolderMan::registerFolderMonitor(Folder *folder)
+void FolderMan::registerFolderWithSocketApi(Folder *folder)
 {
     if (!folder)
         return;
     if (!QDir(folder->path()).exists())
         return;
 
-    if (!_folderWatchers.contains(folder->alias())) {
-        FolderWatcher *fw = new FolderWatcher(folder->path(), folder);
-
-        // Connect the pathChanged signal, which comes with the changed path,
-        // to the signal mapper which maps to the folder alias. The changed path
-        // is lost this way, but we do not need it for the current implementation.
-        connect(fw, &FolderWatcher::pathChanged, folder, &Folder::slotWatchedPathChanged);
-
-        _folderWatchers.insert(folder->alias(), fw);
-    }
-
     // register the folder with the socket API
     if (folder->canSync())
         _socketApi->slotRegisterPath(folder->alias());
 }
 
-void FolderMan::addMonitorPath(const QString &alias, const QString &path)
-{
-    if (!alias.isEmpty() && _folderWatchers.contains(alias)) {
-        FolderWatcher *fw = _folderWatchers[alias];
-
-        if (fw) {
-            fw->addPath(path);
-        }
-    }
-}
-
-void FolderMan::removeMonitorPath(const QString &alias, const QString &path)
-{
-    if (!alias.isEmpty() && _folderWatchers.contains(alias)) {
-        FolderWatcher *fw = _folderWatchers[alias];
-
-        if (fw) {
-            fw->removePath(path);
-        }
-    }
-}
-
 int FolderMan::setupFolders()
 {
     unloadAndDeleteAllFolders();
 
+    QStringList skipSettingsKeys, deleteSettingsKeys;
+    backwardMigrationSettingsKeys(&deleteSettingsKeys, &skipSettingsKeys);
+    // deleteKeys should already have been deleted on application startup.
+    // We ignore them here just in case.
+    skipSettingsKeys += deleteSettingsKeys;
+
     auto settings = ConfigFile::settingsWithGroup(QLatin1String("Accounts"));
     const auto accountsWithSettings = settings->childGroups();
     if (accountsWithSettings.isEmpty()) {
@@ -218,14 +190,24 @@ int FolderMan::setupFolders()
         }
         settings->beginGroup(id);
 
-        settings->beginGroup(QLatin1String("Folders"));
-        setupFoldersHelper(*settings, account, true);
-        settings->endGroup();
+        // The "backwardsCompatible" flag here is related to migrating old
+        // database locations
+        auto process = [&](const QString &groupName, bool backwardsCompatible, bool foldersWithPlaceholders) {
+            settings->beginGroup(groupName);
+            if (skipSettingsKeys.contains(settings->group())) {
+                // Should not happen: bad container keys should have been deleted
+                qCWarning(lcFolderMan) << "Folder structure" << groupName << "is too new, ignoring";
+            } else {
+                setupFoldersHelper(*settings, account, skipSettingsKeys, backwardsCompatible, foldersWithPlaceholders);
+            }
+            settings->endGroup();
+        };
 
-        // See Folder::saveToSettings for details about why this exists.
-        settings->beginGroup(QLatin1String("Multifolders"));
-        setupFoldersHelper(*settings, account, false);
-        settings->endGroup();
+        process(QStringLiteral("Folders"), true, false);
+
+        // See Folder::saveToSettings for details about why these exists.
+        process(QStringLiteral("Multifolders"), false, false);
+        process(QStringLiteral("FoldersWithPlaceholders"), false, true);
 
         settings->endGroup(); // <account>
     }
@@ -235,9 +217,19 @@ int FolderMan::setupFolders()
     return _folderMap.size();
 }
 
-void FolderMan::setupFoldersHelper(QSettings &settings, AccountStatePtr account, bool backwardsCompatible)
+void FolderMan::setupFoldersHelper(QSettings &settings, AccountStatePtr account, const QStringList &ignoreKeys, bool backwardsCompatible, bool foldersWithPlaceholders)
 {
     foreach (const auto &folderAlias, settings.childGroups()) {
+        // Skip folders with too-new version
+        settings.beginGroup(folderAlias);
+        if (ignoreKeys.contains(settings.group())) {
+            qCInfo(lcFolderMan) << "Folder" << folderAlias << "is too new, ignoring";
+            _additionalBlockedFolderAliases.insert(folderAlias);
+            settings.endGroup();
+            continue;
+        }
+        settings.endGroup();
+
         FolderDefinition folderDefinition;
         if (FolderDefinition::load(settings, folderAlias, &folderDefinition)) {
             auto defaultJournalPath = folderDefinition.defaultJournalPath(account->account());
@@ -264,9 +256,10 @@ void FolderMan::setupFoldersHelper(QSettings &settings, AccountStatePtr account,
             Folder *f = addFolderInternal(std::move(folderDefinition), account.data());
             if (f) {
                 // Migration: Mark folders that shall be saved in a backwards-compatible way
-                if (backwardsCompatible) {
+                if (backwardsCompatible)
                     f->setSaveBackwardsCompatible(true);
-                }
+                if (foldersWithPlaceholders)
+                    f->setSaveInFoldersWithPlaceholders();
                 scheduleFolder(f);
                 emit folderSyncStateChange(f);
             }
@@ -303,6 +296,37 @@ int FolderMan::setupFoldersMigration()
     return _folderMap.size();
 }
 
+void FolderMan::backwardMigrationSettingsKeys(QStringList *deleteKeys, QStringList *ignoreKeys)
+{
+    auto settings = ConfigFile::settingsWithGroup(QLatin1String("Accounts"));
+
+    auto processSubgroup = [&](const QString &name) {
+        settings->beginGroup(name);
+        const int foldersVersion = settings->value(QLatin1String(versionC), 1).toInt();
+        if (foldersVersion <= maxFoldersVersion) {
+            foreach (const auto &folderAlias, settings->childGroups()) {
+                settings->beginGroup(folderAlias);
+                const int folderVersion = settings->value(QLatin1String(versionC), 1).toInt();
+                if (folderVersion > FolderDefinition::maxSettingsVersion()) {
+                    ignoreKeys->append(settings->group());
+                }
+                settings->endGroup();
+            }
+        } else {
+            deleteKeys->append(settings->group());
+        }
+        settings->endGroup();
+    };
+
+    for (const auto &accountId : settings->childGroups()) {
+        settings->beginGroup(accountId);
+        processSubgroup("Folders");
+        processSubgroup("Multifolders");
+        processSubgroup("FoldersWithPlaceholders");
+        settings->endGroup();
+    }
+}
+
 bool FolderMan::ensureJournalGone(const QString &journalDbFile)
 {
     // remove the old journal file
@@ -690,13 +714,13 @@ void FolderMan::startScheduledSyncSoon()
 
     // Require a pause based on the duration of the last sync run.
     if (Folder *lastFolder = _lastSyncFolder) {
-        msSinceLastSync = lastFolder->msecSinceLastSync();
+        msSinceLastSync = lastFolder->msecSinceLastSync().count();
 
         //  1s   -> 1.5s pause
         // 10s   -> 5s pause
         //  1min -> 12s pause
         //  1h   -> 90s pause
-        qint64 pause = qSqrt(lastFolder->msecLastSyncDuration()) / 20.0 * 1000.0;
+        qint64 pause = qSqrt(lastFolder->msecLastSyncDuration().count()) / 20.0 * 1000.0;
         msDelay = qMax(msDelay, pause);
     }
 
@@ -750,7 +774,8 @@ void FolderMan::slotStartScheduledFolderSync()
     if (folder) {
         // Safe to call several times, and necessary to try again if
         // the folder path didn't exist previously.
-        registerFolderMonitor(folder);
+        folder->registerFolderWatcher();
+        registerFolderWithSocketApi(folder);
 
         _currentSyncFolder = folder;
         folder->startSync(QStringList());
@@ -760,7 +785,7 @@ void FolderMan::slotStartScheduledFolderSync()
 void FolderMan::slotEtagPollTimerTimeout()
 {
     ConfigFile cfg;
-    int polltime = cfg.remotePollInterval();
+    auto polltime = cfg.remotePollInterval();
 
     foreach (Folder *f, _folderMap) {
         if (!f) {
@@ -843,11 +868,10 @@ void FolderMan::slotScheduleFolderByTime()
         auto msecsSinceSync = f->msecSinceLastSync();
 
         // Possibly it's just time for a new sync run
-        bool forceSyncIntervalExpired =
-            quint64(msecsSinceSync) > ConfigFile().forceSyncInterval();
+        bool forceSyncIntervalExpired = msecsSinceSync > ConfigFile().forceSyncInterval();
         if (forceSyncIntervalExpired) {
             qCInfo(lcFolderMan) << "Scheduling folder" << f->alias()
-                                << "because it has been" << msecsSinceSync << "ms "
+                                << "because it has been" << msecsSinceSync.count() << "ms "
                                 << "since the last sync";
 
             scheduleFolder(f);
@@ -858,16 +882,15 @@ void FolderMan::slotScheduleFolderByTime()
         bool syncAgain =
             (f->consecutiveFailingSyncs() > 0 && f->consecutiveFailingSyncs() < 3)
             || f->syncEngine().isAnotherSyncNeeded() == DelayedFollowUp;
-        qint64 syncAgainDelay = 10 * 1000; // 10s for the first retry-after-fail
+        auto syncAgainDelay = std::chrono::seconds(10); // 10s for the first retry-after-fail
         if (f->consecutiveFailingSyncs() > 1)
-            syncAgainDelay = 60 * 1000; // 60s for each further attempt
-        if (syncAgain
-            && msecsSinceSync > syncAgainDelay) {
+            syncAgainDelay = std::chrono::seconds(60); // 60s for each further attempt
+        if (syncAgain && msecsSinceSync > syncAgainDelay) {
             qCInfo(lcFolderMan) << "Scheduling folder" << f->alias()
                                 << ", the last" << f->consecutiveFailingSyncs() << "syncs failed"
                                 << ", anotherSyncNeeded" << f->syncEngine().isAnotherSyncNeeded()
                                 << ", last status:" << f->syncResult().statusString()
-                                << ", time since last sync:" << msecsSinceSync;
+                                << ", time since last sync:" << msecsSinceSync.count();
 
             scheduleFolder(f);
             continue;
@@ -932,6 +955,9 @@ Folder *FolderMan::addFolder(AccountState *accountState, const FolderDefinition
         emit folderSyncStateChange(folder);
         emit folderListChanged(_folderMap);
     }
+
+    _navigationPaneHelper.scheduleUpdateCloudStorageRegistry();
+
     return folder;
 }
 
@@ -940,7 +966,9 @@ Folder *FolderMan::addFolderInternal(FolderDefinition folderDefinition,
 {
     auto alias = folderDefinition.alias;
     int count = 0;
-    while (folderDefinition.alias.isEmpty() || _folderMap.contains(folderDefinition.alias)) {
+    while (folderDefinition.alias.isEmpty()
+        || _folderMap.contains(folderDefinition.alias)
+        || _additionalBlockedFolderAliases.contains(folderDefinition.alias)) {
         // There is already a folder configured with this name and folder names need to be unique
         folderDefinition.alias = alias + QString::number(++count);
     }
@@ -964,11 +992,12 @@ Folder *FolderMan::addFolderInternal(FolderDefinition folderDefinition,
     connect(folder, &Folder::watchedFileChangedExternally,
         &folder->syncEngine().syncFileStatusTracker(), &SyncFileStatusTracker::slotPathTouched);
 
-    registerFolderMonitor(folder);
+    folder->registerFolderWatcher();
+    registerFolderWithSocketApi(folder);
     return folder;
 }
 
-Folder *FolderMan::folderForPath(const QString &path)
+Folder *FolderMan::folderForPath(const QString &path, QString *relativePath)
 {
     QString absolutePath = QDir::cleanPath(path) + QLatin1Char('/');
 
@@ -976,10 +1005,16 @@ Folder *FolderMan::folderForPath(const QString &path)
         const QString folderPath = folder->cleanPath() + QLatin1Char('/');
 
         if (absolutePath.startsWith(folderPath, (Utility::isWindows() || Utility::isMac()) ? Qt::CaseInsensitive : Qt::CaseSensitive)) {
+            if (relativePath) {
+                *relativePath = absolutePath.mid(folderPath.length());
+                relativePath->chop(1); // we added a '/' above
+            }
             return folder;
         }
     }
 
+    if (relativePath)
+        relativePath->clear();
     return 0;
 }
 
@@ -1040,6 +1075,8 @@ void FolderMan::removeFolder(Folder *f)
         delete f;
     }
 
+    _navigationPaneHelper.scheduleUpdateCloudStorageRegistry();
+
     emit folderListChanged(_folderMap);
 }
 
@@ -1049,7 +1086,7 @@ QString FolderMan::getBackupName(QString fullPathName) const
         fullPathName.chop(1);
 
     if (fullPathName.isEmpty())
-        return QString::null;
+        return QString();
 
     QString newName = fullPathName + tr(" (backup)");
     QFileInfo fi(newName);
@@ -1115,12 +1152,10 @@ bool FolderMan::startFromScratch(const QString &localFolder)
     return true;
 }
 
-void FolderMan::setDirtyProxy(bool value)
+void FolderMan::setDirtyProxy()
 {
     foreach (Folder *f, _folderMap.values()) {
         if (f) {
-            f->setProxyDirty(value);
-
             if (f->accountState() && f->accountState()->account()
                 && f->accountState()->account()->networkAccessManager()) {
                 // Need to do this so we do not use the old determined system proxy
@@ -1141,9 +1176,11 @@ void FolderMan::setDirtyNetworkLimits()
     }
 }
 
-SyncResult FolderMan::accountStatus(const QList<Folder *> &folders)
+void FolderMan::trayOverallStatus(const QList<Folder *> &folders,
+    SyncResult::Status *status, bool *unresolvedConflicts)
 {
-    SyncResult overallResult;
+    *status = SyncResult::Undefined;
+    *unresolvedConflicts = false;
 
     int cnt = folders.count();
 
@@ -1156,45 +1193,24 @@ SyncResult FolderMan::accountStatus(const QList<Folder *> &folders)
     if (cnt == 1) {
         Folder *folder = folders.at(0);
         if (folder) {
+            auto syncResult = folder->syncResult();
             if (folder->syncPaused()) {
-                overallResult.setStatus(SyncResult::Paused);
+                *status = SyncResult::Paused;
             } else {
-                SyncResult::Status syncStatus = folder->syncResult().status();
-
-
+                SyncResult::Status syncStatus = syncResult.status();
                 switch (syncStatus) {
                 case SyncResult::Undefined:
-                    overallResult.setStatus(SyncResult::Error);
-                    break;
-                case SyncResult::NotYetStarted:
-                    overallResult.setStatus(SyncResult::NotYetStarted);
-                    break;
-                case SyncResult::SyncPrepare:
-                    overallResult.setStatus(SyncResult::SyncPrepare);
-                    break;
-                case SyncResult::SyncRunning:
-                    overallResult.setStatus(SyncResult::SyncRunning);
+                    *status = SyncResult::Error;
                     break;
                 case SyncResult::Problem: // don't show the problem icon in tray.
-                case SyncResult::Success:
-                    if (overallResult.status() == SyncResult::Undefined)
-                        overallResult.setStatus(SyncResult::Success);
+                    *status = SyncResult::Success;
                     break;
-                case SyncResult::Error:
-                    overallResult.setStatus(SyncResult::Error);
-                    break;
-                case SyncResult::SetupError:
-                    if (overallResult.status() != SyncResult::Error)
-                        overallResult.setStatus(SyncResult::SetupError);
-                    break;
-                case SyncResult::SyncAbortRequested:
-                    overallResult.setStatus(SyncResult::SyncAbortRequested);
-                    break;
-                case SyncResult::Paused:
-                    overallResult.setStatus(SyncResult::Paused);
+                default:
+                    *status = syncStatus;
                     break;
                 }
             }
+            *unresolvedConflicts = syncResult.hasUnresolvedConflicts();
         }
     } else {
         int errorsSeen = 0;
@@ -1204,10 +1220,10 @@ SyncResult FolderMan::accountStatus(const QList<Folder *> &folders)
         int various = 0;
 
         foreach (Folder *folder, folders) {
+            SyncResult folderResult = folder->syncResult();
             if (folder->syncPaused()) {
                 abortOrPausedSeen++;
             } else {
-                SyncResult folderResult = folder->syncResult();
                 SyncResult::Status syncStatus = folderResult.status();
 
                 switch (syncStatus) {
@@ -1233,31 +1249,24 @@ SyncResult FolderMan::accountStatus(const QList<Folder *> &folders)
                     // no default case on purpose, check compiler warnings
                 }
             }
+            if (folderResult.hasUnresolvedConflicts())
+                *unresolvedConflicts = true;
         }
-        bool set = false;
         if (errorsSeen > 0) {
-            overallResult.setStatus(SyncResult::Error);
-            set = true;
-        }
-        if (!set && abortOrPausedSeen > 0 && abortOrPausedSeen == cnt) {
+            *status = SyncResult::Error;
+        } else if (abortOrPausedSeen > 0 && abortOrPausedSeen == cnt) {
             // only if all folders are paused
-            overallResult.setStatus(SyncResult::Paused);
-            set = true;
-        }
-        if (!set && runSeen > 0) {
-            overallResult.setStatus(SyncResult::SyncRunning);
-            set = true;
-        }
-        if (!set && goodSeen > 0) {
-            overallResult.setStatus(SyncResult::Success);
-            set = true;
+            *status = SyncResult::Paused;
+        } else if (runSeen > 0) {
+            *status = SyncResult::SyncRunning;
+        } else if (goodSeen > 0) {
+            *status = SyncResult::Success;
         }
     }
-
-    return overallResult;
 }
 
-QString FolderMan::statusToString(SyncResult::Status syncStatus, bool paused) const
+QString FolderMan::trayTooltipStatusString(
+    SyncResult::Status syncStatus, bool hasUnresolvedConflicts, bool paused)
 {
     QString folderMessage;
     switch (syncStatus) {
@@ -1274,13 +1283,15 @@ QString FolderMan::statusToString(SyncResult::Status syncStatus, bool paused) co
         folderMessage = tr("Sync is running.");
         break;
     case SyncResult::Success:
-        folderMessage = tr("Last Sync was successful.");
+    case SyncResult::Problem:
+        if (hasUnresolvedConflicts) {
+            folderMessage = tr("Sync was successful, unresolved conflicts.");
+        } else {
+            folderMessage = tr("Last Sync was successful.");
+        }
         break;
     case SyncResult::Error:
         break;
-    case SyncResult::Problem:
-        folderMessage = tr("Last Sync was successful, but with warnings on individual files.");
-        break;
     case SyncResult::SetupError:
         folderMessage = tr("Setup Error.");
         break;
@@ -1299,25 +1310,45 @@ QString FolderMan::statusToString(SyncResult::Status syncStatus, bool paused) co
     return folderMessage;
 }
 
-QString FolderMan::checkPathValidityForNewFolder(const QString &path, const QUrl &serverUrl, bool forNewDirectory) const
+static QString checkPathValidityRecursive(const QString &path)
 {
     if (path.isEmpty()) {
-        return tr("No valid folder selected!");
+        return FolderMan::tr("No valid folder selected!");
     }
 
     QFileInfo selFile(path);
 
     if (!selFile.exists()) {
-        return checkPathValidityForNewFolder(selFile.dir().path(), serverUrl, true);
+        return checkPathValidityRecursive(selFile.dir().path());
     }
 
     if (!selFile.isDir()) {
-        return tr("The selected path is not a folder!");
+        return FolderMan::tr("The selected path is not a folder!");
     }
 
     if (!selFile.isWritable()) {
-        return tr("You have no permission to write to the selected folder!");
+        return FolderMan::tr("You have no permission to write to the selected folder!");
     }
+    return QString();
+}
+
+// QFileInfo::canonicalPath returns an empty string if the file does not exist.
+// This function also works with files that does not exist and resolve the symlinks in the
+// parent directories.
+static QString canonicalPath(const QString &path)
+{
+    QFileInfo selFile(path);
+    if (!selFile.exists()) {
+        return canonicalPath(selFile.dir().path()) + '/' + selFile.fileName();
+    }
+    return selFile.canonicalFilePath();
+}
+
+QString FolderMan::checkPathValidityForNewFolder(const QString &path, const QUrl &serverUrl) const
+{
+    QString recursiveValidity = checkPathValidityRecursive(path);
+    if (!recursiveValidity.isEmpty())
+        return recursiveValidity;
 
     // check if the local directory isn't used yet in another ownCloud sync
     Qt::CaseSensitivity cs = Qt::CaseSensitive;
@@ -1325,57 +1356,28 @@ QString FolderMan::checkPathValidityForNewFolder(const QString &path, const QUrl
         cs = Qt::CaseInsensitive;
     }
 
+    const QString userDir = QDir::cleanPath(canonicalPath(path)) + '/';
     for (auto i = _folderMap.constBegin(); i != _folderMap.constEnd(); ++i) {
         Folder *f = static_cast<Folder *>(i.value());
-        QString folderDir = QDir(f->path()).canonicalPath();
-        if (folderDir.isEmpty()) {
-            continue;
-        }
-        if (!folderDir.endsWith(QLatin1Char('/'), cs))
-            folderDir.append(QLatin1Char('/'));
+        QString folderDir = QDir::cleanPath(canonicalPath(f->path())) + '/';
 
-        const QString folderDirClean = QDir::cleanPath(folderDir) + '/';
-        const QString userDirClean = QDir::cleanPath(path) + '/';
-
-        // folderDir follows sym links, path not.
-        bool differentPathes = !Utility::fileNamesEqual(QDir::cleanPath(folderDir), QDir::cleanPath(path));
-
-        if (!forNewDirectory && differentPathes && folderDirClean.startsWith(userDirClean, cs)) {
+        bool differentPaths = QString::compare(folderDir, userDir, cs) != 0;
+        if (differentPaths && folderDir.startsWith(userDir, cs)) {
             return tr("The local folder %1 already contains a folder used in a folder sync connection. "
                       "Please pick another one!")
                 .arg(QDir::toNativeSeparators(path));
         }
 
-        // QDir::cleanPath keeps links
-        // canonicalPath() remove symlinks and uses the symlink targets.
-        QString absCleanUserFolder = QDir::cleanPath(QDir(path).canonicalPath()) + '/';
-
-        if ((forNewDirectory || differentPathes) && userDirClean.startsWith(folderDirClean, cs)) {
+        if (differentPaths && userDir.startsWith(folderDir, cs)) {
             return tr("The local folder %1 is already contained in a folder used in a folder sync connection. "
                       "Please pick another one!")
                 .arg(QDir::toNativeSeparators(path));
         }
 
-        // both follow symlinks.
-        bool cleanUserEqualsCleanFolder = Utility::fileNamesEqual(absCleanUserFolder, folderDirClean);
-        if (differentPathes && absCleanUserFolder.startsWith(folderDirClean, cs) && !cleanUserEqualsCleanFolder) {
-            return tr("The local folder %1 is a symbolic link. "
-                      "The link target is already contained in a folder used in a folder sync connection. "
-                      "Please pick another one!")
-                .arg(QDir::toNativeSeparators(path));
-        }
-
-        if (differentPathes && folderDirClean.startsWith(absCleanUserFolder, cs) && !cleanUserEqualsCleanFolder && !forNewDirectory) {
-            return tr("The local folder %1 contains a symbolic link. "
-                      "The link target contains an already synced folder "
-                      "Please pick another one!")
-                .arg(QDir::toNativeSeparators(path));
-        }
-
         // if both pathes are equal, the server url needs to be different
         // otherwise it would mean that a new connection from the same local folder
         // to the same account is added which is not wanted. The account must differ.
-        if (serverUrl.isValid() && Utility::fileNamesEqual(absCleanUserFolder, folderDir)) {
+        if (serverUrl.isValid() && !differentPaths) {
             QUrl folderUrl = f->accountState()->account()->url();
             QString user = f->accountState()->account()->credentials()->user();
             folderUrl.setUserName(user);
