@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -22,7 +23,7 @@ class IgnoreListEditor;
 class SyncLogDialog;
 
 namespace Ui {
-class GeneralSettings;
+    class GeneralSettings;
 }
 
 /**
@@ -38,19 +39,24 @@ public:
     ~GeneralSettings();
     QSize sizeHint() const;
 
+signals:
+    void showAbout();
+
 private slots:
     void saveMiscSettings();
     void slotToggleLaunchOnStartup(bool);
     void slotToggleOptionalDesktopNotifications(bool);
+    void slotShowInExplorerNavigationPane(bool);
     void slotUpdateInfo();
+    void slotUpdateChannelChanged(int index);
     void slotIgnoreFilesEditor();
-
-private:
     void loadMiscSettings();
 
+private:
     Ui::GeneralSettings *_ui;
     QPointer<IgnoreListEditor> _ignoreEditor;
     QPointer<SyncLogDialog> _syncLogDialog;
+    bool _currentlyLoading;
 };
 
 
