@@ -15,6 +15,7 @@
 #ifndef LOGBROWSER_H
 #define LOGBROWSER_H
 
+#include <QCheckBox>
 #include <QPlainTextEdit>
 #include <QTextStream>
 #include <QFile>
@@ -32,14 +33,13 @@ namespace OCC {
  * @brief The LogWidget class
  * @ingroup gui
  */
-class LogWidget : public QPlainTextEdit 
+class LogWidget : public QPlainTextEdit
 {
     Q_OBJECT
 public:
     explicit LogWidget(QWidget *parent = 0);
 
 signals:
-
 };
 
 /**
@@ -48,30 +48,34 @@ signals:
  */
 class LogBrowser : public QDialog
 {
-  Q_OBJECT
+    Q_OBJECT
 public:
     explicit LogBrowser(QWidget *parent = 0);
     ~LogBrowser();
 
-    void setLogFile(const QString& , bool );
+    void setLogFile(const QString &, bool);
 
 protected:
+    void showEvent(QShowEvent *) Q_DECL_OVERRIDE;
     void closeEvent(QCloseEvent *) Q_DECL_OVERRIDE;
 
 protected slots:
-    void slotNewLog( const QString &msg );
+    void slotNewLog(const QString &msg);
     void slotFind();
-    void search( const QString& );
+    void slotDebugCheckStateChanged(int);
+    void search(const QString &);
     void slotSave();
     void slotClearLog();
+    void togglePermanentLogging(bool enabled);
 
 private:
     LogWidget *_logWidget;
     QLineEdit *_findTermEdit;
+    QCheckBox *_logDebugCheckBox;
+    QCheckBox *_permanentLogging;
     QPushButton *_saveBtn;
     QPushButton *_clearBtn;
-    QLabel      *_statusLabel;
-
+    QLabel *_statusLabel;
 };
 
 } // namespace
