@@ -20,8 +20,11 @@
 #include "application.h"
 #include "configfile.h"
 #include "folderman.h"
+#include "accountmanager.h"
 
 #include <QNetworkProxy>
+#include <QString>
+#include <QList>
 
 namespace OCC {
 
@@ -73,6 +76,10 @@ NetworkSettings::NetworkSettings(QWidget *parent)
     connect(_ui->autoDownloadLimitRadioButton, &QAbstractButton::clicked, this, &NetworkSettings::saveBWLimitSettings);
     connect(_ui->downloadSpinBox, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), this, &NetworkSettings::saveBWLimitSettings);
     connect(_ui->uploadSpinBox, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), this, &NetworkSettings::saveBWLimitSettings);
+
+    // Warn about empty proxy host
+    connect(_ui->hostLineEdit, &QLineEdit::textChanged, this, &NetworkSettings::checkEmptyProxyHost);
+    checkEmptyProxyHost();
 }
 
 NetworkSettings::~NetworkSettings()
@@ -150,12 +157,16 @@ void NetworkSettings::saveProxySettings()
 {
     ConfigFile cfgFile;
 
+    checkEmptyProxyHost();
     if (_ui->noProxyRadioButton->isChecked()) {
         cfgFile.setProxyType(QNetworkProxy::NoProxy);
     } else if (_ui->systemProxyRadioButton->isChecked()) {
         cfgFile.setProxyType(QNetworkProxy::DefaultProxy);
     } else if (_ui->manualProxyRadioButton->isChecked()) {
         int type = _ui->typeComboBox->itemData(_ui->typeComboBox->currentIndex()).toInt();
+        QString host = _ui->hostLineEdit->text();
+        if (host.isEmpty())
+            type = QNetworkProxy::NoProxy;
         bool needsAuth = _ui->authRequiredcheckBox->isChecked();
         QString user = _ui->userLineEdit->text();
         QString pass = _ui->passwordLineEdit->text();
@@ -169,7 +180,11 @@ void NetworkSettings::saveProxySettings()
 
     // ...and set the folders dirty, they refresh their proxy next time they
     // start the sync.
-    FolderMan::instance()->setDirtyProxy(true);
+    FolderMan::instance()->setDirtyProxy();
+
+    for (auto account : AccountManager::instance()->accounts()) {
+        account->freshConnectionAttempt();
+    }
 }
 
 void NetworkSettings::saveBWLimitSettings()
@@ -196,4 +211,26 @@ void NetworkSettings::saveBWLimitSettings()
     FolderMan::instance()->setDirtyNetworkLimits();
 }
 
+void NetworkSettings::checkEmptyProxyHost()
+{
+    if (_ui->hostLineEdit->isEnabled() && _ui->hostLineEdit->text().isEmpty()) {
+        _ui->hostLineEdit->setStyleSheet("border: 1px solid red");
+    } else {
+        _ui->hostLineEdit->setStyleSheet(QString());
+    }
+}
+
+void NetworkSettings::showEvent(QShowEvent *event)
+{
+    if (!event->spontaneous()
+        && _ui->manualProxyRadioButton->isChecked()
+        && _ui->hostLineEdit->text().isEmpty()) {
+        _ui->noProxyRadioButton->setChecked(true);
+        checkEmptyProxyHost();
+        saveProxySettings();
+    }
+
+    QWidget::showEvent(event);
+}
+
 } // namespace OCC
