@@ -35,12 +35,18 @@ class NetworkSettings : public QWidget
 public:
     explicit NetworkSettings(QWidget *parent = 0);
     ~NetworkSettings();
-    QSize sizeHint() const;
+    QSize sizeHint() const override;
 
 private slots:
     void saveProxySettings();
     void saveBWLimitSettings();
 
+    /// Red marking of host field if empty and enabled
+    void checkEmptyProxyHost();
+
+protected:
+    void showEvent(QShowEvent *event) override;
+
 private:
     void loadProxySettings();
     void loadBWLimitSettings();
