@@ -38,6 +38,8 @@
 #include <QDesktopServices>
 #include <QDir>
 #include <QMessageBox>
+#include <QDialog>
+#include <QHBoxLayout>
 
 #if defined(Q_OS_X11)
 #include <QX11Info>
@@ -50,18 +52,13 @@ const char propertyAccountC[] = "oc_account";
 ownCloudGui::ownCloudGui(Application *parent)
     : QObject(parent)
     , _tray(0)
-    ,
 #if defined(Q_OS_MAC)
-    _settingsDialog(new SettingsDialogMac(this))
-    ,
+    , _settingsDialog(new SettingsDialogMac(this))
 #else
-    _settingsDialog(new SettingsDialog(this))
-    ,
+    , _settingsDialog(new SettingsDialog(this))
 #endif
-    _logBrowser(0)
-    , _contextMenuVisibleOsx(false)
+    , _logBrowser(0)
     , _recentActionsMenu(0)
-    , _qdbusmenuWorkaround(false)
     , _app(parent)
 {
     _tray = new Systray();
@@ -117,7 +114,7 @@ void ownCloudGui::slotOpenSettingsDialog()
 
 void ownCloudGui::slotTrayClicked(QSystemTrayIcon::ActivationReason reason)
 {
-    if (_qdbusmenuWorkaround) {
+    if (_workaroundFakeDoubleClick) {
         static QElapsedTimer last_click;
         if (last_click.isValid() && last_click.elapsed() < 200) {
             return;
@@ -165,7 +162,10 @@ void ownCloudGui::slotSyncStateChange(Folder *folder)
 
     qCInfo(lcApplication) << "Sync state changed for folder " << folder->remoteUrl().toString() << ": " << result.statusString();
 
-    if (result.status() == SyncResult::Success || result.status() == SyncResult::Error) {
+    if (result.status() == SyncResult::Success
+        || result.status() == SyncResult::Problem
+        || result.status() == SyncResult::SyncAbortRequested
+        || result.status() == SyncResult::Error) {
         Logger::instance()->enterNextLogFile();
     }
 
@@ -196,7 +196,7 @@ void ownCloudGui::slotTrayMessageIfServerUnsupported(Account *account)
     if (account->serverVersionUnsupported()) {
         slotShowTrayMessage(
             tr("Unsupported Server Version"),
-            tr("The server on account %1 runs an old and unsupported version %2. "
+            tr("The server on account %1 runs an unsupported version %2. "
                "Using this client with unsupported server versions is untested and "
                "potentially dangerous. Proceed at your own risk.")
                 .arg(account->displayName(), account->serverVersion()));
@@ -278,38 +278,56 @@ void ownCloudGui::slotComputeOverallSyncStatus()
     QString trayMessage;
     FolderMan *folderMan = FolderMan::instance();
     Folder::Map map = folderMan->map();
-    SyncResult::Status overallResult = FolderMan::accountStatus(map.values()).status();
+
+    SyncResult::Status overallStatus = SyncResult::Undefined;
+    bool hasUnresolvedConflicts = false;
+    FolderMan::trayOverallStatus(map.values(), &overallStatus, &hasUnresolvedConflicts);
+
+    // If the sync succeeded but there are unresolved conflicts,
+    // show the problem icon!
+    auto iconStatus = overallStatus;
+    if (iconStatus == SyncResult::Success && hasUnresolvedConflicts) {
+        iconStatus = SyncResult::Problem;
+    }
+
+    // If we don't get a status for whatever reason, that's a Problem
+    if (iconStatus == SyncResult::Undefined) {
+        iconStatus = SyncResult::Problem;
+    }
+
+    QIcon statusIcon = Theme::instance()->syncStateIcon(iconStatus, true, contextMenuVisible());
+    _tray->setIcon(statusIcon);
 
     // create the tray blob message, check if we have an defined state
-    if (overallResult != SyncResult::Undefined && map.count() > 0) {
+    if (map.count() > 0) {
 #ifdef Q_OS_WIN
         // Windows has a 128-char tray tooltip length limit.
-        trayMessage = folderMan->statusToString(overallResult, false);
+        trayMessage = folderMan->trayTooltipStatusString(overallStatus, hasUnresolvedConflicts, false);
 #else
         QStringList allStatusStrings;
         foreach (Folder *folder, map.values()) {
-            QString folderMessage = folderMan->statusToString(folder->syncResult().status(), folder->syncPaused());
+            QString folderMessage = FolderMan::trayTooltipStatusString(
+                folder->syncResult().status(),
+                folder->syncResult().hasUnresolvedConflicts(),
+                folder->syncPaused());
             allStatusStrings += tr("Folder %1: %2").arg(folder->shortGuiLocalPath(), folderMessage);
         }
         trayMessage = allStatusStrings.join(QLatin1String("\n"));
 #endif
-
-        QIcon statusIcon = Theme::instance()->syncStateIcon(overallResult, true, contextMenuVisible());
-        _tray->setIcon(statusIcon);
         _tray->setToolTip(trayMessage);
 
-        if (overallResult == SyncResult::Success || overallResult == SyncResult::Problem) {
-            setStatusText(tr("Up to date"));
-        } else if (overallResult == SyncResult::Paused) {
+        if (overallStatus == SyncResult::Success || overallStatus == SyncResult::Problem) {
+            if (hasUnresolvedConflicts) {
+                setStatusText(tr("Unresolved conflicts"));
+            } else {
+                setStatusText(tr("Up to date"));
+            }
+        } else if (overallStatus == SyncResult::Paused) {
             setStatusText(tr("Synchronization is paused"));
         } else {
             setStatusText(tr("Error during synchronization"));
         }
     } else {
-        if (overallResult == SyncResult::Undefined)
-            overallResult = SyncResult::Problem;
-        QIcon icon = Theme::instance()->syncStateIcon(overallResult, true, contextMenuVisible());
-        _tray->setIcon(icon);
         _tray->setToolTip(tr("There are no sync folders configured."));
         setStatusText(tr("No sync folders configured"));
     }
@@ -381,17 +399,19 @@ void ownCloudGui::addAccountContextMenu(AccountStatePtr accountState, QMenu *men
 
 void ownCloudGui::slotContextMenuAboutToShow()
 {
-    // For some reason on OS X _contextMenu->isVisible returns always false
-    _contextMenuVisibleOsx = true;
+    _contextMenuVisibleManual = true;
 
     // Update icon in sys tray, as it might change depending on the context menu state
     slotComputeOverallSyncStatus();
+
+    if (!_workaroundNoAboutToShowUpdate) {
+        updateContextMenu();
+    }
 }
 
 void ownCloudGui::slotContextMenuAboutToHide()
 {
-    // For some reason on OS X _contextMenu->isVisible returns always false
-    _contextMenuVisibleOsx = false;
+    _contextMenuVisibleManual = false;
 
     // Update icon in sys tray, as it might change depending on the context menu state
     slotComputeOverallSyncStatus();
@@ -399,11 +419,11 @@ void ownCloudGui::slotContextMenuAboutToHide()
 
 bool ownCloudGui::contextMenuVisible() const
 {
-#ifdef Q_OS_MAC
-    return _contextMenuVisibleOsx;
-#else
+    // On some platforms isVisible doesn't work and always returns false,
+    // elsewhere aboutToHide is unreliable.
+    if (_workaroundManualVisibility)
+        return _contextMenuVisibleManual;
     return _contextMenu->isVisible();
-#endif
 }
 
 static bool minimalTrayMenu()
@@ -426,12 +446,36 @@ static bool updateWhileVisible()
     }
 }
 
-static QByteArray forceQDBusTrayWorkaround()
+static QByteArray envForceQDBusTrayWorkaround()
 {
     static QByteArray var = qgetenv("OWNCLOUD_FORCE_QDBUS_TRAY_WORKAROUND");
     return var;
 }
 
+static QByteArray envForceWorkaroundShowAndHideTray()
+{
+    static QByteArray var = qgetenv("OWNCLOUD_FORCE_TRAY_SHOW_HIDE");
+    return var;
+}
+
+static QByteArray envForceWorkaroundNoAboutToShowUpdate()
+{
+    static QByteArray var = qgetenv("OWNCLOUD_FORCE_TRAY_NO_ABOUT_TO_SHOW");
+    return var;
+}
+
+static QByteArray envForceWorkaroundFakeDoubleClick()
+{
+    static QByteArray var = qgetenv("OWNCLOUD_FORCE_TRAY_FAKE_DOUBLE_CLICK");
+    return var;
+}
+
+static QByteArray envForceWorkaroundManualVisibility()
+{
+    static QByteArray var = qgetenv("OWNCLOUD_FORCE_TRAY_MANUAL_VISIBILITY");
+    return var;
+}
+
 void ownCloudGui::setupContextMenu()
 {
     if (_contextMenu) {
@@ -450,55 +494,73 @@ void ownCloudGui::setupContextMenu()
     // The tray menu is surprisingly problematic. Being able to switch to
     // a minimal version of it is a useful workaround and testing tool.
     if (minimalTrayMenu()) {
+        if (! Theme::instance()->about().isEmpty()) {
+            _contextMenu->addSeparator();
+            _contextMenu->addAction(_actionAbout);
+        }
         _contextMenu->addAction(_actionQuit);
         return;
     }
 
-// Enables workarounds for bugs introduced in Qt 5.5.0
-// In particular QTBUG-47863 #3672 (tray menu fails to update and
-// becomes unresponsive) and QTBUG-48068 #3722 (click signal is
-// emitted several times)
-// The Qt version check intentionally uses 5.0.0 (where platformMenu()
-// was introduced) instead of 5.5.0 to avoid issues where the Qt
-// version used to build is different from the one used at runtime.
-// If we build with 5.6.1 or newer, we can skip this because the
-// bugs should be fixed there.
-#ifdef Q_OS_LINUX
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)) && (QT_VERSION < QT_VERSION_CHECK(5, 6, 0))
-    if (qVersion() == QByteArray("5.5.0")) {
-        QObject *platformMenu = reinterpret_cast<QObject *>(_tray->contextMenu()->platformMenu());
-        if (platformMenu
-            && platformMenu->metaObject()->className() == QLatin1String("QDBusPlatformMenu")) {
-            _qdbusmenuWorkaround = true;
-            qCWarning(lcApplication) << "Enabled QDBusPlatformMenu workaround";
-        }
-    }
-#endif
-#endif
+    auto applyEnvVariable = [](bool *sw, const QByteArray &value) {
+        if (value == "1")
+            *sw = true;
+        if (value == "0")
+            *sw = false;
+    };
 
-    if (forceQDBusTrayWorkaround() == "1") {
-        _qdbusmenuWorkaround = true;
-    } else if (forceQDBusTrayWorkaround() == "0") {
-        _qdbusmenuWorkaround = false;
+    // This is an old compound flag that people might still depend on
+    bool qdbusmenuWorkarounds = false;
+    applyEnvVariable(&qdbusmenuWorkarounds, envForceQDBusTrayWorkaround());
+    if (qdbusmenuWorkarounds) {
+        _workaroundFakeDoubleClick = true;
+        _workaroundNoAboutToShowUpdate = true;
+        _workaroundShowAndHideTray = true;
     }
 
-    // When the qdbusmenuWorkaround is necessary, we can't do on-demand updates
-    // because the workaround is to hide and show the tray icon.
-    if (_qdbusmenuWorkaround) {
-        connect(&_workaroundBatchTrayUpdate, &QTimer::timeout, this, &ownCloudGui::updateContextMenu);
-        _workaroundBatchTrayUpdate.setInterval(30 * 1000);
-        _workaroundBatchTrayUpdate.setSingleShot(true);
-    } else {
-// Update the context menu whenever we're about to show it
-// to the user.
 #ifdef Q_OS_MAC
-        // https://bugreports.qt.io/browse/QTBUG-54633
-        connect(_contextMenu.data(), SIGNAL(aboutToShow()), SLOT(slotContextMenuAboutToShow()));
-        connect(_contextMenu.data(), SIGNAL(aboutToHide()), SLOT(slotContextMenuAboutToHide()));
-#else
-        connect(_contextMenu.data(), &QMenu::aboutToShow, this, &ownCloudGui::updateContextMenu);
+    // https://bugreports.qt.io/browse/QTBUG-54633
+    _workaroundNoAboutToShowUpdate = true;
+    _workaroundManualVisibility = true;
 #endif
+
+#ifdef Q_OS_LINUX
+    // For KDE sessions if the platform plugin is missing,
+    // neither aboutToShow() updates nor the isVisible() call
+    // work. At least aboutToHide is reliable.
+    // https://github.com/owncloud/client/issues/6545
+    static QByteArray xdgCurrentDesktop = qgetenv("XDG_CURRENT_DESKTOP");
+    static QByteArray desktopSession = qgetenv("DESKTOP_SESSION");
+    bool isKde =
+        xdgCurrentDesktop.contains("KDE")
+        || desktopSession.contains("plasma")
+        || desktopSession.contains("kde");
+    QObject *platformMenu = reinterpret_cast<QObject *>(_tray->contextMenu()->platformMenu());
+    if (isKde && platformMenu && platformMenu->metaObject()->className() == QLatin1String("QDBusPlatformMenu")) {
+        _workaroundManualVisibility = true;
+        _workaroundNoAboutToShowUpdate = true;
     }
+#endif
+
+    applyEnvVariable(&_workaroundNoAboutToShowUpdate, envForceWorkaroundNoAboutToShowUpdate());
+    applyEnvVariable(&_workaroundFakeDoubleClick, envForceWorkaroundFakeDoubleClick());
+    applyEnvVariable(&_workaroundShowAndHideTray, envForceWorkaroundShowAndHideTray());
+    applyEnvVariable(&_workaroundManualVisibility, envForceWorkaroundManualVisibility());
+
+    qCInfo(lcApplication) << "Tray menu workarounds:"
+                          << "noabouttoshow:" << _workaroundNoAboutToShowUpdate
+                          << "fakedoubleclick:" << _workaroundFakeDoubleClick
+                          << "showhide:" << _workaroundShowAndHideTray
+                          << "manualvisibility:" << _workaroundManualVisibility;
+
+
+    connect(&_delayedTrayUpdateTimer, &QTimer::timeout, this, &ownCloudGui::updateContextMenu);
+    _delayedTrayUpdateTimer.setInterval(2 * 1000);
+    _delayedTrayUpdateTimer.setSingleShot(true);
+
+    connect(_contextMenu.data(), SIGNAL(aboutToShow()), SLOT(slotContextMenuAboutToShow()));
+    // unfortunately aboutToHide is unreliable, it seems to work on OSX though
+    connect(_contextMenu.data(), SIGNAL(aboutToHide()), SLOT(slotContextMenuAboutToHide()));
 
     // Populate the context menu now.
     updateContextMenu();
@@ -510,13 +572,21 @@ void ownCloudGui::updateContextMenu()
         return;
     }
 
-    if (_qdbusmenuWorkaround) {
+    // If it's visible, we can't update live, and it won't be updated lazily: reschedule
+    if (contextMenuVisible() && !updateWhileVisible() && _workaroundNoAboutToShowUpdate) {
+        if (!_delayedTrayUpdateTimer.isActive()) {
+            _delayedTrayUpdateTimer.start();
+        }
+        return;
+    }
+
+    if (_workaroundShowAndHideTray) {
         // To make tray menu updates work with these bugs (see setupContextMenu)
         // we need to hide and show the tray icon. We don't want to do that
         // while it's visible!
         if (contextMenuVisible()) {
-            if (!_workaroundBatchTrayUpdate.isActive()) {
-                _workaroundBatchTrayUpdate.start();
+            if (!_delayedTrayUpdateTimer.isActive()) {
+                _delayedTrayUpdateTimer.start();
             }
             return;
         }
@@ -629,37 +699,38 @@ void ownCloudGui::updateContextMenu()
         }
         _contextMenu->addAction(_actionLogin);
     }
+
+    if (! Theme::instance()->about().isEmpty()) {
+        _contextMenu->addSeparator();
+        _contextMenu->addAction(_actionAbout);
+    }
+
     _contextMenu->addAction(_actionQuit);
 
-    if (_qdbusmenuWorkaround) {
+    if (_workaroundShowAndHideTray) {
         _tray->show();
     }
 }
 
 void ownCloudGui::updateContextMenuNeeded()
 {
-    // For the workaround case updating while visible is impossible. Instead
-    // occasionally update the menu when it's invisible.
-    if (_qdbusmenuWorkaround) {
-        if (!_workaroundBatchTrayUpdate.isActive()) {
-            _workaroundBatchTrayUpdate.start();
-        }
+    // if it's visible and we can update live: update now
+    if (contextMenuVisible() && updateWhileVisible()) {
+        // Note: don't update while visible on OSX
+        // https://bugreports.qt.io/browse/QTBUG-54845
+        updateContextMenu();
         return;
     }
 
-#ifdef Q_OS_MAC
-    // https://bugreports.qt.io/browse/QTBUG-54845
-    // We cannot update on demand or while visible -> update when invisible.
-    if (!contextMenuVisible()) {
-        updateContextMenu();
+    // if we can't lazily update: update later
+    if (_workaroundNoAboutToShowUpdate) {
+        // Note: don't update immediately even in the invisible case
+        // as that can lead to extremely frequent menu updates
+        if (!_delayedTrayUpdateTimer.isActive()) {
+            _delayedTrayUpdateTimer.start();
+        }
+        return;
     }
-#else
-    if (updateWhileVisible() && contextMenuVisible())
-        updateContextMenu();
-#endif
-
-    // If no update was done here, we might update it on-demand due to
-    // the aboutToShow() signal.
 }
 
 void ownCloudGui::slotShowTrayMessage(const QString &title, const QString &msg)
@@ -716,6 +787,8 @@ void ownCloudGui::setupActions()
     QObject::connect(_actionNewAccountWizard, &QAction::triggered, this, &ownCloudGui::slotNewAccountWizard);
     _actionHelp = new QAction(tr("Help"), this);
     QObject::connect(_actionHelp, &QAction::triggered, this, &ownCloudGui::slotHelp);
+    _actionAbout = new QAction(tr("About %1").arg(Theme::instance()->appNameGUI()), this);
+    QObject::connect(_actionAbout, &QAction::triggered, this, &ownCloudGui::slotAbout);
     _actionQuit = new QAction(tr("Quit %1").arg(Theme::instance()->appNameGUI()), this);
     QObject::connect(_actionQuit, SIGNAL(triggered(bool)), _app, SLOT(quit()));
 
@@ -762,9 +835,12 @@ void ownCloudGui::slotUpdateProgress(const QString &folder, const ProgressInfo &
     Q_UNUSED(folder);
 
     if (progress.status() == ProgressInfo::Discovery) {
-        if (!progress._currentDiscoveredFolder.isEmpty()) {
-            _actionStatus->setText(tr("Checking for changes in '%1'")
-                                       .arg(progress._currentDiscoveredFolder));
+        if (!progress._currentDiscoveredRemoteFolder.isEmpty()) {
+            _actionStatus->setText(tr("Checking for changes in remote '%1'")
+                                       .arg(progress._currentDiscoveredRemoteFolder));
+        } else if (!progress._currentDiscoveredLocalFolder.isEmpty()) {
+            _actionStatus->setText(tr("Checking for changes in local '%1'")
+                                       .arg(progress._currentDiscoveredLocalFolder));
         }
     } else if (progress.status() == ProgressInfo::Done) {
         QTimer::singleShot(2000, this, &ownCloudGui::slotComputeOverallSyncStatus);
@@ -1008,20 +1084,17 @@ void ownCloudGui::raiseDialog(QWidget *raiseWidget)
 }
 
 
-void ownCloudGui::slotShowShareDialog(const QString &sharePath, const QString &localPath)
+void ownCloudGui::slotShowShareDialog(const QString &sharePath, const QString &localPath, ShareDialogStartPage startPage)
 {
-    const auto folder = FolderMan::instance()->folderForPath(localPath);
+    QString file;
+    const auto folder = FolderMan::instance()->folderForPath(localPath, &file);
     if (!folder) {
         qCWarning(lcApplication) << "Could not open share dialog for" << localPath << "no responsible folder found";
         return;
     }
 
-    // For https://github.com/owncloud/client/issues/3783
-    _settingsDialog->hide();
-
     const auto accountState = folder->accountState();
 
-    const QString file = localPath.mid(folder->cleanPath().length() + 1);
     SyncJournalFileRecord fileRecord;
 
     bool resharingAllowed = true; // lets assume the good
@@ -1052,7 +1125,7 @@ void ownCloudGui::slotShowShareDialog(const QString &sharePath, const QString &l
         w = _shareDialogs[localPath];
     } else {
         qCInfo(lcApplication) << "Opening share dialog" << sharePath << localPath << maxSharingPermissions;
-        w = new ShareDialog(accountState, sharePath, localPath, maxSharingPermissions, fileRecord.numericFileId());
+        w = new ShareDialog(accountState, sharePath, localPath, maxSharingPermissions, fileRecord.legacyDeriveNumericFileId(), startPage);
         w->setAttribute(Qt::WA_DeleteOnClose, true);
 
         _shareDialogs[localPath] = w;
@@ -1072,5 +1145,28 @@ void ownCloudGui::slotRemoveDestroyedShareDialogs()
     }
 }
 
+void ownCloudGui::slotAbout()
+{
+    QString title = tr("About %1").arg(Theme::instance()->appNameGUI());
+    QString about = Theme::instance()->about();
+    QMessageBox *msgBox = new QMessageBox(this->_settingsDialog);
+#ifdef Q_OS_MAC
+    // From Qt doc: "On macOS, the window title is ignored (as required by the macOS Guidelines)."
+    msgBox->setText(title);
+#else
+    msgBox->setWindowTitle(title);
+#endif
+    msgBox->setAttribute(Qt::WA_DeleteOnClose, true);
+    msgBox->setTextFormat(Qt::RichText);
+    msgBox->setTextInteractionFlags(Qt::TextBrowserInteraction);
+    msgBox->setInformativeText("<qt>"+about+"</qt>");
+    msgBox->setStandardButtons(QMessageBox::Ok);
+    QIcon appIcon = Theme::instance()->applicationIcon();
+    // Assume icon is always small enough to fit an about dialog?
+    qDebug() << appIcon.availableSizes().last();
+    msgBox->setIconPixmap(appIcon.pixmap(appIcon.availableSizes().last()));
+    msgBox->show();
+}
+
 
 } // end namespace
