@@ -57,7 +57,6 @@ OwncloudSetupWizard::OwncloudSetupWizard(QObject *parent)
     connect(_ocWizard, &OwncloudWizard::basicSetupFinished,
         this, &OwncloudSetupWizard::slotAssistantFinished, Qt::QueuedConnection);
     connect(_ocWizard, &QDialog::finished, this, &QObject::deleteLater);
-    connect(_ocWizard, &OwncloudWizard::skipFolderConfiguration, this, &OwncloudSetupWizard::slotSkipFolderConfiguration);
 }
 
 OwncloudSetupWizard::~OwncloudSetupWizard()
@@ -153,6 +152,8 @@ void OwncloudSetupWizard::slotCheckServer(const QString &urlString)
     // Reset the proxy which might had been determined previously in ConnectionValidator::checkServerAndAuth()
     // when there was a previous account.
     account->networkAccessManager()->setProxy(QNetworkProxy(QNetworkProxy::NoProxy));
+    // And also reset the QSslConfiguration, for the same reason (#6832)
+    account->setSslConfiguration({});
 
     // Lookup system proxy in a thread https://github.com/owncloud/client/issues/2993
     if (ClientProxy::isUsingSystemDefault()) {
@@ -597,15 +598,12 @@ bool OwncloudSetupWizard::ensureStartFromScratch(const QString &localFolder)
 // Method executed when the user end has finished the basic setup.
 void OwncloudSetupWizard::slotAssistantFinished(int result)
 {
-    FolderMan *folderMan = FolderMan::instance();
-
     if (result == QDialog::Rejected) {
-        qCInfo(lcWizard) << "Rejected the new config, use the old!";
-
-    } else if (result == QDialog::Accepted) {
-        // This may or may not wipe all folder definitions, depending
-        // on whether a new account is activated or the existing one
-        // is changed.
+        // Wizard was cancelled
+    } else if (_ocWizard->manualFolderConfig()) {
+        applyAccountChanges();
+    } else {
+        FolderMan *folderMan = FolderMan::instance();
         auto account = applyAccountChanges();
 
         QString localFolder = FolderDefinition::prepareLocalPath(_ocWizard->localFolder());
@@ -617,6 +615,9 @@ void OwncloudSetupWizard::slotAssistantFinished(int result)
             folderDefinition.localPath = localFolder;
             folderDefinition.targetPath = FolderDefinition::prepareTargetPath(_remoteFolder);
             folderDefinition.ignoreHiddenFiles = folderMan->ignoreHiddenFiles();
+            folderDefinition.useVirtualFiles = _ocWizard->useVirtualFileSync();
+            if (folderMan->navigationPaneHelper().showInExplorerNavigationPane())
+                folderDefinition.navigationPaneClsid = QUuid::createUuid();
 
             auto f = folderMan->addFolder(account, folderDefinition);
             if (f) {
@@ -636,16 +637,6 @@ void OwncloudSetupWizard::slotAssistantFinished(int result)
     emit ownCloudWizardDone(result);
 }
 
-void OwncloudSetupWizard::slotSkipFolderConfiguration()
-{
-    applyAccountChanges();
-
-    disconnect(_ocWizard, &OwncloudWizard::basicSetupFinished,
-        this, &OwncloudSetupWizard::slotAssistantFinished);
-    _ocWizard->close();
-    emit ownCloudWizardDone(QDialog::Accepted);
-}
-
 AccountState *OwncloudSetupWizard::applyAccountChanges()
 {
     AccountPtr newAccount = _ocWizard->account();
