@@ -35,16 +35,44 @@ namespace Ui {
 class Application;
 
 /**
- * A QTreeWidgetItem with special sorting.
+ * The items used in the protocol and issue QTreeWidget
  *
- * It allows items for global entries to be moved to the top if the
+ * Special sorting: It allows items for global entries to be moved to the top if the
  * sorting section is the "Time" column.
  */
-class SortedTreeWidgetItem : public QTreeWidgetItem
+class ProtocolItem : public QTreeWidgetItem
 {
 public:
     using QTreeWidgetItem::QTreeWidgetItem;
 
+    // Shared with IssueWidget
+    static ProtocolItem *create(const QString &folder, const SyncFileItem &item);
+    static QString timeString(QDateTime dt, QLocale::FormatType format = QLocale::NarrowFormat);
+
+    struct ExtraData
+    {
+        ExtraData()
+            : status(SyncFileItem::NoStatus)
+            , direction(SyncFileItem::None)
+        {
+        }
+
+        QString path;
+        QString folderName;
+        QDateTime timestamp;
+        quint64 size = 0;
+        SyncFileItem::Status status BITFIELD(4);
+        SyncFileItem::Direction direction BITFIELD(3);
+    };
+
+    static ExtraData extraData(const QTreeWidgetItem *item);
+    static void setExtraData(QTreeWidgetItem *item, const ExtraData &data);
+
+    static SyncJournalFileRecord syncJournalRecord(QTreeWidgetItem *item);
+    static Folder *folder(QTreeWidgetItem *item);
+
+    static void openContextMenu(QPoint globalPos, QTreeWidgetItem *item, QWidget *parent);
+
 private:
     bool operator<(const QTreeWidgetItem &other) const override;
 };
@@ -63,10 +91,6 @@ public:
 
     void storeSyncActivity(QTextStream &ts);
 
-    // Shared with IssueWidget
-    static QTreeWidgetItem *createCompletedTreewidgetItem(const QString &folder, const SyncFileItem &item);
-    static QString timeString(QDateTime dt, QLocale::FormatType format = QLocale::NarrowFormat);
-
 public slots:
     void slotItemCompleted(const QString &folder, const SyncFileItemPtr &item);
     void slotOpenFile(QTreeWidgetItem *item, int);
@@ -75,6 +99,9 @@ protected:
     void showEvent(QShowEvent *);
     void hideEvent(QHideEvent *);
 
+private slots:
+    void slotItemContextMenu(const QPoint &pos);
+
 signals:
     void copyToClipboard();
 
