@@ -83,8 +83,9 @@ ShareLinkWidget::ShareLinkWidget(AccountPtr account,
     connect(_ui->pushButton_setPassword, &QAbstractButton::clicked, this, &ShareLinkWidget::slotPasswordReturnPressed);
     connect(_ui->checkBox_expire, &QAbstractButton::clicked, this, &ShareLinkWidget::slotCheckBoxExpireClicked);
     connect(_ui->calendar, &QDateTimeEdit::dateChanged, this, &ShareLinkWidget::slotExpireDateChanged);
-    connect(_ui->checkBox_editing, &QAbstractButton::clicked, this, &ShareLinkWidget::slotPermissionsCheckboxClicked);
-    connect(_ui->checkBox_fileListing, &QAbstractButton::clicked, this, &ShareLinkWidget::slotPermissionsCheckboxClicked);
+    connect(_ui->radio_readOnly, &QAbstractButton::clicked, this, &ShareLinkWidget::slotPermissionsClicked);
+    connect(_ui->radio_readWrite, &QAbstractButton::clicked, this, &ShareLinkWidget::slotPermissionsClicked);
+    connect(_ui->radio_uploadOnly, &QAbstractButton::clicked, this, &ShareLinkWidget::slotPermissionsClicked);
 
     _ui->errorLabel->hide();
 
@@ -151,7 +152,7 @@ ShareLinkWidget::ShareLinkWidget(AccountPtr account,
     // File can't have public upload set; we also hide it if the capability isn't there
     _ui->widget_editing->setVisible(
         !_isFile && _account->capabilities().sharePublicLinkAllowUpload());
-    _ui->checkBox_fileListing->setVisible(
+    _ui->radio_uploadOnly->setVisible(
         _account->capabilities().sharePublicLinkSupportsUploadOnly());
 
 
@@ -196,20 +197,23 @@ void ShareLinkWidget::slotSharesFetched(const QList<QSharedPointer<Share>> &shar
     const QString versionString = _account->serverVersion();
     qCInfo(lcSharing) << versionString << "Fetched" << shares.count() << "shares";
 
-    // Preserve the previous selection
-    QString selectedShareId;
-    if (auto share = selectedShare()) {
-        selectedShareId = share->getId();
-    }
-    // ...except if selection should move to a new share
+    // Select the share that was previously selected,
+    // except if an explicit override was asked for
+    QString reselectShareId = _selectedShareId;
     if (!_newShareOverrideSelectionId.isEmpty()) {
-        selectedShareId = _newShareOverrideSelectionId;
+        reselectShareId = _newShareOverrideSelectionId;
         _newShareOverrideSelectionId.clear();
     }
 
     auto table = _ui->linkShares;
+
+    // Wipe the table without updating the ui elements, we
+    // might want their state untouched if the same share ends
+    // up being selected
+    disconnect(table, &QTableWidget::itemSelectionChanged, this, &ShareLinkWidget::slotShareSelectionChanged);
     table->clearContents();
     table->setRowCount(0);
+    connect(table, &QTableWidget::itemSelectionChanged, this, &ShareLinkWidget::slotShareSelectionChanged);
 
     auto deleteIcon = QIcon::fromTheme(QLatin1String("user-trash"),
         QIcon(QLatin1String(":/client/resources/delete.png")));
@@ -255,14 +259,20 @@ void ShareLinkWidget::slotSharesFetched(const QList<QSharedPointer<Share>> &shar
         table->setCellWidget(row, 2, deleteButton);
 
         // Reestablish the previous selection
-        if (selectedShareId == share->getId()) {
+        if (reselectShareId == share->getId()) {
             table->selectRow(row);
         }
     }
 
-    // Select the first share by default
-    if (!selectedShare() && table->rowCount() != 0) {
-        table->selectRow(0);
+    if (!selectedShare()) {
+        if (table->rowCount() != 0) {
+            // Select the first share by default
+            table->selectRow(0);
+        } else {
+            // explicitly note the deselection,
+            // since this was not triggered on table clear above
+            slotShareSelectionChanged();
+        }
     }
 
     if (!_namesSupported) {
@@ -282,33 +292,41 @@ void ShareLinkWidget::slotShareSelectionChanged()
 
     auto share = selectedShare();
     if (!share) {
+        _selectedShareId.clear();
         _ui->shareProperties->setEnabled(false);
-        _ui->checkBox_editing->setChecked(false);
+        _ui->radio_readOnly->setChecked(false);
+        _ui->radio_readWrite->setChecked(false);
+        _ui->radio_uploadOnly->setChecked(false);
         _ui->checkBox_expire->setChecked(false);
         _ui->checkBox_password->setChecked(false);
         return;
     }
+    bool selectionUnchanged = _selectedShareId == share->getId();
+    _selectedShareId = share->getId();
 
     _ui->shareProperties->setEnabled(true);
 
     _ui->checkBox_password->setEnabled(!_passwordRequired);
     _ui->checkBox_expire->setEnabled(!_expiryRequired);
-    _ui->checkBox_editing->setEnabled(
-        _account->capabilities().sharePublicLinkAllowUpload());
+    _ui->widget_editing->setEnabled(true);
+    if (!_account->capabilities().sharePublicLinkAllowUpload()) {
+        _ui->radio_readWrite->setEnabled(false);
+        _ui->radio_uploadOnly->setEnabled(false);
+    }
 
     // Password state
     _ui->checkBox_password->setText(tr("P&assword protect"));
-    if (share->isPasswordSet()) {
-        _ui->checkBox_password->setChecked(true);
-        _ui->lineEdit_password->setEnabled(true);
-        _ui->lineEdit_password->setPlaceholderText("********");
+    if (!selectionUnchanged) {
+        if (share->isPasswordSet()) {
+            _ui->checkBox_password->setChecked(true);
+            _ui->lineEdit_password->setPlaceholderText("********");
+            _ui->lineEdit_password->setEnabled(true);
+        } else {
+            _ui->checkBox_password->setChecked(false);
+            _ui->lineEdit_password->setPlaceholderText(QString());
+            _ui->lineEdit_password->setEnabled(false);
+        }
         _ui->lineEdit_password->setText(QString());
-        _ui->lineEdit_password->setEnabled(true);
-        _ui->pushButton_setPassword->setEnabled(false);
-    } else {
-        _ui->checkBox_password->setChecked(false);
-        _ui->lineEdit_password->setPlaceholderText(QString());
-        _ui->lineEdit_password->setEnabled(false);
         _ui->pushButton_setPassword->setEnabled(false);
     }
 
@@ -325,9 +343,15 @@ void ShareLinkWidget::slotShareSelectionChanged()
 
     // Public upload state (box is hidden for files)
     if (!_isFile) {
-        _ui->checkBox_editing->setChecked(share->getPublicUpload());
-        _ui->checkBox_fileListing->setChecked(share->getShowFileListing());
-        _ui->checkBox_fileListing->setEnabled(share->getPublicUpload());
+        if (share->getPublicUpload()) {
+            if (share->getShowFileListing()) {
+                _ui->radio_readWrite->setChecked(true);
+            } else {
+                _ui->radio_uploadOnly->setChecked(true);
+            }
+        } else {
+            _ui->radio_readOnly->setChecked(true);
+        }
     }
 }
 
@@ -408,14 +432,27 @@ void ShareLinkWidget::setPassword(const QString &password)
 
 void ShareLinkWidget::slotPasswordSet()
 {
+    auto share = selectedShare();
+    if (sender() != share.data())
+        return;
+
     _pi_password->stopAnimation();
+    _ui->checkBox_password->setEnabled(true);
     _ui->lineEdit_password->setText(QString());
-    _ui->lineEdit_password->setPlaceholderText(tr("Password Protected"));
+    if (share->isPasswordSet()) {
+        _ui->lineEdit_password->setPlaceholderText("********");
+        _ui->lineEdit_password->setEnabled(true);
+    } else {
+        _ui->lineEdit_password->setPlaceholderText(QString());
+        _ui->lineEdit_password->setEnabled(false);
+    }
 
     /*
      * When setting/deleting a password from a share the old share is
      * deleted and a new one is created. So we need to refetch the shares
      * at this point.
+     *
+     * NOTE: I don't see this happening with oC > 10
      */
     getShares();
 }
@@ -457,7 +494,7 @@ void ShareLinkWidget::slotCreateShareRequiresPassword(const QString &message)
     _ui->checkBox_password->setEnabled(false);
     _ui->checkBox_password->setText(tr("Public sh&aring requires a password"));
     _ui->checkBox_expire->setEnabled(false);
-    _ui->checkBox_editing->setEnabled(false);
+    _ui->widget_editing->setEnabled(false);
     if (!message.isEmpty()) {
         _ui->errorLabel->setText(message);
         _ui->errorLabel->show();
@@ -502,7 +539,7 @@ void ShareLinkWidget::emailShareLink(const QUrl &url)
 {
     QString fileName = _sharePath.mid(_sharePath.lastIndexOf('/') + 1);
     Utility::openEmailComposer(
-        QString("I shared %1 with you").arg(fileName),
+        tr("I shared %1 with you").arg(fileName),
         url.toString(),
         this);
 }
@@ -580,19 +617,18 @@ void ShareLinkWidget::slotDeleteShareClicked()
     confirmAndDeleteShare(share);
 }
 
-void ShareLinkWidget::slotPermissionsCheckboxClicked()
+void ShareLinkWidget::slotPermissionsClicked()
 {
     if (auto current = selectedShare()) {
-        _ui->checkBox_editing->setEnabled(false);
-        _ui->checkBox_fileListing->setEnabled(false);
+        _ui->widget_editing->setEnabled(false);
         _pi_editing->startAnimation();
         _ui->errorLabel->hide();
 
         SharePermissions perm = SharePermissionRead;
-        if (_ui->checkBox_editing->isChecked() && _ui->checkBox_fileListing->isChecked()) {
+        if (_ui->radio_readWrite->isChecked()) {
             perm = SharePermissionRead | SharePermissionCreate
                 | SharePermissionUpdate | SharePermissionDelete;
-        } else if (_ui->checkBox_editing->isChecked() && !_ui->checkBox_fileListing->isChecked()) {
+        } else if (_ui->radio_uploadOnly->isChecked()) {
             perm = SharePermissionCreate;
         }
         current->setPermissions(perm);
