@@ -23,15 +23,20 @@ namespace OCC {
 
 class SparkleUpdater : public Updater
 {
+    Q_OBJECT
 public:
-    SparkleUpdater(const QString &appCastUrl);
+    SparkleUpdater(const QUrl &appCastUrl);
     ~SparkleUpdater();
 
+    void setUpdateUrl(const QUrl &url);
+
     // unused in this updater
     void checkForUpdate() Q_DECL_OVERRIDE;
     void backgroundCheckForUpdate() Q_DECL_OVERRIDE;
     bool handleStartup() Q_DECL_OVERRIDE { return false; }
 
+    QString statusString();
+
 private:
     class Private;
     Private *d;
