@@ -16,22 +16,25 @@
 #include "propagatedownload.h"
 #include "propagateupload.h"
 #include "propagatorjobs.h"
-#include "utility.h"
+#include "common/utility.h"
 
 #ifdef Q_OS_WIN
 #include <windef.h>
 #include <winbase.h>
 #endif
 
+#include <QLoggingCategory>
 #include <QTimer>
 #include <QObject>
 
 namespace OCC {
 
+Q_LOGGING_CATEGORY(lcBandwidthManager, "sync.bandwidthmanager", QtInfoMsg)
+
 // Because of the many layers of buffering inside Qt (and probably the OS and the network)
 // we cannot lower this value much more. If we do, the estimated bw will be very high
 // because the buffers fill fast while the actual network algorithms are not relevant yet.
-static qint64 relativeLimitMeasuringTimerIntervalMsec = 1000*2;
+static qint64 relativeLimitMeasuringTimerIntervalMsec = 1000 * 2;
 // See also WritingState in http://code.woboq.org/qt5/qtbase/src/network/access/qhttpprotocolhandler.cpp.html#_ZN20QHttpProtocolHandler11sendRequestEv
 
 // FIXME At some point:
@@ -41,59 +44,58 @@ static qint64 relativeLimitMeasuringTimerIntervalMsec = 1000*2;
 //  * For relative limiting, do less measuring and more delaying+giving quota
 //  * For relative limiting, smoothen measurements
 
-BandwidthManager::BandwidthManager(OwncloudPropagator *p) : QObject(),
-    _propagator(p),
-    _relativeLimitCurrentMeasuredDevice(0),
-    _relativeUploadLimitProgressAtMeasuringRestart(0),
-    _currentUploadLimit(0),
-    _relativeLimitCurrentMeasuredJob(0),
-    _currentDownloadLimit(0)
+BandwidthManager::BandwidthManager(OwncloudPropagator *p)
+    : QObject()
+    , _propagator(p)
+    , _relativeLimitCurrentMeasuredDevice(0)
+    , _relativeUploadLimitProgressAtMeasuringRestart(0)
+    , _currentUploadLimit(0)
+    , _relativeLimitCurrentMeasuredJob(0)
+    , _currentDownloadLimit(0)
 {
     _currentUploadLimit = _propagator->_uploadLimit.fetchAndAddAcquire(0);
     _currentDownloadLimit = _propagator->_downloadLimit.fetchAndAddAcquire(0);
 
-    QObject::connect(&_switchingTimer, SIGNAL(timeout()), this, SLOT(switchingTimerExpired()));
-    _switchingTimer.setInterval(10*1000);
+    QObject::connect(&_switchingTimer, &QTimer::timeout, this, &BandwidthManager::switchingTimerExpired);
+    _switchingTimer.setInterval(10 * 1000);
     _switchingTimer.start();
     QMetaObject::invokeMethod(this, "switchingTimerExpired", Qt::QueuedConnection);
 
     // absolute uploads/downloads
-    QObject::connect(&_absoluteLimitTimer, SIGNAL(timeout()), this, SLOT(absoluteLimitTimerExpired()));
+    QObject::connect(&_absoluteLimitTimer, &QTimer::timeout, this, &BandwidthManager::absoluteLimitTimerExpired);
     _absoluteLimitTimer.setInterval(1000);
     _absoluteLimitTimer.start();
 
     // Relative uploads
-    QObject::connect(&_relativeUploadMeasuringTimer,SIGNAL(timeout()),
-                     this, SLOT(relativeUploadMeasuringTimerExpired()));
+    QObject::connect(&_relativeUploadMeasuringTimer, &QTimer::timeout,
+        this, &BandwidthManager::relativeUploadMeasuringTimerExpired);
     _relativeUploadMeasuringTimer.setInterval(relativeLimitMeasuringTimerIntervalMsec);
     _relativeUploadMeasuringTimer.start();
     _relativeUploadMeasuringTimer.setSingleShot(true); // will be restarted from the delay timer
-    QObject::connect(&_relativeUploadDelayTimer, SIGNAL(timeout()),
-                     this, SLOT(relativeUploadDelayTimerExpired()));
+    QObject::connect(&_relativeUploadDelayTimer, &QTimer::timeout,
+        this, &BandwidthManager::relativeUploadDelayTimerExpired);
     _relativeUploadDelayTimer.setSingleShot(true); // will be restarted from the measuring timer
 
     // Relative downloads
-    QObject::connect(&_relativeDownloadMeasuringTimer,SIGNAL(timeout()),
-                     this, SLOT(relativeDownloadMeasuringTimerExpired()));
+    QObject::connect(&_relativeDownloadMeasuringTimer, &QTimer::timeout,
+        this, &BandwidthManager::relativeDownloadMeasuringTimerExpired);
     _relativeDownloadMeasuringTimer.setInterval(relativeLimitMeasuringTimerIntervalMsec);
     _relativeDownloadMeasuringTimer.start();
     _relativeDownloadMeasuringTimer.setSingleShot(true); // will be restarted from the delay timer
-    QObject::connect(&_relativeDownloadDelayTimer, SIGNAL(timeout()),
-                     this, SLOT(relativeDownloadDelayTimerExpired()));
+    QObject::connect(&_relativeDownloadDelayTimer, &QTimer::timeout,
+        this, &BandwidthManager::relativeDownloadDelayTimerExpired);
     _relativeDownloadDelayTimer.setSingleShot(true); // will be restarted from the measuring timer
 }
 
 BandwidthManager::~BandwidthManager()
 {
-    qDebug() << Q_FUNC_INFO;
 }
 
 void BandwidthManager::registerUploadDevice(UploadDevice *p)
 {
-    //qDebug() << Q_FUNC_INFO << p;
     _absoluteUploadDeviceList.append(p);
     _relativeUploadDeviceList.append(p);
-    QObject::connect(p, SIGNAL(destroyed(QObject*)), this, SLOT(unregisterUploadDevice(QObject*)));
+    QObject::connect(p, &QObject::destroyed, this, &BandwidthManager::unregisterUploadDevice);
 
     if (usingAbsoluteUploadLimit()) {
         p->setBandwidthLimited(true);
@@ -109,15 +111,7 @@ void BandwidthManager::registerUploadDevice(UploadDevice *p)
 
 void BandwidthManager::unregisterUploadDevice(QObject *o)
 {
-    UploadDevice *p = qobject_cast<UploadDevice*>(o);
-    if (p) {
-        unregisterUploadDevice(p);
-    }
-}
-
-void BandwidthManager::unregisterUploadDevice(UploadDevice* p)
-{
-    //qDebug() << Q_FUNC_INFO << p;
+    auto p = reinterpret_cast<UploadDevice *>(o); // note, we might already be in the ~QObject
     _absoluteUploadDeviceList.removeAll(p);
     _relativeUploadDeviceList.removeAll(p);
     if (p == _relativeLimitCurrentMeasuredDevice) {
@@ -126,11 +120,10 @@ void BandwidthManager::unregisterUploadDevice(UploadDevice* p)
     }
 }
 
-void BandwidthManager::registerDownloadJob(GETFileJob* j)
+void BandwidthManager::registerDownloadJob(GETFileJob *j)
 {
-    //qDebug() << Q_FUNC_INFO << j;
     _downloadJobList.append(j);
-    QObject::connect(j, SIGNAL(destroyed(QObject*)), this, SLOT(unregisterDownloadJob(QObject*)));
+    QObject::connect(j, &QObject::destroyed, this, &BandwidthManager::unregisterDownloadJob);
 
     if (usingAbsoluteDownloadLimit()) {
         j->setBandwidthLimited(true);
@@ -144,8 +137,9 @@ void BandwidthManager::registerDownloadJob(GETFileJob* j)
     }
 }
 
-void BandwidthManager::unregisterDownloadJob(GETFileJob* j)
+void BandwidthManager::unregisterDownloadJob(QObject *o)
 {
+    GETFileJob *j = reinterpret_cast<GETFileJob *>(o); // note, we might already be in the ~QObject
     _downloadJobList.removeAll(j);
     if (_relativeLimitCurrentMeasuredJob == j) {
         _relativeLimitCurrentMeasuredJob = 0;
@@ -153,14 +147,6 @@ void BandwidthManager::unregisterDownloadJob(GETFileJob* j)
     }
 }
 
-void BandwidthManager::unregisterDownloadJob(QObject* o)
-{
-    GETFileJob *p = qobject_cast<GETFileJob*>(o);
-    if (p) {
-        unregisterDownloadJob(p);
-    }
-}
-
 void BandwidthManager::relativeUploadMeasuringTimerExpired()
 {
     if (!usingRelativeUploadLimit() || _relativeUploadDeviceList.count() == 0) {
@@ -170,25 +156,27 @@ void BandwidthManager::relativeUploadMeasuringTimerExpired()
         return;
     }
     if (_relativeLimitCurrentMeasuredDevice == 0) {
-        qDebug() << Q_FUNC_INFO << "No device set, just waiting 1 sec";
+        qCDebug(lcBandwidthManager) << "No device set, just waiting 1 sec";
         _relativeUploadDelayTimer.setInterval(1000);
         _relativeUploadDelayTimer.start();
         return;
     }
 
-//    qDebug() << Q_FUNC_INFO << _relativeUploadDeviceList.count() << "Starting Delay";
+    qCDebug(lcBandwidthManager) << _relativeUploadDeviceList.count() << "Starting Delay";
 
     qint64 relativeLimitProgressMeasured = (_relativeLimitCurrentMeasuredDevice->_readWithProgress
-                                            + _relativeLimitCurrentMeasuredDevice->_read) / 2;
+                                               + _relativeLimitCurrentMeasuredDevice->_read)
+        / 2;
     qint64 relativeLimitProgressDifference = relativeLimitProgressMeasured - _relativeUploadLimitProgressAtMeasuringRestart;
-//    qDebug() << Q_FUNC_INFO << _relativeUploadLimitProgressAtMeasuringRestart
-//             << relativeLimitProgressMeasured << relativeLimitProgressDifference;
+    qCDebug(lcBandwidthManager) << _relativeUploadLimitProgressAtMeasuringRestart
+                                << relativeLimitProgressMeasured << relativeLimitProgressDifference;
 
-//    qint64 speedkBPerSec = (relativeLimitProgressDifference / relativeLimitMeasuringTimerIntervalMsec*1000.0) / 1024.0;
-//    qDebug() << Q_FUNC_INFO << relativeLimitProgressDifference/1024 <<"kB =>" << speedkBPerSec << "kB/sec on full speed ("
-//             << _relativeLimitCurrentMeasuredDevice->_readWithProgress << _relativeLimitCurrentMeasuredDevice->_read
-//             << qAbs(_relativeLimitCurrentMeasuredDevice->_readWithProgress
-//                     - _relativeLimitCurrentMeasuredDevice->_read) << ")";
+    qint64 speedkBPerSec = (relativeLimitProgressDifference / relativeLimitMeasuringTimerIntervalMsec * 1000.0) / 1024.0;
+    qCDebug(lcBandwidthManager) << relativeLimitProgressDifference / 1024 << "kB =>" << speedkBPerSec << "kB/sec on full speed ("
+                                << _relativeLimitCurrentMeasuredDevice->_readWithProgress << _relativeLimitCurrentMeasuredDevice->_read
+                                << qAbs(_relativeLimitCurrentMeasuredDevice->_readWithProgress
+                                       - _relativeLimitCurrentMeasuredDevice->_read)
+                                << ")";
 
     qint64 uploadLimitPercent = -_currentUploadLimit;
     // don't use too extreme values
@@ -197,9 +185,7 @@ void BandwidthManager::relativeUploadMeasuringTimerExpired()
     qint64 wholeTimeMsec = (100.0 / uploadLimitPercent) * relativeLimitMeasuringTimerIntervalMsec;
     qint64 waitTimeMsec = wholeTimeMsec - relativeLimitMeasuringTimerIntervalMsec;
     qint64 realWaitTimeMsec = waitTimeMsec + wholeTimeMsec;
-//    qDebug() << Q_FUNC_INFO << waitTimeMsec << " - "<< realWaitTimeMsec <<
-//                " msec for " << uploadLimitPercent << "%";
-//    qDebug() << Q_FUNC_INFO << "XXXX" << uploadLimitPercent << relativeLimitMeasuringTimerIntervalMsec;
+    qCDebug(lcBandwidthManager) << waitTimeMsec << " - " << realWaitTimeMsec << " msec for " << uploadLimitPercent << "%";
 
     // We want to wait twice as long since we want to give all
     // devices the same quota we used now since we don't want
@@ -209,12 +195,11 @@ void BandwidthManager::relativeUploadMeasuringTimerExpired()
 
     int deviceCount = _relativeUploadDeviceList.count();
     qint64 quotaPerDevice = relativeLimitProgressDifference * (uploadLimitPercent / 100.0) / deviceCount + 1.0;
-//    qDebug() << Q_FUNC_INFO << "YYYY" << relativeLimitProgressDifference << uploadLimitPercent << deviceCount;
-    Q_FOREACH(UploadDevice *ud, _relativeUploadDeviceList) {
+    Q_FOREACH (UploadDevice *ud, _relativeUploadDeviceList) {
         ud->setBandwidthLimited(true);
         ud->setChoked(false);
         ud->giveBandwidthQuota(quotaPerDevice);
-//        qDebug() << Q_FUNC_INFO << "Gave" << quotaPerDevice/1024.0 << "kB to" << ud;
+        qCDebug(lcBandwidthManager) << "Gave" << quotaPerDevice / 1024.0 << "kB to" << ud;
     }
     _relativeLimitCurrentMeasuredDevice = 0;
 }
@@ -232,19 +217,20 @@ void BandwidthManager::relativeUploadDelayTimerExpired()
         return;
     }
 
-//    qDebug() << Q_FUNC_INFO << _relativeUploadDeviceList.count() << "Starting measuring";
+    qCDebug(lcBandwidthManager) << _relativeUploadDeviceList.count() << "Starting measuring";
 
     // Take first device and then append it again (= we round robin all devices)
     _relativeLimitCurrentMeasuredDevice = _relativeUploadDeviceList.takeFirst();
     _relativeUploadDeviceList.append(_relativeLimitCurrentMeasuredDevice);
 
     _relativeUploadLimitProgressAtMeasuringRestart = (_relativeLimitCurrentMeasuredDevice->_readWithProgress
-                                                      + _relativeLimitCurrentMeasuredDevice->_read) / 2;
+                                                         + _relativeLimitCurrentMeasuredDevice->_read)
+        / 2;
     _relativeLimitCurrentMeasuredDevice->setBandwidthLimited(false);
     _relativeLimitCurrentMeasuredDevice->setChoked(false);
 
     // choke all other UploadDevices
-    Q_FOREACH(UploadDevice *ud, _relativeUploadDeviceList) {
+    Q_FOREACH (UploadDevice *ud, _relativeUploadDeviceList) {
         if (ud != _relativeLimitCurrentMeasuredDevice) {
             ud->setBandwidthLimited(true);
             ud->setChoked(true);
@@ -264,22 +250,22 @@ void BandwidthManager::relativeDownloadMeasuringTimerExpired()
         return;
     }
     if (_relativeLimitCurrentMeasuredJob == 0) {
-        qDebug() << Q_FUNC_INFO << "No job set, just waiting 1 sec";
+        qCDebug(lcBandwidthManager) << "No job set, just waiting 1 sec";
         _relativeDownloadDelayTimer.setInterval(1000);
         _relativeDownloadDelayTimer.start();
         return;
     }
 
-//    qDebug() << Q_FUNC_INFO << _downloadJobList.count() << "Starting Delay";
+    qCDebug(lcBandwidthManager) << _downloadJobList.count() << "Starting Delay";
 
     qint64 relativeLimitProgressMeasured = _relativeLimitCurrentMeasuredJob->currentDownloadPosition();
     qint64 relativeLimitProgressDifference = relativeLimitProgressMeasured - _relativeDownloadLimitProgressAtMeasuringRestart;
-    qDebug() << Q_FUNC_INFO << _relativeDownloadLimitProgressAtMeasuringRestart
-             << relativeLimitProgressMeasured << relativeLimitProgressDifference;
+    qCDebug(lcBandwidthManager) << _relativeDownloadLimitProgressAtMeasuringRestart
+                                << relativeLimitProgressMeasured << relativeLimitProgressDifference;
 
-//     qint64 speedkBPerSec = (relativeLimitProgressDifference / relativeLimitMeasuringTimerIntervalMsec*1000.0) / 1024.0;
-//    qDebug() << Q_FUNC_INFO << relativeLimitProgressDifference/1024 <<"kB =>" << speedkBPerSec << "kB/sec on full speed ("
-//             << _relativeLimitCurrentMeasuredJob->currentDownloadPosition() ;
+    qint64 speedkBPerSec = (relativeLimitProgressDifference / relativeLimitMeasuringTimerIntervalMsec * 1000.0) / 1024.0;
+    qCDebug(lcBandwidthManager) << relativeLimitProgressDifference / 1024 << "kB =>" << speedkBPerSec << "kB/sec on full speed ("
+                                << _relativeLimitCurrentMeasuredJob->currentDownloadPosition();
 
     qint64 downloadLimitPercent = -_currentDownloadLimit;
     // don't use too extreme values
@@ -288,9 +274,7 @@ void BandwidthManager::relativeDownloadMeasuringTimerExpired()
     qint64 wholeTimeMsec = (100.0 / downloadLimitPercent) * relativeLimitMeasuringTimerIntervalMsec;
     qint64 waitTimeMsec = wholeTimeMsec - relativeLimitMeasuringTimerIntervalMsec;
     qint64 realWaitTimeMsec = waitTimeMsec + wholeTimeMsec;
-//    qDebug() << Q_FUNC_INFO << waitTimeMsec << " - "<< realWaitTimeMsec <<
-//                " msec for " << downloadLimitPercent << "%";
-//    qDebug() << Q_FUNC_INFO << "XXXX" << downloadLimitPercent << relativeLimitMeasuringTimerIntervalMsec;
+    qCDebug(lcBandwidthManager) << waitTimeMsec << " - " << realWaitTimeMsec << " msec for " << downloadLimitPercent << "%";
 
     // We want to wait twice as long since we want to give all
     // devices the same quota we used now since we don't want
@@ -300,17 +284,16 @@ void BandwidthManager::relativeDownloadMeasuringTimerExpired()
 
     int jobCount = _downloadJobList.count();
     qint64 quota = relativeLimitProgressDifference * (downloadLimitPercent / 100.0);
-//    if (quota > 20*1024) {
-//        qDebug() << "======== ADJUSTING QUOTA FROM " << quota << " TO " << quota - 20*1024;
-//        quota -= 20*1024;
-//    }
+    if (quota > 20 * 1024) {
+        qCInfo(lcBandwidthManager) << "ADJUSTING QUOTA FROM " << quota << " TO " << quota - 20 * 1024;
+        quota -= 20 * 1024;
+    }
     qint64 quotaPerJob = quota / jobCount + 1.0;
-//    qDebug() << Q_FUNC_INFO << "YYYY" << relativeLimitProgressDifference << downloadLimitPercent << jobCount;
-    Q_FOREACH(GETFileJob *gfj, _downloadJobList) {
+    Q_FOREACH (GETFileJob *gfj, _downloadJobList) {
         gfj->setBandwidthLimited(true);
         gfj->setChoked(false);
         gfj->giveBandwidthQuota(quotaPerJob);
-//        qDebug() << Q_FUNC_INFO << "Gave" << quotaPerJob/1024.0 << "kB to" << gfj;
+        qCDebug(lcBandwidthManager) << "Gave" << quotaPerJob / 1024.0 << "kB to" << gfj;
     }
     _relativeLimitCurrentMeasuredDevice = 0;
 }
@@ -325,11 +308,11 @@ void BandwidthManager::relativeDownloadDelayTimerExpired()
     }
 
     if (_downloadJobList.isEmpty()) {
-        //qDebug() << Q_FUNC_INFO << _downloadJobList.count() << "No jobs?";
+        qCDebug(lcBandwidthManager) << _downloadJobList.count() << "No jobs?";
         return;
     }
 
-//    qDebug() << Q_FUNC_INFO << _downloadJobList.count() << "Starting measuring";
+    qCDebug(lcBandwidthManager) << _downloadJobList.count() << "Starting measuring";
 
     // Take first device and then append it again (= we round robin all devices)
     _relativeLimitCurrentMeasuredJob = _downloadJobList.takeFirst();
@@ -340,7 +323,7 @@ void BandwidthManager::relativeDownloadDelayTimerExpired()
     _relativeLimitCurrentMeasuredJob->setChoked(false);
 
     // choke all other download jobs
-    Q_FOREACH(GETFileJob *gfj, _downloadJobList) {
+    Q_FOREACH (GETFileJob *gfj, _downloadJobList) {
         if (gfj != _relativeLimitCurrentMeasuredJob) {
             gfj->setBandwidthLimited(true);
             gfj->setChoked(true);
@@ -352,12 +335,13 @@ void BandwidthManager::relativeDownloadDelayTimerExpired()
 
 // end downloads
 
-void BandwidthManager::switchingTimerExpired() {
+void BandwidthManager::switchingTimerExpired()
+{
     qint64 newUploadLimit = _propagator->_uploadLimit.fetchAndAddAcquire(0);
     if (newUploadLimit != _currentUploadLimit) {
-        qDebug() << Q_FUNC_INFO << "Upload Bandwidth limit changed" << _currentUploadLimit << newUploadLimit;
+        qCInfo(lcBandwidthManager) << "Upload Bandwidth limit changed" << _currentUploadLimit << newUploadLimit;
         _currentUploadLimit = newUploadLimit;
-        Q_FOREACH(UploadDevice *ud, _relativeUploadDeviceList) {
+        Q_FOREACH (UploadDevice *ud, _relativeUploadDeviceList) {
             if (newUploadLimit == 0) {
                 ud->setBandwidthLimited(false);
                 ud->setChoked(false);
@@ -372,9 +356,9 @@ void BandwidthManager::switchingTimerExpired() {
     }
     qint64 newDownloadLimit = _propagator->_downloadLimit.fetchAndAddAcquire(0);
     if (newDownloadLimit != _currentDownloadLimit) {
-        qDebug() << Q_FUNC_INFO << "Download Bandwidth limit changed" << _currentDownloadLimit << newDownloadLimit;
+        qCInfo(lcBandwidthManager) << "Download Bandwidth limit changed" << _currentDownloadLimit << newDownloadLimit;
         _currentDownloadLimit = newDownloadLimit;
-        Q_FOREACH(GETFileJob *j, _downloadJobList) {
+        Q_FOREACH (GETFileJob *j, _downloadJobList) {
             if (usingAbsoluteDownloadLimit()) {
                 j->setBandwidthLimited(true);
                 j->setChoked(false);
@@ -393,21 +377,19 @@ void BandwidthManager::absoluteLimitTimerExpired()
 {
     if (usingAbsoluteUploadLimit() && _absoluteUploadDeviceList.count() > 0) {
         qint64 quotaPerDevice = _currentUploadLimit / qMax(1, _absoluteUploadDeviceList.count());
-//        qDebug() << Q_FUNC_INFO << quotaPerDevice <<  _absoluteUploadDeviceList.count() << _currentUploadLimit;
-        Q_FOREACH(UploadDevice *device, _absoluteUploadDeviceList) {
+        qCDebug(lcBandwidthManager) << quotaPerDevice << _absoluteUploadDeviceList.count() << _currentUploadLimit;
+        Q_FOREACH (UploadDevice *device, _absoluteUploadDeviceList) {
             device->giveBandwidthQuota(quotaPerDevice);
-//            qDebug() << Q_FUNC_INFO << "Gave " << quotaPerDevice/1024.0 << " kB to" << device;
+            qCDebug(lcBandwidthManager) << "Gave " << quotaPerDevice / 1024.0 << " kB to" << device;
         }
     }
     if (usingAbsoluteDownloadLimit() && _downloadJobList.count() > 0) {
         qint64 quotaPerJob = _currentDownloadLimit / qMax(1, _downloadJobList.count());
-//        qDebug() << Q_FUNC_INFO << quotaPerJob <<  _downloadJobList.count() << _currentDownloadLimit;
-        Q_FOREACH(GETFileJob *j, _downloadJobList) {
+        qCDebug(lcBandwidthManager) << quotaPerJob << _downloadJobList.count() << _currentDownloadLimit;
+        Q_FOREACH (GETFileJob *j, _downloadJobList) {
             j->giveBandwidthQuota(quotaPerJob);
-//            qDebug() << Q_FUNC_INFO << "Gave " << quotaPerJob/1024.0 << " kB to" << j;
+            qCDebug(lcBandwidthManager) << "Gave " << quotaPerJob / 1024.0 << " kB to" << j;
         }
     }
 }
-
-
 }
