@@ -16,15 +16,18 @@
 
 #include "configfile.h"
 #include "theme.h"
+#include "version.h"
 #include "common/utility.h"
 #include "common/asserts.h"
+#include "version.h"
 
 #include "creds/abstractcredentials.h"
 
+#include "csync_exclude.h"
+
 #ifndef TOKEN_AUTH_ONLY
 #include <QWidget>
 #include <QHeaderView>
-#include <QDesktopServices>
 #endif
 
 #include <QCoreApplication>
@@ -34,30 +37,39 @@
 #include <QLoggingCategory>
 #include <QSettings>
 #include <QNetworkProxy>
+#include <QStandardPaths>
 
 #define DEFAULT_REMOTE_POLL_INTERVAL 30000 // default remote poll time in milliseconds
 #define DEFAULT_MAX_LOG_LINES 20000
 
 namespace OCC {
 
+namespace chrono = std::chrono;
+
 Q_LOGGING_CATEGORY(lcConfigFile, "sync.configfile", QtInfoMsg)
 
 //static const char caCertsKeyC[] = "CaCertificates"; only used from account.cpp
 static const char remotePollIntervalC[] = "remotePollInterval";
 static const char forceSyncIntervalC[] = "forceSyncInterval";
+static const char fullLocalDiscoveryIntervalC[] = "fullLocalDiscoveryInterval";
 static const char notificationRefreshIntervalC[] = "notificationRefreshInterval";
 static const char monoIconsC[] = "monoIcons";
 static const char promptDeleteC[] = "promptDeleteAllFiles";
 static const char crashReporterC[] = "crashReporter";
 static const char optionalDesktopNoficationsC[] = "optionalDesktopNotifications";
+static const char showInExplorerNavigationPaneC[] = "showInExplorerNavigationPane";
 static const char skipUpdateCheckC[] = "skipUpdateCheck";
 static const char updateCheckIntervalC[] = "updateCheckInterval";
+static const char updateChannelC[] = "updateChannel";
 static const char geometryC[] = "geometry";
 static const char timeoutC[] = "timeout";
 static const char chunkSizeC[] = "chunkSize";
 static const char minChunkSizeC[] = "minChunkSize";
 static const char maxChunkSizeC[] = "maxChunkSize";
 static const char targetChunkUploadDurationC[] = "targetChunkUploadDuration";
+static const char automaticLogDirC[] = "logToTemporaryLogDir";
+static const char showExperimentalOptionsC[] = "showExperimentalOptions";
+static const char clientVersionC[] = "clientVersion";
 
 static const char proxyHostC[] = "Proxy/host";
 static const char proxyTypeC[] = "Proxy/type";
@@ -74,14 +86,21 @@ static const char downloadLimitC[] = "BWLimit/downloadLimit";
 static const char newBigFolderSizeLimitC[] = "newBigFolderSizeLimit";
 static const char useNewBigFolderSizeLimitC[] = "useNewBigFolderSizeLimit";
 static const char confirmExternalStorageC[] = "confirmExternalStorage";
+static const char moveToTrashC[] = "moveToTrash";
 
 static const char maxLogLinesC[] = "Logging/maxLogLines";
 
 const char certPath[] = "http_certificatePath";
 const char certPasswd[] = "http_certificatePasswd";
-QString ConfigFile::_confDir = QString::null;
+QString ConfigFile::_confDir = QString();
 bool ConfigFile::_askedUser = false;
 
+static chrono::milliseconds millisecondsValue(const QSettings &setting, const char *key,
+    chrono::milliseconds defaultValue)
+{
+    return chrono::milliseconds(setting.value(QLatin1String(key), qlonglong(defaultValue.count())).toLongLong());
+}
+
 ConfigFile::ConfigFile()
 {
     // QDesktopServices uses the application name to create a config path
@@ -122,6 +141,26 @@ bool ConfigFile::optionalDesktopNotifications() const
     return settings.value(QLatin1String(optionalDesktopNoficationsC), true).toBool();
 }
 
+bool ConfigFile::showInExplorerNavigationPane() const
+{
+    const bool defaultValue =
+#ifdef Q_OS_WIN
+        QSysInfo::windowsVersion() >= QSysInfo::WV_WINDOWS10
+#else
+        false
+#endif
+        ;
+    QSettings settings(configFile(), QSettings::IniFormat);
+    return settings.value(QLatin1String(showInExplorerNavigationPaneC), defaultValue).toBool();
+}
+
+void ConfigFile::setShowInExplorerNavigationPane(bool show)
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.setValue(QLatin1String(showInExplorerNavigationPaneC), show);
+    settings.sync();
+}
+
 int ConfigFile::timeout() const
 {
     QSettings settings(configFile(), QSettings::IniFormat);
@@ -146,10 +185,10 @@ quint64 ConfigFile::minChunkSize() const
     return settings.value(QLatin1String(minChunkSizeC), 1000 * 1000).toLongLong(); // default to 1 MB
 }
 
-quint64 ConfigFile::targetChunkUploadDuration() const
+chrono::milliseconds ConfigFile::targetChunkUploadDuration() const
 {
     QSettings settings(configFile(), QSettings::IniFormat);
-    return settings.value(QLatin1String(targetChunkUploadDurationC), 60 * 1000).toLongLong(); // default to 1 minute
+    return millisecondsValue(settings, targetChunkUploadDurationC, chrono::minutes(1));
 }
 
 void ConfigFile::setOptionalDesktopNotifications(bool show)
@@ -227,13 +266,11 @@ QVariant ConfigFile::getPolicySetting(const QString &setting, const QVariant &de
 
 QString ConfigFile::configPath() const
 {
-#ifndef TOKEN_AUTH_ONLY
     if (_confDir.isEmpty()) {
-        //  Qt 5's QStandardPaths::writableLocation gives us wrong results (without /data/),
-        //  so we'll have to use the deprecated version for now
-        _confDir = QDesktopServices::storageLocation(QDesktopServices::DataLocation);
+        // On Unix, use the AppConfigLocation for the settings, that's configurable with the XDG_CONFIG_HOME env variable.
+        // On Windows, use AppDataLocation, that's where the roaming data is and where we should store the config file
+        _confDir = QStandardPaths::writableLocation(Utility::isWindows() ? QStandardPaths::AppDataLocation : QStandardPaths::AppConfigLocation);
     }
-#endif
     QString dir = _confDir;
 
     if (!dir.endsWith(QLatin1Char('/')))
@@ -241,12 +278,6 @@ QString ConfigFile::configPath() const
     return dir;
 }
 
-QString ConfigFile::configPathWithAppName() const
-{
-    //HACK
-    return QFileInfo(configFile()).dir().absolutePath().append("/");
-}
-
 static const QLatin1String exclFile("sync-exclude.lst");
 
 QString ConfigFile::excludeFile(Scope scope) const
@@ -289,6 +320,19 @@ QString ConfigFile::excludeFileFromSystem()
         QFileInfo nextToBinary(QCoreApplication::applicationDirPath(), exclFile);
         if (nextToBinary.exists()) {
             fi = nextToBinary;
+        } else {
+            // For AppImage, the file might reside under a temporary mount path
+            QDir d(QCoreApplication::applicationDirPath()); // supposed to be /tmp/mount.xyz/usr/bin
+            d.cdUp(); // go out of bin
+            d.cdUp(); // go out of usr
+            if (!d.isRoot()) { // it is really a mountpoint
+                if (d.cd("etc") && d.cd(Theme::instance()->appName())) {
+                    QFileInfo inMountDir(d, exclFile);
+                    if (inMountDir.exists()) {
+                        fi = inMountDir;
+                    }
+                };
+            }
         }
     }
 #endif
@@ -301,6 +345,28 @@ QString ConfigFile::excludeFileFromSystem()
     return fi.absoluteFilePath();
 }
 
+QString ConfigFile::backup() const
+{
+    QString baseFile = configFile();
+    auto versionString = clientVersionString();
+    if (!versionString.isEmpty())
+        versionString.prepend('_');
+    QString backupFile =
+        QString("%1.backup_%2%3")
+            .arg(baseFile)
+            .arg(QDateTime::currentDateTime().toString("yyyyMMdd_HHmmss"))
+            .arg(versionString);
+
+    // If this exact file already exists it's most likely that a backup was
+    // already done. (two backup calls directly after each other, potentially
+    // even with source alterations in between!)
+    if (!QFile::exists(backupFile)) {
+        QFile f(baseFile);
+        f.copy(backupFile);
+    }
+    return backupFile;
+}
+
 QString ConfigFile::configFile() const
 {
     return configPath() + Theme::instance()->configFileName();
@@ -354,7 +420,7 @@ bool ConfigFile::dataExists(const QString &group, const QString &key) const
     return settings.contains(key);
 }
 
-int ConfigFile::remotePollInterval(const QString &connection) const
+chrono::milliseconds ConfigFile::remotePollInterval(const QString &connection) const
 {
     QString con(connection);
     if (connection.isEmpty())
@@ -363,33 +429,34 @@ int ConfigFile::remotePollInterval(const QString &connection) const
     QSettings settings(configFile(), QSettings::IniFormat);
     settings.beginGroup(con);
 
-    int remoteInterval = settings.value(QLatin1String(remotePollIntervalC), DEFAULT_REMOTE_POLL_INTERVAL).toInt();
-    if (remoteInterval < 5000) {
+    auto defaultPollInterval = chrono::milliseconds(DEFAULT_REMOTE_POLL_INTERVAL);
+    auto remoteInterval = millisecondsValue(settings, remotePollIntervalC, defaultPollInterval);
+    if (remoteInterval < chrono::seconds(5)) {
         qCWarning(lcConfigFile) << "Remote Interval is less than 5 seconds, reverting to" << DEFAULT_REMOTE_POLL_INTERVAL;
-        remoteInterval = DEFAULT_REMOTE_POLL_INTERVAL;
+        remoteInterval = defaultPollInterval;
     }
     return remoteInterval;
 }
 
-void ConfigFile::setRemotePollInterval(int interval, const QString &connection)
+void ConfigFile::setRemotePollInterval(chrono::milliseconds interval, const QString &connection)
 {
     QString con(connection);
     if (connection.isEmpty())
         con = defaultConnection();
 
-    if (interval < 5000) {
-        qCWarning(lcConfigFile) << "Remote Poll interval of " << interval << " is below five seconds.";
+    if (interval < chrono::seconds(5)) {
+        qCWarning(lcConfigFile) << "Remote Poll interval of " << interval.count() << " is below five seconds.";
         return;
     }
     QSettings settings(configFile(), QSettings::IniFormat);
     settings.beginGroup(con);
-    settings.setValue(QLatin1String(remotePollIntervalC), interval);
+    settings.setValue(QLatin1String(remotePollIntervalC), qlonglong(interval.count()));
     settings.sync();
 }
 
-quint64 ConfigFile::forceSyncInterval(const QString &connection) const
+chrono::milliseconds ConfigFile::forceSyncInterval(const QString &connection) const
 {
-    uint pollInterval = remotePollInterval(connection);
+    auto pollInterval = remotePollInterval(connection);
 
     QString con(connection);
     if (connection.isEmpty())
@@ -397,16 +464,23 @@ quint64 ConfigFile::forceSyncInterval(const QString &connection) const
     QSettings settings(configFile(), QSettings::IniFormat);
     settings.beginGroup(con);
 
-    quint64 defaultInterval = 2 * 60 * 60 * 1000ull; // 2h
-    quint64 interval = settings.value(QLatin1String(forceSyncIntervalC), defaultInterval).toULongLong();
+    auto defaultInterval = chrono::hours(2);
+    auto interval = millisecondsValue(settings, forceSyncIntervalC, defaultInterval);
     if (interval < pollInterval) {
-        qCWarning(lcConfigFile) << "Force sync interval is less than the remote poll inteval, reverting to" << pollInterval;
+        qCWarning(lcConfigFile) << "Force sync interval is less than the remote poll inteval, reverting to" << pollInterval.count();
         interval = pollInterval;
     }
     return interval;
 }
 
-quint64 ConfigFile::notificationRefreshInterval(const QString &connection) const
+chrono::milliseconds OCC::ConfigFile::fullLocalDiscoveryInterval() const
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.beginGroup(defaultConnection());
+    return millisecondsValue(settings, fullLocalDiscoveryIntervalC, chrono::hours(1));
+}
+
+chrono::milliseconds ConfigFile::notificationRefreshInterval(const QString &connection) const
 {
     QString con(connection);
     if (connection.isEmpty())
@@ -414,16 +488,16 @@ quint64 ConfigFile::notificationRefreshInterval(const QString &connection) const
     QSettings settings(configFile(), QSettings::IniFormat);
     settings.beginGroup(con);
 
-    quint64 defaultInterval = 5 * 60 * 1000ull; // 5 minutes
-    quint64 interval = settings.value(QLatin1String(notificationRefreshIntervalC), defaultInterval).toULongLong();
-    if (interval < 60 * 1000ull) {
+    auto defaultInterval = chrono::minutes(5);
+    auto interval = millisecondsValue(settings, notificationRefreshIntervalC, defaultInterval);
+    if (interval < chrono::minutes(1)) {
         qCWarning(lcConfigFile) << "Notification refresh interval smaller than one minute, setting to one minute";
-        interval = 60 * 1000ull;
+        interval = chrono::minutes(1);
     }
     return interval;
 }
 
-int ConfigFile::updateCheckInterval(const QString &connection) const
+chrono::milliseconds ConfigFile::updateCheckInterval(const QString &connection) const
 {
     QString con(connection);
     if (connection.isEmpty())
@@ -431,12 +505,12 @@ int ConfigFile::updateCheckInterval(const QString &connection) const
     QSettings settings(configFile(), QSettings::IniFormat);
     settings.beginGroup(con);
 
-    int defaultInterval = 1000 * 60 * 60 * 10; // ten hours
-    int interval = settings.value(QLatin1String(updateCheckIntervalC), defaultInterval).toInt();
+    auto defaultInterval = chrono::hours(10);
+    auto interval = millisecondsValue(settings, updateCheckIntervalC, defaultInterval);
 
-    int minInterval = 1000 * 60 * 5;
+    auto minInterval = chrono::minutes(5);
     if (interval < minInterval) {
-        qCWarning(lcConfigFile) << "Update check interval less than five minutes, setting " << minInterval;
+        qCWarning(lcConfigFile) << "Update check interval less than five minutes, resetting to 5 minutes";
         interval = minInterval;
     }
     return interval;
@@ -468,6 +542,28 @@ void ConfigFile::setSkipUpdateCheck(bool skip, const QString &connection)
     settings.sync();
 }
 
+QString ConfigFile::updateChannel() const
+{
+    QString defaultUpdateChannel = QStringLiteral("stable");
+    QString suffix = QString::fromLatin1(MIRALL_STRINGIFY(MIRALL_VERSION_SUFFIX));
+    if (suffix.startsWith("daily")
+        || suffix.startsWith("nightly")
+        || suffix.startsWith("alpha")
+        || suffix.startsWith("rc")
+        || suffix.startsWith("beta")) {
+        defaultUpdateChannel = QStringLiteral("beta");
+    }
+
+    QSettings settings(configFile(), QSettings::IniFormat);
+    return settings.value(QLatin1String(updateChannelC), defaultUpdateChannel).toString();
+}
+
+void ConfigFile::setUpdateChannel(const QString &channel)
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.setValue(QLatin1String(updateChannelC), channel);
+}
+
 int ConfigFile::maxLogLines() const
 {
     QSettings settings(configFile(), QSettings::IniFormat);
@@ -577,12 +673,12 @@ QString ConfigFile::proxyPassword() const
 
 int ConfigFile::useUploadLimit() const
 {
-    return getValue(useUploadLimitC, QString::null, 0).toInt();
+    return getValue(useUploadLimitC, QString(), 0).toInt();
 }
 
 int ConfigFile::useDownloadLimit() const
 {
-    return getValue(useDownloadLimitC, QString::null, 0).toInt();
+    return getValue(useDownloadLimitC, QString(), 0).toInt();
 }
 
 void ConfigFile::setUseUploadLimit(int val)
@@ -597,12 +693,12 @@ void ConfigFile::setUseDownloadLimit(int val)
 
 int ConfigFile::uploadLimit() const
 {
-    return getValue(uploadLimitC, QString::null, 10).toInt();
+    return getValue(uploadLimitC, QString(), 10).toInt();
 }
 
 int ConfigFile::downloadLimit() const
 {
-    return getValue(downloadLimitC, QString::null, 80).toInt();
+    return getValue(downloadLimitC, QString(), 80).toInt();
 }
 
 void ConfigFile::setUploadLimit(int kbytes)
@@ -639,6 +735,16 @@ void ConfigFile::setConfirmExternalStorage(bool isChecked)
     setValue(confirmExternalStorageC, isChecked);
 }
 
+bool ConfigFile::moveToTrash() const
+{
+    return getValue(moveToTrashC, QString(), false).toBool();
+}
+
+void ConfigFile::setMoveToTrash(bool isChecked)
+{
+    setValue(moveToTrashC, isChecked);
+}
+
 bool ConfigFile::promptDeleteFiles() const
 {
     QSettings settings(configFile(), QSettings::IniFormat);
@@ -680,6 +786,24 @@ void ConfigFile::setCrashReporter(bool enabled)
     settings.setValue(QLatin1String(crashReporterC), enabled);
 }
 
+bool ConfigFile::automaticLogDir() const
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    return settings.value(QLatin1String(automaticLogDirC), false).toBool();
+}
+
+void ConfigFile::setAutomaticLogDir(bool enabled)
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.setValue(QLatin1String(automaticLogDirC), enabled);
+}
+
+bool ConfigFile::showExperimentalOptions() const
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    return settings.value(QLatin1String(showExperimentalOptionsC), false).toBool();
+}
+
 QString ConfigFile::certificatePath() const
 {
     return retrieveData(QString(), QLatin1String(certPath)).toString();
@@ -704,6 +828,18 @@ void ConfigFile::setCertificatePasswd(const QString &cPasswd)
     settings.sync();
 }
 
+QString ConfigFile::clientVersionString() const
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    return settings.value(QLatin1String(clientVersionC), QString()).toString();
+}
+
+void ConfigFile::setClientVersionString(const QString &version)
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.setValue(QLatin1String(clientVersionC), version);
+}
+
 Q_GLOBAL_STATIC(QString, g_configFileName)
 
 std::unique_ptr<QSettings> ConfigFile::settingsWithGroup(const QString &group, QObject *parent)
@@ -718,4 +854,17 @@ std::unique_ptr<QSettings> ConfigFile::settingsWithGroup(const QString &group, Q
     return settings;
 }
 
+void ConfigFile::setupDefaultExcludeFilePaths(ExcludedFiles &excludedFiles)
+{
+    ConfigFile cfg;
+    QString systemList = cfg.excludeFile(ConfigFile::SystemScope);
+    qCInfo(lcConfigFile) << "Adding system ignore list to csync:" << systemList;
+    excludedFiles.addExcludeFilePath(systemList);
+
+    QString userList = cfg.excludeFile(ConfigFile::UserScope);
+    if (QFile::exists(userList)) {
+        qCInfo(lcConfigFile) << "Adding user defined ignore list to csync:" << userList;
+        excludedFiles.addExcludeFilePath(userList);
+    }
+}
 }
