@@ -21,9 +21,11 @@
 #include <QSettings>
 #include <QString>
 #include <QVariant>
+#include <chrono>
 
 class QWidget;
 class QHeaderView;
+class ExcludedFiles;
 
 namespace OCC {
 
@@ -42,11 +44,17 @@ public:
         SystemScope };
 
     QString configPath() const;
-    QString configPathWithAppName() const;
     QString configFile() const;
     QString excludeFile(Scope scope) const;
     static QString excludeFileFromSystem(); // doesn't access config dir
 
+    /**
+     * Creates a backup of the file
+     *
+     * Returns the path of the new backup.
+     */
+    QString backup() const;
+
     bool exists();
 
     QString defaultConnection() const;
@@ -55,22 +63,29 @@ public:
     QByteArray caCerts();
     void setCaCerts(const QByteArray &);
 
-    bool passwordStorageAllowed(const QString &connection = QString::null);
+    bool passwordStorageAllowed(const QString &connection = QString());
 
     // max count of lines in the log window
     int maxLogLines() const;
     void setMaxLogLines(int);
 
     /* Server poll interval in milliseconds */
-    int remotePollInterval(const QString &connection = QString()) const;
+    std::chrono::milliseconds remotePollInterval(const QString &connection = QString()) const;
     /* Set poll interval. Value in milliseconds has to be larger than 5000 */
-    void setRemotePollInterval(int interval, const QString &connection = QString());
+    void setRemotePollInterval(std::chrono::milliseconds interval, const QString &connection = QString());
 
     /* Interval to check for new notifications */
-    quint64 notificationRefreshInterval(const QString &connection = QString()) const;
+    std::chrono::milliseconds notificationRefreshInterval(const QString &connection = QString()) const;
 
     /* Force sync interval, in milliseconds */
-    quint64 forceSyncInterval(const QString &connection = QString()) const;
+    std::chrono::milliseconds forceSyncInterval(const QString &connection = QString()) const;
+
+    /**
+     * Interval in milliseconds within which full local discovery is required
+     *
+     * Use -1 to disable regular full local discoveries.
+     */
+    std::chrono::milliseconds fullLocalDiscoveryInterval() const;
 
     bool monoIcons() const;
     void setMonoIcons(bool);
@@ -81,6 +96,12 @@ public:
     bool crashReporter() const;
     void setCrashReporter(bool enabled);
 
+    bool automaticLogDir() const;
+    void setAutomaticLogDir(bool enabled);
+
+    // Whether experimental UI options should be shown
+    bool showExperimentalOptions() const;
+
     // proxy settings
     void setProxyType(int proxyType,
         const QString &host = QString(),
@@ -111,26 +132,36 @@ public:
     bool confirmExternalStorage() const;
     void setConfirmExternalStorage(bool);
 
+    /** If we should move the files deleted on the server in the trash  */
+    bool moveToTrash() const;
+    void setMoveToTrash(bool);
+
     static bool setConfDir(const QString &value);
 
     bool optionalDesktopNotifications() const;
     void setOptionalDesktopNotifications(bool show);
 
+    bool showInExplorerNavigationPane() const;
+    void setShowInExplorerNavigationPane(bool show);
+
     int timeout() const;
     quint64 chunkSize() const;
     quint64 maxChunkSize() const;
     quint64 minChunkSize() const;
-    quint64 targetChunkUploadDuration() const;
+    std::chrono::milliseconds targetChunkUploadDuration() const;
 
     void saveGeometry(QWidget *w);
     void restoreGeometry(QWidget *w);
 
-    // how often the check about new versions runs, default two hours
-    int updateCheckInterval(const QString &connection = QString()) const;
+    // how often the check about new versions runs
+    std::chrono::milliseconds updateCheckInterval(const QString &connection = QString()) const;
 
     bool skipUpdateCheck(const QString &connection = QString()) const;
     void setSkipUpdateCheck(bool, const QString &);
 
+    QString updateChannel() const;
+    void setUpdateChannel(const QString &channel);
+
     void saveGeometryHeader(QHeaderView *header);
     void restoreGeometryHeader(QHeaderView *header);
 
@@ -139,10 +170,18 @@ public:
     QString certificatePasswd() const;
     void setCertificatePasswd(const QString &cPasswd);
 
+    /** The client version that last used this settings file.
+        Updated by configVersionMigration() at client startup. */
+    QString clientVersionString() const;
+    void setClientVersionString(const QString &version);
+
     /**  Returns a new settings pre-set in a specific group.  The Settings will be created
          with the given parent. If no parent is specified, the caller must destroy the settings */
     static std::unique_ptr<QSettings> settingsWithGroup(const QString &group, QObject *parent = 0);
 
+    /// Add the system and user exclude file path to the ExcludedFiles instance.
+    static void setupDefaultExcludeFilePaths(ExcludedFiles &excludedFiles);
+
 protected:
     QVariant getPolicySetting(const QString &policy, const QVariant &defaultValue = QVariant()) const;
     void storeData(const QString &group, const QString &key, const QVariant &value);
@@ -151,7 +190,7 @@ protected:
     bool dataExists(const QString &group, const QString &key) const;
 
 private:
-    QVariant getValue(const QString &param, const QString &group = QString::null,
+    QVariant getValue(const QString &param, const QString &group = QString(),
         const QVariant &defaultValue = QVariant()) const;
     void setValue(const QString &key, const QVariant &value);
 
