@@ -15,7 +15,6 @@
 #include "discoveryphase.h"
 
 #include "account.h"
-#include "theme.h"
 #include "common/asserts.h"
 #include "common/checksums.h"
 
@@ -91,7 +90,7 @@ int DiscoveryJob::isInSelectiveSyncBlackListCallback(void *data, const QByteArra
 
 bool DiscoveryJob::checkSelectiveSyncNewFolder(const QString &path, RemotePermissions remotePerm)
 {
-    if (_syncOptions._confirmExternalStorage
+    if (_syncOptions._confirmExternalStorage && !_syncOptions._newFilesAreVirtual
         && remotePerm.hasPermission(RemotePermissions::IsMounted)) {
         // external storage.
 
@@ -114,7 +113,7 @@ bool DiscoveryJob::checkSelectiveSyncNewFolder(const QString &path, RemotePermis
     }
 
     auto limit = _syncOptions._newBigFolderSizeLimit;
-    if (limit < 0) {
+    if (limit < 0 || _syncOptions._newFilesAreVirtual) {
         // no limit, everything is allowed;
         return false;
     }
@@ -301,26 +300,27 @@ void DiscoverySingleDirectoryJob::abort()
     }
 }
 
-static std::unique_ptr<csync_file_stat_t> propertyMapToFileStat(const QMap<QString, QString> &map)
+static void propertyMapToFileStat(const QMap<QString, QString> &map, csync_file_stat_t *file_stat)
 {
-    std::unique_ptr<csync_file_stat_t> file_stat(new csync_file_stat_t);
-
     for (auto it = map.constBegin(); it != map.constEnd(); ++it) {
         QString property = it.key();
         QString value = it.value();
         if (property == "resourcetype") {
             if (value.contains("collection")) {
-                file_stat->type = CSYNC_FTW_TYPE_DIR;
+                file_stat->type = ItemTypeDirectory;
             } else {
-                file_stat->type = CSYNC_FTW_TYPE_FILE;
+                file_stat->type = ItemTypeFile;
             }
         } else if (property == "getlastmodified") {
             file_stat->modtime = oc_httpdate_parse(value.toUtf8());
         } else if (property == "getcontentlength") {
+            // See #4573, sometimes negative size values are returned
             bool ok = false;
             qlonglong ll = value.toLongLong(&ok);
             if (ok && ll >= 0) {
                 file_stat->size = ll;
+            } else {
+                file_stat->size = 0;
             }
         } else if (property == "getetag") {
             file_stat->etag = Utility::normalizeEtag(value.toUtf8());
@@ -331,7 +331,7 @@ static std::unique_ptr<csync_file_stat_t> propertyMapToFileStat(const QMap<QStri
         } else if (property == "dDC") {
             file_stat->directDownloadCookies = value.toUtf8();
         } else if (property == "permissions") {
-            file_stat->remotePerm = RemotePermissions(value);
+            file_stat->remotePerm = RemotePermissions::fromServerString(value);
         } else if (property == "checksums") {
             file_stat->checksumHeader = findBestChecksum(value.toUtf8());
         } else if (property == "share-types" && !value.isEmpty()) {
@@ -347,7 +347,6 @@ static std::unique_ptr<csync_file_stat_t> propertyMapToFileStat(const QMap<QStri
             }
         }
     }
-    return file_stat;
 }
 
 void DiscoverySingleDirectoryJob::directoryListingIteratedSlot(QString file, const QMap<QString, QString> &map)
@@ -356,12 +355,16 @@ void DiscoverySingleDirectoryJob::directoryListingIteratedSlot(QString file, con
         // The first entry is for the folder itself, we should process it differently.
         _ignoredFirst = true;
         if (map.contains("permissions")) {
-            RemotePermissions perm(map.value("permissions"));
+            auto perm = RemotePermissions::fromServerString(map.value("permissions"));
             emit firstDirectoryPermissions(perm);
             _isExternalStorage = perm.hasPermission(RemotePermissions::IsMounted);
         }
         if (map.contains("data-fingerprint")) {
             _dataFingerprint = map.value("data-fingerprint").toUtf8();
+            if (_dataFingerprint.isEmpty()) {
+                // Placeholder that means that the server supports the feature even if it did not set one.
+                _dataFingerprint = "[empty]";
+            }
         }
     } else {
         // Remove <webDAV-Url>/folder/ from <webDAV-Url>/folder/subfile.txt
@@ -375,12 +378,24 @@ void DiscoverySingleDirectoryJob::directoryListingIteratedSlot(QString file, con
             file = file.remove(0, 1);
         }
 
-
-        std::unique_ptr<csync_file_stat_t> file_stat(propertyMapToFileStat(map));
+        std::unique_ptr<csync_file_stat_t> file_stat(new csync_file_stat_t);
         file_stat->path = file.toUtf8();
-        if (file_stat->etag.isEmpty()) {
-            qCCritical(lcDiscovery) << "etag of" << file_stat->path << "is" << file_stat->etag << "This must not happen.";
+        file_stat->size = -1;
+        propertyMapToFileStat(map, file_stat.get());
+        if (file_stat->type == ItemTypeDirectory)
+            file_stat->size = 0;
+        if (file_stat->type == ItemTypeSkip
+            || file_stat->size == -1
+            || file_stat->remotePerm.isNull()
+            || file_stat->etag.isEmpty()
+            || file_stat->file_id.isEmpty()) {
+            _error = tr("The server file discovery reply is missing data.");
+            qCWarning(lcDiscovery)
+                << "Missing properties:" << file << file_stat->type << file_stat->size
+                << file_stat->modtime << file_stat->remotePerm.toString()
+                << file_stat->etag << file_stat->file_id;
         }
+
         if (_isExternalStorage && file_stat->remotePerm.hasPermission(RemotePermissions::IsMounted)) {
             /* All the entries in a external storage have 'M' in their permission. However, for all
                purposes in the desktop client, we only need to know about the mount points.
@@ -415,6 +430,10 @@ void DiscoverySingleDirectoryJob::lsJobFinishedWithoutErrorSlot()
         emit finishedWithError(ERRNO_WRONG_CONTENT, QLatin1String("Server error: PROPFIND reply is not XML formatted!"));
         deleteLater();
         return;
+    } else if (!_error.isEmpty()) {
+        emit finishedWithError(ERRNO_WRONG_CONTENT, _error);
+        deleteLater();
+        return;
     }
     emit etag(_firstEtag);
     emit etagConcatenation(_etagConcatenation);
@@ -471,8 +490,7 @@ void DiscoveryMainThread::doOpendirSlot(const QString &subPath, DiscoveryDirecto
         fullPath.chop(1);
     }
 
-    // emit _discoveryJob->folderDiscovered(false, subPath);
-    _discoveryJob->update_job_update_callback(false, subPath.toUtf8(), _discoveryJob);
+    _discoveryJob->update_job_update_callback(/*local=*/false, subPath.toUtf8(), _discoveryJob);
 
     // Result gets written in there
     _currentDiscoveryDirectoryResult = r;
@@ -646,8 +664,17 @@ csync_vio_handle_t *DiscoveryJob::remote_vio_opendir_hook(const char *url,
         if (directoryResult->code != 0) {
             qCDebug(lcDiscovery) << directoryResult->code << "when opening" << url << "msg=" << directoryResult->msg;
             errno = directoryResult->code;
+
             // save the error string to the context
-            discoveryJob->_csync_ctx->error_string = qstrdup(directoryResult->msg.toUtf8().constData());
+            QString errorString = directoryResult->path;
+            if (errorString.startsWith("/"))
+                errorString = errorString.mid(1);
+            if (!directoryResult->msg.isEmpty()) {
+                errorString.append(": ");
+                errorString.append(directoryResult->msg);
+            }
+            discoveryJob->_csync_ctx->error_string = errorString;
+
             return NULL;
         }
 
@@ -698,10 +725,6 @@ void DiscoveryJob::start()
     _csync_ctx->callbacks.remote_closedir_hook = remote_vio_closedir_hook;
     _csync_ctx->callbacks.vio_userdata = this;
 
-    csync_exclude_traversal_prepare(_csync_ctx); // Converts the flat exclude list to optimized regexps
-
-    csync_set_log_callback(_log_callback);
-    csync_set_log_level(_log_level);
     _lastUpdateProgressCallbackCall.invalidate();
     int ret = csync_update(_csync_ctx);
 
