@@ -22,7 +22,7 @@
 #include "propagateremotemove.h"
 #include "propagateremotemkdir.h"
 #include "propagatorjobs.h"
-#include "configfile.h"
+#include "filesystem.h"
 #include "common/utility.h"
 #include "account.h"
 #include "common/asserts.h"
@@ -141,6 +141,7 @@ static SyncJournalErrorBlacklistRecord createBlacklistEntry(
     entry._lastTryTime = Utility::qDateTimeToTime_t(QDateTime::currentDateTimeUtc());
     entry._renameTarget = item._renameTarget;
     entry._retryCount = old._retryCount + 1;
+    entry._requestId = item._requestId;
 
     static qint64 minBlacklistTime(getMinBlacklistTime());
     static qint64 maxBlacklistTime(qMax(getMaxBlacklistTime(), minBlacklistTime));
@@ -286,60 +287,6 @@ void PropagateItemJob::done(SyncFileItem::Status statusArg, const QString &error
     }
 }
 
-/**
- * For delete or remove, check that we are not removing from a shared directory.
- * If we are, try to restore the file
- *
- * Return true if the problem is handled.
- */
-bool PropagateItemJob::checkForProblemsWithShared(int httpStatusCode, const QString &msg)
-{
-    PropagateItemJob *newJob = NULL;
-
-    if (httpStatusCode == 403 && propagator()->isInSharedDirectory(_item->_file)) {
-        if (!_item->isDirectory()) {
-            SyncFileItemPtr downloadItem(new SyncFileItem(*_item));
-            if (downloadItem->_instruction == CSYNC_INSTRUCTION_NEW
-                || downloadItem->_instruction == CSYNC_INSTRUCTION_TYPE_CHANGE) {
-                // don't try to recover pushing new files
-                return false;
-            } else if (downloadItem->_instruction == CSYNC_INSTRUCTION_SYNC) {
-                // we modified the file locally, just create a conflict then
-                downloadItem->_instruction = CSYNC_INSTRUCTION_CONFLICT;
-
-                // HACK to avoid continuation: See task #1448:  We do not know the _modtime from the
-                //  server, at this point, so just set the current one. (rather than the one locally)
-                downloadItem->_modtime = Utility::qDateTimeToTime_t(QDateTime::currentDateTimeUtc());
-            } else {
-                // the file was removed or renamed, just recover the old one
-                downloadItem->_instruction = CSYNC_INSTRUCTION_SYNC;
-            }
-            downloadItem->_direction = SyncFileItem::Down;
-            newJob = new PropagateDownloadFile(propagator(), downloadItem);
-        } else {
-            // Directories are harder to recover.
-            // But just re-create the directory, next sync will be able to recover the files
-            SyncFileItemPtr mkdirItem(new SyncFileItem(*_item));
-            mkdirItem->_instruction = CSYNC_INSTRUCTION_NEW;
-            mkdirItem->_direction = SyncFileItem::Down;
-            newJob = new PropagateLocalMkdir(propagator(), mkdirItem);
-            // Also remove the inodes and fileid from the db so no further renames are tried for
-            // this item.
-            propagator()->_journal->avoidRenamesOnNextSync(_item->_file);
-            propagator()->_anotherSyncNeeded = true;
-        }
-        if (newJob) {
-            newJob->setRestoreJobMsg(msg);
-            _restoreJob.reset(newJob);
-            connect(_restoreJob.data(), &PropagatorJob::finished,
-                this, &PropagateItemJob::slotRestoreJobFinished);
-            QMetaObject::invokeMethod(newJob, "start");
-        }
-        return true;
-    }
-    return false;
-}
-
 void PropagateItemJob::slotRestoreJobFinished(SyncFileItem::Status status)
 {
     QString msg;
@@ -369,8 +316,10 @@ PropagateItemJob *OwncloudPropagator::createJob(const SyncFileItemPtr &item)
             return new PropagateRemoteDelete(this, item);
     case CSYNC_INSTRUCTION_NEW:
     case CSYNC_INSTRUCTION_TYPE_CHANGE:
+    case CSYNC_INSTRUCTION_CONFLICT:
         if (item->isDirectory()) {
-            if (item->_direction == SyncFileItem::Down) {
+            // CONFLICT has _direction == None
+            if (item->_direction != SyncFileItem::Up) {
                 auto job = new PropagateLocalMkdir(this, item);
                 job->setDeleteExistingFile(deleteExisting);
                 return job;
@@ -381,7 +330,6 @@ PropagateItemJob *OwncloudPropagator::createJob(const SyncFileItemPtr &item)
             }
         } //fall through
     case CSYNC_INSTRUCTION_SYNC:
-    case CSYNC_INSTRUCTION_CONFLICT:
         if (item->_direction != SyncFileItem::Up) {
             auto job = new PropagateDownloadFile(this, item);
             job->setDeleteExistingFolder(deleteExisting);
@@ -432,6 +380,7 @@ void OwncloudPropagator::start(const SyncFileItemVector &items)
     directories.push(qMakePair(QString(), _rootJob.data()));
     QVector<PropagatorJob *> directoriesToRemove;
     QString removedDirectory;
+    QString maybeConflictDirectory;
     foreach (const SyncFileItemPtr &item, items) {
         if (!removedDirectory.isEmpty() && item->_file.startsWith(removedDirectory)) {
             // this is an item in a directory which is going to be removed.
@@ -465,6 +414,20 @@ void OwncloudPropagator::start(const SyncFileItemVector &items)
             }
         }
 
+        // If a CONFLICT item contains files these can't be processed because
+        // the conflict handling is likely to rename the directory. This can happen
+        // when there's a new local directory at the same time as a remote file.
+        if (!maybeConflictDirectory.isEmpty()) {
+            if (item->destination().startsWith(maybeConflictDirectory)) {
+                qCInfo(lcPropagator) << "Skipping job inside CONFLICT directory"
+                                     << item->_file << item->_instruction;
+                item->_instruction = CSYNC_INSTRUCTION_NONE;
+                continue;
+            } else {
+                maybeConflictDirectory.clear();
+            }
+        }
+
         while (!item->destination().startsWith(directories.top().first)) {
             directories.pop();
         }
@@ -514,6 +477,12 @@ void OwncloudPropagator::start(const SyncFileItemVector &items)
             } else {
                 directories.top().second->appendTask(item);
             }
+
+            if (item->_instruction == CSYNC_INSTRUCTION_CONFLICT) {
+                // This might be a file or a directory on the local side. If it's a
+                // directory we want to skip processing items inside it.
+                maybeConflictDirectory = item->_file + "/";
+            }
         }
     }
 
@@ -537,36 +506,6 @@ void OwncloudPropagator::setSyncOptions(const SyncOptions &syncOptions)
     _chunkSize = syncOptions._initialChunkSize;
 }
 
-// ownCloud server  < 7.0 did not had permissions so we need some other euristics
-// to detect wrong doing in a Shared directory
-bool OwncloudPropagator::isInSharedDirectory(const QString &file)
-{
-    bool re = false;
-    if (_remoteFolder.startsWith(QLatin1String("Shared"))) {
-        // The Shared directory is synced as its own sync connection
-        re = true;
-    } else {
-        if (file.startsWith("Shared/") || file == "Shared") {
-            // The whole ownCloud is synced and Shared is always a top dir
-            re = true;
-        }
-    }
-    return re;
-}
-
-int OwncloudPropagator::httpTimeout()
-{
-    static int timeout = 0;
-    if (!timeout) {
-        timeout = qgetenv("OWNCLOUD_TIMEOUT").toUInt();
-        if (timeout == 0) {
-            ConfigFile cfg;
-            timeout = cfg.timeout();
-        }
-    }
-    return timeout;
-}
-
 bool OwncloudPropagator::localFileNameClash(const QString &relFile)
 {
     bool re = false;
@@ -579,8 +518,7 @@ bool OwncloudPropagator::localFileNameClash(const QString &relFile)
             re = false;
             qCWarning(lcPropagator) << "No valid fileinfo";
         } else {
-            // Need to normalize to composited form because of
-            // https://bugreports.qt-project.org/browse/QTBUG-39622
+            // Need to normalize to composited form because of QTBUG-39622/QTBUG-55896
             const QString cName = fileInfo.canonicalFilePath().normalized(QString::NormalizationForm_C);
             bool equal = (file == cName);
             re = (!equal && !cName.endsWith(relFile, Qt::CaseSensitive));
@@ -652,6 +590,11 @@ QString OwncloudPropagator::getFilePath(const QString &tmp_file_name) const
     return _localDir + tmp_file_name;
 }
 
+QString OwncloudPropagator::addVirtualFileSuffix(const QString &fileName) const
+{
+    return fileName + _syncOptions._virtualFileSuffix;
+}
+
 void OwncloudPropagator::scheduleNextJob()
 {
     QTimer::singleShot(0, this, &OwncloudPropagator::scheduleNextJobImpl);
@@ -716,6 +659,76 @@ OwncloudPropagator::DiskSpaceResult OwncloudPropagator::diskSpaceCheck() const
     return DiskSpaceOk;
 }
 
+bool OwncloudPropagator::createConflict(const SyncFileItemPtr &item,
+    PropagatorCompositeJob *composite, QString *error)
+{
+    QString fn = getFilePath(item->_file);
+
+    QString renameError;
+    auto conflictModTime = FileSystem::getModTime(fn);
+    QString conflictUserName;
+    if (account()->capabilities().uploadConflictFiles())
+        conflictUserName = account()->davDisplayName();
+    QString conflictFileName = Utility::makeConflictFileName(
+        item->_file, Utility::qDateTimeFromTime_t(conflictModTime), conflictUserName);
+    QString conflictFilePath = getFilePath(conflictFileName);
+
+    emit touchedFile(fn);
+    emit touchedFile(conflictFilePath);
+
+    if (!FileSystem::rename(fn, conflictFilePath, &renameError)) {
+        // If the rename fails, don't replace it.
+
+        // If the file is locked, we want to retry this sync when it
+        // becomes available again.
+        if (FileSystem::isFileLocked(fn)) {
+            emit seenLockedFile(fn);
+        }
+
+        if (error)
+            *error = renameError;
+        return false;
+    }
+    qCInfo(lcPropagator) << "Created conflict file" << fn << "->" << conflictFileName;
+
+    // Create a new conflict record. To get the base etag, we need to read it from the db.
+    ConflictRecord conflictRecord;
+    conflictRecord.path = conflictFileName.toUtf8();
+    conflictRecord.baseModtime = item->_previousModtime;
+    conflictRecord.initialBasePath = item->_file.toUtf8();
+
+    SyncJournalFileRecord baseRecord;
+    if (_journal->getFileRecord(item->_originalFile, &baseRecord) && baseRecord.isValid()) {
+        conflictRecord.baseEtag = baseRecord._etag;
+        conflictRecord.baseFileId = baseRecord._fileId;
+    } else {
+        // We might very well end up with no fileid/etag for new/new conflicts
+    }
+
+    _journal->setConflictRecord(conflictRecord);
+
+    // Create a new upload job if the new conflict file should be uploaded
+    if (account()->capabilities().uploadConflictFiles()) {
+        if (composite && !QFileInfo(conflictFilePath).isDir()) {
+            SyncFileItemPtr conflictItem = SyncFileItemPtr(new SyncFileItem);
+            conflictItem->_file = conflictFileName;
+            conflictItem->_type = ItemTypeFile;
+            conflictItem->_direction = SyncFileItem::Up;
+            conflictItem->_instruction = CSYNC_INSTRUCTION_NEW;
+            conflictItem->_modtime = conflictModTime;
+            conflictItem->_size = item->_previousSize;
+            emit newItem(conflictItem);
+            composite->appendTask(conflictItem);
+        } else {
+            // Directories we can't process in one go. The next sync run
+            // will take care of uploading the conflict dir contents.
+            _anotherSyncNeeded = true;
+        }
+    }
+
+    return true;
+}
+
 // ================================================================================
 
 PropagatorJob::PropagatorJob(OwncloudPropagator *propagator)
@@ -753,6 +766,12 @@ void PropagatorCompositeJob::slotSubJobAbortFinished()
     }
 }
 
+void PropagatorCompositeJob::appendJob(PropagatorJob *job)
+{
+    job->setAssociatedComposite(this);
+    _jobsToDo.append(job);
+}
+
 bool PropagatorCompositeJob::scheduleSelfOrChild()
 {
     if (_state == Finished) {
@@ -781,13 +800,8 @@ bool PropagatorCompositeJob::scheduleSelfOrChild()
     }
 
     // Now it's our turn, check if we have something left to do.
-    if (!_jobsToDo.isEmpty()) {
-        PropagatorJob *nextJob = _jobsToDo.first();
-        _jobsToDo.remove(0);
-        _runningJobs.append(nextJob);
-        return possiblyRunNextJob(nextJob);
-    }
-    while (!_tasksToDo.isEmpty()) {
+    // First, convert a task to a job if necessary
+    while (_jobsToDo.isEmpty() && !_tasksToDo.isEmpty()) {
         SyncFileItemPtr nextTask = _tasksToDo.first();
         _tasksToDo.remove(0);
         PropagatorJob *job = propagator()->createJob(nextTask);
@@ -795,9 +809,15 @@ bool PropagatorCompositeJob::scheduleSelfOrChild()
             qCWarning(lcDirectory) << "Useless task found for file" << nextTask->destination() << "instruction" << nextTask->_instruction;
             continue;
         }
-
-        _runningJobs.append(job);
-        return possiblyRunNextJob(job);
+        appendJob(job);
+        break;
+    }
+    // Then run the next job
+    if (!_jobsToDo.isEmpty()) {
+        PropagatorJob *nextJob = _jobsToDo.first();
+        _jobsToDo.remove(0);
+        _runningJobs.append(nextJob);
+        return possiblyRunNextJob(nextJob);
     }
 
     // If neither us or our children had stuff left to do we could hang. Make sure
@@ -821,10 +841,13 @@ void PropagatorCompositeJob::slotSubJobFinished(SyncFileItem::Status status)
     ASSERT(i >= 0);
     _runningJobs.remove(i);
 
+    // Any sub job error will cause the whole composite to fail. This is important
+    // for knowing whether to update the etag in PropagateDirectory, for example.
     if (status == SyncFileItem::FatalError
         || status == SyncFileItem::NormalError
         || status == SyncFileItem::SoftError
-        || status == SyncFileItem::DetailError) {
+        || status == SyncFileItem::DetailError
+        || status == SyncFileItem::BlacklistedError) {
         _hasError = status;
     }
 
@@ -865,6 +888,7 @@ PropagateDirectory::PropagateDirectory(OwncloudPropagator *propagator, const Syn
 {
     if (_firstJob) {
         connect(_firstJob.data(), &PropagatorJob::finished, this, &PropagateDirectory::slotFirstJobFinished);
+        _firstJob->setAssociatedComposite(&_subJobs);
     }
     connect(&_subJobs, &PropagatorJob::finished, this, &PropagateDirectory::slotSubJobsFinished);
 }
@@ -908,7 +932,9 @@ void PropagateDirectory::slotFirstJobFinished(SyncFileItem::Status status)
 {
     _firstJob.take()->deleteLater();
 
-    if (status != SyncFileItem::Success && status != SyncFileItem::Restoration) {
+    if (status != SyncFileItem::Success
+        && status != SyncFileItem::Restoration
+        && status != SyncFileItem::Conflict) {
         if (_state != Finished) {
             // Synchronously abort
             abort(AbortType::Synchronous);
