@@ -28,18 +28,6 @@ class ownCloudTheme : public Theme
     Q_OBJECT
 public:
     ownCloudTheme();
-
-    QString configFileName() const Q_DECL_OVERRIDE;
-    QString about() const Q_DECL_OVERRIDE;
-
-#ifndef TOKEN_AUTH_ONLY
-    QIcon trayFolderIcon(const QString &) const Q_DECL_OVERRIDE;
-    QIcon applicationIcon() const Q_DECL_OVERRIDE;
-#endif
-    QString appName() const Q_DECL_OVERRIDE;
-    QString appNameGUI() const Q_DECL_OVERRIDE;
-
-    QString helpUrl() const Q_DECL_OVERRIDE;
 #ifndef TOKEN_AUTH_ONLY
     QVariant customMedia(CustomMediaType type) Q_DECL_OVERRIDE;
 
@@ -48,6 +36,9 @@ public:
     QPixmap wizardHeaderLogo() const Q_DECL_OVERRIDE;
 #endif
 
+    // For owncloud-brandings *do* show the virtual files option.
+    bool showVirtualFilesOption() const override { return true; }
+
 private:
 };
 }
