@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -13,18 +14,17 @@
 
 #include "syncfilestatus.h"
 
-#include <QDebug>
-
 namespace OCC {
 SyncFileStatus::SyncFileStatus()
-    :_tag(StatusNone), _sharedWithMe(false)
+    : _tag(StatusNone)
+    , _shared(false)
 {
 }
 
 SyncFileStatus::SyncFileStatus(SyncFileStatusTag tag)
-    :_tag(tag), _sharedWithMe(false)
+    : _tag(tag)
+    , _shared(false)
 {
-
 }
 
 void SyncFileStatus::set(SyncFileStatusTag tag)
@@ -32,19 +32,19 @@ void SyncFileStatus::set(SyncFileStatusTag tag)
     _tag = tag;
 }
 
-SyncFileStatus::SyncFileStatusTag SyncFileStatus::tag()
+SyncFileStatus::SyncFileStatusTag SyncFileStatus::tag() const
 {
     return _tag;
 }
 
-void SyncFileStatus::setSharedWithMe(bool isShared)
+void SyncFileStatus::setShared(bool isShared)
 {
-    _sharedWithMe = isShared;
+    _shared = isShared;
 }
 
-bool SyncFileStatus::sharedWithMe()
+bool SyncFileStatus::shared() const
 {
-    return _sharedWithMe;
+    return _shared;
 }
 
 QString SyncFileStatus::toSocketAPIString() const
@@ -52,8 +52,7 @@ QString SyncFileStatus::toSocketAPIString() const
     QString statusString;
     bool canBeShared = true;
 
-    switch(_tag)
-    {
+    switch (_tag) {
     case StatusNone:
         statusString = QLatin1String("NOP");
         canBeShared = false;
@@ -72,7 +71,7 @@ QString SyncFileStatus::toSocketAPIString() const
         statusString = QLatin1String("ERROR");
         break;
     }
-    if(canBeShared && _sharedWithMe) {
+    if (canBeShared && _shared) {
         statusString += QLatin1String("+SWM");
     }
 
