@@ -134,7 +134,8 @@ static int teardown(void **state) {
     rc = wipe_testdir();
     assert_int_equal(rc, 0);
 
-    *state = NULL;
+    SAFE_FREE(((statevar*)*state)->result);
+    SAFE_FREE(*state);
     return 0;
 }
 
@@ -218,7 +219,7 @@ static void traverse_dir(void **state, const char *dir, int *cnt)
           continue;
         }
 
-        is_dir = (dirent->type == CSYNC_FTW_TYPE_DIR) ? 1:0;
+        is_dir = (dirent->type == ItemTypeDirectory) ? 1:0;
 
         assert_int_not_equal( asprintf( &subdir, "%s/%s", dir, dirent->path.constData() ), -1 );
 
@@ -419,6 +420,8 @@ static void check_readdir_longtree(void **state)
     assert_int_equal(files_cnt, 0);
     /* and compare. */
     assert_string_equal( sv->result, result);
+
+    SAFE_FREE(result);
 }
 
 // https://github.com/owncloud/client/issues/3128 https://github.com/owncloud/client/issues/2777
